
/*-
 * fwl_subr.c
 *
 *	@(#)fwl_subr.c	1.5 (UCLA Ficus Project) 5/8/95
 *
 * Copyright (c) 1994 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifdef I_FWL

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/user.h>
#include <sys/file.h>
#include <sys/vfs.h>
#include <sys/vnode.h>
#include <i405/i405.h>
#include <sys/pathname.h>
#include <sys/uio.h>




/*-
 *--------------------------------------------------------------------------
 *
 * fwl_mount_fwl -- configure a single new fwl on the stack
 *
 * RETURN VALUES:
 *      Returns an errno value.
 *
 *--------------------------------------------------------------------------
 */
int
fwl_mount_fwl(fwl_name, opv_dest_list, opv_dest_list_len)
	char *fwl_name;
	PFI **opv_dest_list;
	int opv_dest_list_len;
{
	struct fwl_mount_desc *fwlmp;
	struct fwl_vnodeopv_entry_desc *entry_desc;
	int i;

	/*
	 * Find it in the list.
	 */
	for (fwlmp = &fwl_mount_descs[0]; fwlmp->fwlm_name; fwlmp++) {
		if (strcmp(fwlmp->fwlm_name, fwl_name) == 0)
			break;
	};
	if (fwlmp->fwlm_name == NULL) {
		/* This printf is for debugging only. */
		printf ("fwl_mount_fwl: layer %s not found in fwl_mount_descs.\n", fwl_name);
		return ENODEV;   /* just like mount(2) */
	};

	/*
	 * Now go change the ops.
	 */
	for(entry_desc = fwlmp->fwlm_vopv_entries; entry_desc->opve_op; entry_desc++) {
		for (i = 0; i < opv_dest_list_len; i++) {
			/*
			 * To double or not to double,
			 * that is the question.
			 */
			if (entry_desc->opve_flags & FWL_OP_DOUBLES) {
				/* Double, double, boil and trouble... */
				if (entry_desc->opve_op->vdesc_offset == 0) {
					char tmps[256];
					sprintf(tmps, "fwl_mount_fwl: doubled op %s not in i405_op_descs.", entry_desc->opve_op->vdesc_name);
					panic(tmps);
				};

				/*
				 * It's a bug to list the same doubled
				 * op twice in; when you try to call it you'll 
				 * end up with an infinite loop.  Check
				 * for this bug here.
				 *   (As a special case ignore the default
				 * op.)
				 */
				if (opv_dest_list[i][entry_desc->opve_op->vdesc_offset] == opv_dest_list[i][entry_desc->opve_doubled_op->vdesc_offset] &&
					opv_dest_list[i][entry_desc->opve_op->vdesc_offset] != opv_dest_list[i][VOFFSET(nvop_default)])
					panic("fwl_mount_fwl: operation doubles itslef");
				opv_dest_list[i][entry_desc->opve_op->vdesc_offset] = opv_dest_list[i][entry_desc->opve_doubled_op->vdesc_offset];
				continue;
			};
			if (entry_desc->opve_doubled_op)
				panic("fwl_mount_fwl: saved op but FWL_OP_DOUBLES not set.");
			/*
			 * Not doubling,
			 * just trade the new implementation for old.
			 */
			opv_dest_list[i][entry_desc->opve_op->vdesc_offset] =
				entry_desc->opve_impl;
		};
	};
	return 0;
}


/*-
 *--------------------------------------------------------------------------
 *
 * fwl_mount_fwls -- configure new fwls on the stack
 *
 * SEE ALSO:
 *	fwl_rele_fwls
 *
 * RETURN VALUES:
 *      Returns an errno value.
 *
 *--------------------------------------------------------------------------
 */
int
fwl_mount_fwls(fwl_list, opv_desc_src_list, opv_dest_list)
	char **fwl_list;
	struct vnodeopv_desc **opv_desc_src_list;
	PFI **opv_dest_list;
{
	int error;
	int i;
	int opv_dest_list_len;   /* number of opvs in the mount-on fs */

	/*
	 * Allocate space for the ops vectors
	 * and fill it in with the old list.
	 */
	for (i = 0; opv_desc_src_list[i]; i++) {
		opv_dest_list[i] = (PFI*)
			kmem_alloc(num_operations * sizeof(PFI));
		bcopy(*opv_desc_src_list[i]->opv_desc_vector_p,
		      opv_dest_list[i],
		      num_operations * sizeof(PFI));
	};
	opv_dest_list_len = i;

	/*
	 * Mount each layer seperately.
	 */
	for (i = 0; fwl_list[i]; i++) {
		if (error = fwl_mount_fwl(fwl_list[i], opv_dest_list, opv_dest_list_len))
			return error;
	};

	return 0;
}



/*-
 *--------------------------------------------------------------------------
 *
 * fwl_rele_fwls -- release fwl ops vectors
 *
 * SEE ALSO:
 *	fwl_mount_fwls
 *
 * RETURN VALUES:
 *      None.
 *
 *--------------------------------------------------------------------------
 */
void
fwl_rele_fwls(opv_desc_src_list, opv_dest_list)
	struct vnodeopv_desc **opv_desc_src_list;
	PFI **opv_dest_list;
{
	int error;
	int i;

	/*
	 * Allocate space for the ops vectors
	 * and fill it in with the old list.
	 */
	for (i = 0; opv_desc_src_list[i]; i++) {
		if (opv_dest_list[i])
			kmem_free(opv_dest_list[i], num_operations * sizeof(PFI));
		opv_dest_list[i] = NULL;
	};
}


#endif /* I_FWL */
