
/*-
 * ro_vnodeops.c
 *
 *	@(#)ro_vnodeops.c	1.2 (UCLA Ficus Project) 2/15/95
 *
 * Copyright (c) 1995 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/param.h>
#include <sys/time.h>
#include <sys/vnode.h>
#include <i405/i405.h>
#include <sys/debug.h>   /* for ASSERT */
#include <sys/errno.h>
#include <i405/FWL/ro_vnodeops.h>


static int
ro_readonly_op(ap)
	struct nvop_generic_args *ap;
{
	return EROFS;
}


static int
ro_rdwr (ap)
	struct nvop_rdwr_args *ap;
{
	/* throw out writes */
	if (ap->a_rw == UIO_WRITE)
		return EROFS;
	/* pass through reads */
	return RO_DOUBLED_NVOP_RDWR_AP(ap);
}

#if 0
/*
 * ro_putpage:  This one is a problem.
 * We're supposed to scan the list of pages and pvn_fail over any
 * dirty pages.  Clean pages we can just "pass down".
 * (This is how UFS does it.  NFS apparently doesn't do it.)
 * Unfortuantely, scanning the pages is a lot of work.
 *
 * [Side comment:  can we write to a RO-mounted NFS FS by 
 * mmaping the file?]
 *
 * Actually, mmap'ing a file of a RO file system as writable
 * is prohibited by the upper-level kernel.
 * 
 */
static int
ro_putpage (ap)
	struct nvop_putpage_args *ap;
{
}
#endif

struct fwl_vnodeopv_entry_desc ro_fwl_vnodeop_entries[] = {
	{ &ro_doubled_nvop_rdwr_desc, NULL, FWL_OP_DOUBLES, &nvop_rdwr_desc },
	{ &nvop_rdwr_desc, ro_rdwr, 0, NULL },
#if 0
	{ &ro_doubled_nvop_putpage_desc, NULL, FWL_OP_DOUBLES, &nvop_putpage_desc },
	{ &nvop_putpage_desc, ro_putpage, 0, NULL },
#endif
	{ &nvop_setattr_desc, ro_readonly_op, 0, NULL },
	{ &nvop_create_desc, ro_readonly_op, 0, NULL },
	{ &nvop_remove_desc, ro_readonly_op, 0, NULL },
	{ &nvop_link_desc, ro_readonly_op, 0, NULL },
	{ &nvop_rename_desc, ro_readonly_op, 0, NULL },
	{ &nvop_mkdir_desc, ro_readonly_op, 0, NULL },
	{ &nvop_rmdir_desc, ro_readonly_op, 0, NULL },
	{ &nvop_symlink_desc, ro_readonly_op, 0, NULL },
	{ NULL, NULL, 0, NULL },
};

