
/*-
 * specref_vnodeops.c
 *
 *	@(#)specref_vnodeops.c	1.1 (UCLA Ficus Project) 2/15/95
 *
 * Copyright (c) 1995 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifdef I_FWL

#include <sys/param.h>
#include <sys/time.h>
#include <sys/vnode.h>
#include <i405/i405.h>
#include <sys/errno.h>
#include <sys/debug.h>   /* for ASSERT */
#include <specfs/snode.h>   /* for specvp() */
#include <i405/FWL/specref_vnodeops.h>


#define	ISVDEV(t) ((t == VCHR) || (t == VBLK) || (t == VFIFO))


static int
specref_isalreadyspec(vp, cred)
	struct vnode *vp;
	struct ucred *cred;
{
	/*
	 * There are two ways to do this:
	 * If specfs is converted then we do it the "right"
	 * way with the new op get_fsindex.
	 * If specfs is not converted, we just look
	 * at the ops vector.
	 */
#ifdef SPECFS_CONVERTED
	USES_NVOP_GET_FSINDEX;
	int error;
	int fsindex;
	
	if (error = NVOP_GET_FSINDEX(vp, &fsindex, cred))
		panic("specref_isalreadyspec");
	return (fsindex == specfs_fsindex);
#else /* ! SPECFS_CONVERTED */
	extern struct vnodeops spec_vnodeops;
	return (vp->v_op == &spec_vnodeops);
#endif /* SPECFS_CONVERTED */
}

static int
specref_swap_specvp(vpp)
	struct vnode **vpp;
{
	struct vnode *vp = *vpp;
	struct vnode *newvp = specvp(vp, vp->v_rdev, vp->v_type);
	VN_RELE(vp);
	*vpp = newvp;
	return 0;
}

static int
specref_lookup(ap)
	struct nvop_lookup_args *ap;
{
	int error;
	struct vnode *vp;

	/*
	 * Call the old op.
	 */
	error = SPECREF_DOUBLED_NVOP_LOOKUP_AP(ap);

	/*
	 * Swap realvp for specvp if it's deserving.
	 */
	vp = *ap->a_vpp;
	if (!error && !specref_isalreadyspec(vp) && ISVDEV(vp->v_type))
		error = specref_swap_specvp(ap->a_vpp);

	return error;
}


static int
specref_create(ap)
	struct nvop_create_args *ap;
{
	int error;
	struct vnode *vp;

	/*
	 * Call the old op.
	 */
	error = SPECREF_DOUBLED_NVOP_CREATE_AP(ap);

	/*
	 * Swap realvp for specvp if it's deserving.
	 */
	vp = *ap->a_vpp;
	if (!error && !specref_isalreadyspec(vp) && ISVDEV(vp->v_type))
		error = specref_swap_specvp(ap->a_vpp);

	return error;
}


struct fwl_vnodeopv_entry_desc specref_fwl_vnodeop_entries[] = {
	{ &specref_doubled_nvop_lookup_desc, NULL, FWL_OP_DOUBLES, &nvop_lookup_desc },
	{ &nvop_lookup_desc, specref_lookup, 0, NULL },
	{ &specref_doubled_nvop_create_desc, NULL, FWL_OP_DOUBLES, &nvop_create_desc },
	{ &nvop_create_desc, specref_create, 0, NULL },
	{ NULL, NULL, 0, NULL },
};

#endif /* I_FWL */
