
/*-
 * vmio_vnodeops.c
 *
 *	@(#)vmio_vnodeops.c	1.6 (UCLA Ficus Project) 6/5/95
 *
 * Copyright (c) 1994,1995 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifdef I_FWL

#include <sys/param.h>
#include <sys/time.h>
#include <sys/vnode.h>
#include <i405/i405.h>
#include <sys/errno.h>
#include <sys/debug.h>   /* for ASSERT */


/*-
 *--------------------------------------------------------------------------
 *
 * vmio_rdwr -- rdwr through the page cache
 *
 * DESCRIPTION:
 *	Rdwr must be intercepted because we must force it through
 *	_our_ page cache, otherwise we access our lower-level's page
 *	cache and end up aliasing our own pages.
 *
 *	We just get the svcm locks and call the lower-layer's stackrdwr
 *	routine.
 *
 *
 * RETURN VALUES:
 *      Returns an errno value.
 *
 *--------------------------------------------------------------------------
 */
static int
vmio_rdwr (ap)
	struct nvop_rdwr_args *ap;
{
	USES_NVOP_STACKRDWR;
	USES_NVOP_GET_SVCM_NAME;
	int error;
	struct vnode *vp = ap->a_vp;
	struct svcm_name *svcm_name;

	if (vp->v_type != VREG) {
		/* NEEDSWORK: should bypass this operation */
#if 1
		return EIO;
#else
		return null_bypass(ap);
#endif
	};

	if (error = NVOP_GET_SVCM_NAME(vp, &svcm_name, ap->a_cred))
		return error;

	/*
	 * Don't do special file types (like directories)
	 * since they're not cache coherent.
	 */
	if (error = svcm_lock(svcm_name->lock))
		return error;

	error = NVOP_STACKRDWR(vp,
			       svcm_name, vp,
			       ap->a_uiop, ap->a_rw,
			       ap->a_ioflag, ap->a_cred);

	(void)svcm_unlock(svcm_name->lock);

	return error;
}


struct fwl_vnodeopv_entry_desc vmio_fwl_vnodeop_entries[] = {
	{ &nvop_rdwr_desc, vmio_rdwr, 0, NULL },
	{ NULL, NULL, 0, NULL },
};

#endif /* I_FWL */
