
/*
 * utok_vnodeops.c
 *
 *	@(#)utok_vnodeops.c	2.20 (UCLA Ficus Project) 10/13/94
 *
 * Copyright (c) 1992 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 *
 * A package to emulate vnodes out of kernel.
 *
 * Intercepts returned vnodes from (lookup, mkdir, create, etc.),
 * and wraps a user-level structure around in-kernel vnodes
 * (so VN_HOLD and VN_RELE work like they should).
 *
 * This has one known bug:  all utoknodes are listed as type VNON.
 * Of course, according to Rosenthal, you shouldn't check that
 * field anyway.
 *
 * Another caveat is that this file has all the usual layer files
 * (xfs_vfsops, xfs_subr, etc) thrown into it.
 *
 *
 * Please be aware that this file presents less than ideal code to
 * examine as a sample layer.  The code here is quite old and
 * crufty and has not been maintained nearly as well as other layers.
 * The null layer is a recommended alternative.
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/user.h>
#include <sys/vfs.h>
#include <sys/vfs_stat.h>
#include <sys/vnode.h>
#define STANDALONE_OOK   /* We expect to call directly into the kernel */
#include <i405/i405.h>
#include <i405/i405_syscalls.h>
#include <i405/ook_compat.h>

#ifdef NOTDEF
/*
 * This is now defined in xdr.h, dragged in from i405.h.
 */
#ifdef KERNEL
#define mem_alloc(S) kmem_alloc(S)
#define mem_free(P,S) kmem_free(P,S)
#else /* KERNEL */
#define mem_alloc(S) malloc(S)
#define mem_free(P,S) free(P)
#endif /* KERNEL */
#endif /* NOTDEF */

#ifdef NOTDEF
#define DODEBUG(A) A
#else
#define DODEBUG(A)
#endif

#include <i405/LIB/LAYER/utok_vnodeops.h>
#include <i405/LIB/PROG/user_vnodeop.h>
#include <i405/LIB/PROG/ook_lookupname.h>


#define UTOKNTOVP(UTOKN) (&(UTOKN)->vnode)
#define VPTOUTOKN(VP) ((struct utok_node *)((VP)->v_data))
#define UTOKNTOLOWER(UTOKN) ((struct vnode*)((UTOKN)->lowervp))


struct utok_node {
	struct vnode *lowervp;   /* lower level vnode */
	struct vnode vnode;   /* our vnode */
	struct utok_node *next;
};



struct utok_node *utok_base_node = NULL;


static int utok_bypass(),
	utok_open(),
	utok_inactive(),
	utok_fid(),
	utok_get_proxy(),
	utok_get_actual();

int (**utok_vnodeop_p)();
struct vnodeopv_entry_desc utok_vnodeop_entries[] = {
	{ &nvop_default_desc, utok_bypass },
	{ &nvop_open_desc, utok_open },
	{ &nvop_inactive_desc, utok_inactive },
	{ &nvop_fid_desc, utok_fid },
	{ &nvop_get_proxy_desc, utok_get_proxy },
	{ &nvop_get_actual_desc, utok_get_actual },
	{ NULL, NULL }
};
struct vnodeopv_desc utok_vnodeop_opv_desc =
	{ &utok_vnodeop_p, utok_vnodeop_entries };




/*****************************************************************************
 * What follows should be in utok_vfsops.c
 *****************************************************************************/


static int
utok_error()
{
	panic ("Bad utok vfs operation called.\n");
	/* NOTREACHED */
}

/*
 * Convert to an in-kernel vget.
 * vfs_data is the in-kernel vfs structure.
 */
static int
utok_vget(vfsp, vpp, fidp)
	struct vfs *vfsp;
	struct vnode **vpp;
	struct fid *fidp;
{
	extern int errno;

	if (-1 == syscall(SYS_fid_to_vnode,vfsp->vfs_data, fidp, vpp))
		return errno;

	return makeutoknode (*vpp, vpp);   /* wrap a utoknode around things. */
}

static struct vfsops utok_vfsops = {
	utok_error,   /* mount */
	utok_error,   /* unmount */
	utok_error,   /* root */
	utok_error,   /* statfs */
	utok_error,   /* sync */
	utok_vget,    /* vget */
	utok_error,   /* mountroot */
	utok_error,   /* swapvp */
};

static struct vfs *
utok_makevfs(lowervp)
	struct vnode *lowervp;
{
	struct vfs *vfsp, *lowervfsp;

        if (-1 == syscall(SYS_vnode_to_vfs, lowervp, &lowervfsp, NULL))
		return NULL;

	vfsp = (struct vfs *)mem_alloc(sizeof(struct vfs));
	if (vfsp == NULL)
		return NULL;

	bzero(vfsp, sizeof(struct vfs));
	vfsp->vfs_op = &utok_vfsops;
	vfsp->vfs_bsize = 8*1024;
	vfsp->vfs_data = (caddr_t)lowervfsp;

	return vfsp;
}



/*****************************************************************************
 * What follows should be in utok_subr.c
 *****************************************************************************/

/*
 *--------------------------------------------------------------------------
 *
 * utok_find -- search cached utok nodes for already existing node
 *
 * DESCRIPTION:
 *	Given a lower layer node, see if we already have a cached
 * 	utok node representing this file.
 *
 *	This routine is responsible for briding the differences between
 *	fd's representing vnodes and normal vnodes.  These differences
 *	include:
 *
 *		- each fd represents a single vnode reference
 *
 *		- different fds can represent the same vnode
 *
 *		- fds can be released only once
 *
 * RETURN VALUES:
 *	Held cached utok node (if found) or NULL.
 *
 *--------------------------------------------------------------------------
 */
static struct utok_node *
utok_find (lowervp)
	struct vnode *lowervp;
{
	struct utok_node *up = utok_base_node;
	int fd;

	/*
	 * Make different references to the same vnode
	 * identical fds.
	 */
	if (-1 == syscall(SYS_normalize_fdvp, (int)lowervp, &fd) ||
	    fd == -1)
		panic ("utok_find: normalize_fdvp failed");

	lowervp = (struct vnode*)fd;
	while (up)
		if (up->lowervp == lowervp) {
			VN_HOLD(UTOKNTOVP(up));
			return up;
		}
		else up = up->next;
	return NULL;
}

static void
utok_free (old_up)
	struct utok_node *old_up;
{
	struct utok_node *up = utok_base_node, *pup=NULL;

	if (up == old_up) {
		utok_base_node = up->next;
	} else while (up)
		if (up == old_up) {
			pup->next = up->next;
			break;
		}
		else {
			pup = up;
			up = up->next;
			if (up == NULL)
				panic ("utok_free: up not found.");
		};
	mem_free (up, sizeof(struct utok_node));
}



int
makeutoknode (lowervp, vpp)
	struct vnode *lowervp, **vpp;
{
	struct utok_node *utokn;
	static struct vfs *utok_vfsp = NULL;

	if (utokn = utok_find(lowervp)) {
		DODEBUG(printf ("utoknode found.\n"));
		*vpp = UTOKNTOVP(utokn);
		/*
                 * Since the null-node corresponding to lvp already exists
                 * we should reduce the v_count of lvp which is bumped
                 * before null_make_nullnode is called.
		 */
		ook_vn_rele(lowervp);
		return 0;
	};

	utokn = (struct utok_node*)mem_alloc(sizeof(struct utok_node));
	
	if (utokn == NULL)
		return ENOMEM;

	/* make a vfs structure, if necessary. */
	if (utok_vfsp == NULL) {
		utok_vfsp = utok_makevfs(lowervp);
	};

/* VN_INIT(VP, VFSP, TYPE, DEV)	*/
	VN_INIT(UTOKNTOVP(utokn), utok_vfsp, VNON, 0);

	utokn->lowervp = lowervp;
	utokn->next = utok_base_node;
	utok_base_node = utokn;

	UTOKNTOVP(utokn)->v_op = (struct vnodeops *) utok_vnodeop_p;
	UTOKNTOVP(utokn)->v_data = (caddr_t)utokn;
	*vpp = UTOKNTOVP(utokn);
	return 0;
}





/*****************************************************************************
 * What follows should be in utok_vnodeops.c
 *****************************************************************************/



/*
 * utok_open does some grossness to take care of the _only_
 * case where the first parameter is a vpp.
 */
static int
utok_open(ap)
	struct nvop_open_args *ap;
{
	int error;
	struct vnode **old_vpp;
	struct vnode *old_lower_vp, *new_lower_vp;
	
	old_vpp = ap->a_vpp;   /* save old value of vpp */

	/*
	 * Open could change the vpp, so we give the lower-level
	 * a new place to put the pointer.
	 */
	new_lower_vp = old_lower_vp = UTOKNTOLOWER(VPTOUTOKN(*ap->a_vpp));
	ap->a_vpp = &new_lower_vp;

	error = VKCALL(*ap->a_vpp, VOFFSET(nvop_open), ap);

	if (!error) {
		/*
		 * If the lower-pointer changed, build a new utoknode
		 * around it.  Otherwise just replace the old
		 * contents of vpp.
		 */
		if (new_lower_vp != old_lower_vp)
			return makeutoknode(new_lower_vp, ap->a_vpp);
	};
	/*
	 * If error or no change in vpp, put it back.
	 */
	ap->a_vpp = old_vpp;
	return error;
}


/*
 * utok_fid must be done separately because the fid defintion is 
 * bad.  It expects one side to have a pre-allcoated buffer,
 * which isn't kosher with the into the kernel XDR code.
 *
 * Until all instances of utok_fid have been changed, we
 * hack around it like this.
 */
static int
utok_fid(ap)
	struct nvop_fid_args *ap;
{
	struct fid *fidp;
	extern int errno;
	int lowervp = (int)UTOKNTOLOWER(VPTOUTOKN(ap->a_vp));
	
	if (NULL == (fidp = (struct fid*) kmem_alloc(sizeof(struct fid))))
		return ENOMEM;
	if (-1 == syscall (SYS_vnode_to_fid, lowervp, fidp)) {
		kmem_free((caddr_t)fidp, sizeof(struct fid));
		return errno;
	};
	/*
	 * fidp->fid_len = MAXFIDSZ;  (so freefid works)
	 * This can't be done, because then when the fid is passed
	 * back into the kernel, it will be different.
	 * Instead, we hack freefid (in the ook version)
	 * to always free a whole fid structure.
	 *
	 * What should happen is that the maximum fid size
	 * should be figured dynamically (like vnodeop vectors)
	 * and placed in a kernel global variable.
	 * - JSH, 8-Feb-91
	 */
	*(ap->a_fidpp) = fidp;
	return 0;
}


static int
utok_inactive(ap)
	struct nvop_inactive_args *ap;
{
	int error;
	struct vnode *old_vp;
	
	old_vp = ap->a_vp;
	ap->a_vp = UTOKNTOLOWER(VPTOUTOKN(ap->a_vp));
	error = VKCALL(ap->a_vp, VOFFSET(nvop_inactive), ap);

	utok_free(VPTOUTOKN(old_vp));

	return error;	
}


int utok_bug_bypass = 0;

static int
utok_bypass(ap)
	struct nvop_generic_args *ap;
{
	int error;
	struct vnode *old_vps[VDESC_MAX_VPS];
	struct vnode **vps_p[VDESC_MAX_VPS];
	struct vnode ***vppp;
	struct vnodeop_desc *descp = ap->a_desc;
	int flags, i;

	if (utok_bug_bypass)
		printf ("utok_bypass: %s\n", descp->vdesc_name);

	if ((descp->vdesc_flags & VDESC_INVPP) != 0)
		panic ("invpp not handled yet.\n");

	/*
	 * Map the vnodes going in.
	 * Later, we'll invoke the operation based on
	 * the mapped, first vnode's operation vector.
	 */
	flags = descp->vdesc_flags;
	for (i = 0; i < VDESC_MAX_VPS; flags >>= 1, i++) {
		if (descp->vdesc_vp_offsets[i] == VDESC_NO_OFFSET)
			break;   /* bail out at end of list */
		if (flags & 1)   /* skip vps that aren't to be mapped */
			continue;
		vps_p[i] = VOPARG_OFFSETTO(struct vnode**,descp->vdesc_vp_offsets[i],ap);
		old_vps[i] = *(vps_p[i]);
		*(vps_p[i]) = UTOKNTOLOWER(VPTOUTOKN(*(vps_p[i])));
	};

	/*
	 * We assume that any vpp's are only OUT values.
	 * If they're INOUT values (as with that pesky vop_open)
	 * we'd need to do a little more work here.
	 */

	/*
	 * We assume that vp1p is the vnode of choice
	 * (with the desired operation vector),
	 * and that it's been correctly mapped above.
	 *
	 * Call the operation on the lower layer
	 * with the modified argument structure.
	 *
	 * Note: because we're explicitly going into the kernel
	 * we use the UUID instead of offset.
	 */
#ifdef INTGEN_TWO
	error = VKCALL(*(vps_p[0]), descp->vdesc_uuid_p, ap);
#else /* INTGEN_TWO */
	error = VKCALL(*(vps_p[0]), UCLA_TRANSPORT(descp)->vdut_uuid_p, ap);
#endif /* INTGEN_TWO */

	/*
	 * Maintain the illusion of call-by-value.
	 */
	flags = descp->vdesc_flags;
	for (i = 0; i < VDESC_MAX_VPS; flags >>= 1, i++) {
		if (descp->vdesc_vp_offsets[i] == VDESC_NO_OFFSET)
			break;   /* bail out at end of list */
		if (flags & 1)   /* skip vps that aren't to be mapped */
			continue;
		*(vps_p[i]) = old_vps[i];
	};

	/*
	 * Map the possible out-going vpp.
	 */
	if (descp->vdesc_vpp_offset != VDESC_NO_OFFSET &&
	    !(descp->vdesc_flags & VDESC_NOMAP_VPP) &&
	    !error) {
		vppp = VOPARG_OFFSETTO(struct vnode***,
				 descp->vdesc_vpp_offset,ap);
		makeutoknode(**vppp, *vppp);
	};

	return error;
}


#ifdef NOTDEF
static void
utok_purge_cache(vp)
	struct vnode *vp;
{
	panic ("use the new purge_cache vnode op.\n");
/*        syscall(SYS_knfs_purge_caches, VPTOUTOKN(vp)->lowervp); */
}
#endif


/* get_proxy is similar to newfid_to_vp
 * except we identify the foreign vfs by vfsp
 *
 * this is needed to identify a lower level vnode to utok when we
 * don't have the mounted fs
 *
 * Frigate uses this
 *
 * -tek, 13 may 94
 */
static int
utok_get_proxy(ap)
	struct nvop_get_proxy_args *ap;
{
	struct vnode *rootvp = NULL;
	int status;
	extern int errno;
	USES_NVOP_NEWFID_TO_VP;

				/* get carrier rootvp */
	if (syscall(SYS_vfs_to_vnode, ap->a_vfsp, &rootvp) < 0) {
	  	if (rootvp) (void)ook_vn_rele(rootvp);
		return errno;
	}
				/* get the vnode we want */
	status = NVOP_NEWFID_TO_VP(rootvp, ap->a_newfidp, ap->a_vpp,
				   ap->a_cred);
	(void)ook_vn_rele(rootvp);
	if (status)
		return status;
	if (!(*ap->a_vpp))	/* NULL vnode returned */
	  	return EINVAL;
				/* wrap utoknode around things */
	return makeutoknode(*ap->a_vpp, ap->a_vpp);
}


/* get_actual is the reverse of get_proxy
 * return the newfid and in-kernel vfsp for a file
 *
 * this is needed to identify to the outside world the actual
 * in-kernel level vnode used by utok
 *
 * Frigate uses this
 *
 * -tek, 5 june 94
 */
static int
utok_get_actual(ap)
	struct nvop_get_actual_args *ap;
{
	struct vnode *lvp;
	extern int errno;
	USES_NVOP_NEWFID;
				/* vnode we are interested in */
	lvp = UTOKNTOLOWER(VPTOUTOKN(ap->a_vp));
				/* get the vfsp */
	if (syscall(SYS_vnode_to_vfs, lvp, (struct vfs *)ap->a_vfsp, NULL) < 0)
		return errno;
				/* get the newfid */
	return NVOP_NEWFID(lvp, ap->a_newfidp, ap->a_cred);
}
