
/*
 * ook_lookupname.c
 *
 *	@(#)ook_lookupname.c	1.15 (UCLA Ficus Project) 7/21/94
 *
 * Copyright (c) 1992 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This provides some compatibility routines for a very simple
 * ook vnode interface.
 *
 * It doesn't require the whole utok layer, or compat.c.
 *
 * This is meant for typical user-level programs like recon or
 * getfattrs.  The utok layer is for fs layers which should run
 * both in and out of the kernel.
 * 
 */

#include <ficus/cdefs.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/uio.h>
#include <sys/errno.h>
#undef FILE
#include <sys/vnode.h>
#include <i405/i405.h>
#include <i405/i405_syscalls.h>
#include "fix_prog_headers.h"


/*
 *--------------------------------------------------------------------------
 *
 * ook_lookupname -- translate a pathname to a user-level vnode
 *
 * DESCRIPTION:
 *	Ook_lookupname translates a pathname to a user-level vnode.
 *	Use of this routine is preferred over ook_vn_open.
 *
 *	The signature of ook_lookupname matches that of the kernel's
 *	function lookupname with the exception of the "seg" entry.
 *
 * SEE ALSO:
 *	ook_vn_rele.
 *
 * RETURN VALUES:
 *      Returns an errno value.
 *
 *--------------------------------------------------------------------------
 */
int
ook_lookupname (name, followlink, dirvpp, compvpp)
	const char *name;
	enum symfollow followlink;		/* (don't) follow sym links */
	struct vnode **dirvpp;			/* ptr for parent vnode (can be NULL) */
	struct vnode **compvpp;			/* ptr for entry vnode */
{
	int error;

	if (error = syscall (SYS_ook_lookupname, name, followlink, dirvpp, compvpp))
		return error;

	/*
	 * Fd of zero is confused with a null vnode pointer.
	 * Hope that the user doesn't close fd 0.
	 */
	if (*(int*)compvpp==0 || (dirvpp && *(int*)dirvpp==0)) {
		printf ("Internal problem in ook_lookupname (zero fd).\n");
		exit (1);
	};

	return 0;
}


/*
 *--------------------------------------------------------------------------
 *
 * ook_vn_rele -- release a user-level vnode allocated with ook_lookupname
 *
 * DESCRIPTION:
 *	Just call close.  Each vnode reference is exactly one
 *	file descriptor.
 *
 * SEE ALSO:
 *	ook_lookupname.
 *
 * RETURN VALUES:
 *      Returns an errno value.
 *
 *--------------------------------------------------------------------------
 */
int
ook_vn_rele(vp)
	struct vnode *vp;
{
	extern int errno;

	if (-1 == close((int)vp))
		return errno;
	else return 0;
}


