
/*
 * ook_vn_open.c
 *
 *	@(#)ook_vn_open.c	1.12 (UCLA Ficus Project) 5/16/94
 *
 * Copyright (c) 1992 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 *
 *
 * It doesn't require the whole utok layer, or compat.c.
 *
 * This is meant for typical user-level programs like recon or
 * getfattrs.  The utok layer is for fs layers which should run
 * both in and out of the kernel.
 * 
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/uio.h>
#include <sys/errno.h>
#undef FILE
#include <sys/vnode.h>
#include <i405/i405.h>
#include <i405/i405_syscalls.h>
#include <sys/fcntl.h>
#include "fix_prog_headers.h"


/*
 *--------------------------------------------------------------------------
 *
 * ook_vn_open -- translate a pathname to a user-level vnode [depricated]
 *
 * DESCRIPTION:
 *	Ook_vn_open translates a pathname to a user-level vnode.
 *	This routine is depricated; new code should use ook_lookupname.
 *
 *	Problems with ook_vn_open are:
 *		- It's signature does not match the kernel function vn_open.
 *		- It builds regular fd's (DTYPE_VNODE) instead of our
 *		  special OOK kind (DTYPE_OURS).
 *		- One cant' open symlinks as themselves.
 *
 *	Ook_vn_open should be paired with ook_vn_close.
 *
 * SEE ALSO:
 *	ook_vn_close, ook_lookupname.
 *
 * RETURN VALUES:
 *      Returns an errno value.
 *
 *--------------------------------------------------------------------------
 */
int
ook_vn_open(pnamep, vpp)
	char *pnamep;
	struct vnode **vpp;
{
	extern int errno;
	int fd;

	/*
	 * Wish to open a file.
	 * Just call the kernel.
	 */
	if (-1==(fd = open(pnamep, O_RDONLY)))
		return errno;

	/*
	 * fd of zero is confused with a null vnode pointer.
	 * Just hope this error doesn't happen.
	 */
	if (fd==0) {
		printf ("Internal problem in vn_open.\n");
		exit (1);
	};

	*vpp = (struct vnode*)fd;
	return 0;
}


/*
 *--------------------------------------------------------------------------
 *
 * ook_vn_close -- release a user-level vnode allocated with ook_vn_open [depricated]
 *
 * DESCRIPTION:
 *	Ook_vn_open releases a user-level vnode allocated with ook_vn_open
 *	This routine is depricated; new code should use ook_vn_rele.
 *
 *	This routine is depricated; see ook_vn_open for details.
 *
 *	Ook_vn_close should be paired with ook_vn_open.
 *
 * SEE ALSO:
 *	ook_vn_open, ook_lookupname.
 *
 * RETURN VALUES:
 *      Returns an errno value.
 *
 *--------------------------------------------------------------------------
 */
int
ook_vn_close(vp)
	struct vnode *vp;
{
	return close ((int) vp);
}


