
/*
 * userlevelvops.c
 *
 *	@(#)user_vnodeop.c	1.13 (UCLA Ficus Project) 5/16/94
 *
 * Copyright (c) 1992 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This provides some compatibility routines for a very simple
 * ook vnode interface.
 *
 * It doesn't require the whole utok layer, or compat.c.
 *
 * This is meant for typical user-level programs like recon or
 * getfattrs.  The utok layer is for fs layers which should run
 * both in and out of the kernel.
 * 
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/errno.h>
#undef FILE
#include <sys/vnode.h>
#include <i405/i405.h>
#include <i405/i405_syscalls.h>
#include "fix_prog_headers.h"


int user_vnodeop_debug = 0;
/*
 * User level interface to the new vnode operations.
 *
 * DON'T CALL THIS DIRECTLY.
 * USE THE NVOP_* MACROS.
 * (They call this.)
 */
int
user_vnodeop(vp, uuidp, xap)
	struct vnode *vp;
	struct nca_uuid *uuidp;
	void *xap;
{
	struct nvop_generic_args *ap = (struct nvop_generic_args *)xap;
	struct nvop_generic_args **app = &ap;  /* YAH--yet another hack */
	int error;
	extern int errno;

	/*
	 * NEEDSWORK: should call struuid here, but then we'd have
	 * to duplicate code.
	 */
	if (user_vnodeop_debug)
		printf ("calling vop, vp=%p, uuidp=%p, app=%p",
			vp, uuidp, ap->a_desc->vdesc_name);
	error = syscall (SYS_user_uuid_vnodeop, vp, uuidp, app);
	if (user_vnodeop_debug)
		printf (".\n");
	return (error == -1) ? errno : 0;
}



