
/*
 * userlevelvops.c
 *
 * This provides some compatibility routines for a very simple
 * ook vnode interface.
 *
 * It doesn't require the whole utok layer, or compat.c.
 *
 * This is meant for typical user-level programs like recon or
 * getfattrs.  The utok layer is for fs layers which should run
 * both in and out of the kernel.
 * 
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/uio.h>
#include <sys/errno.h>
#undef FILE
#include <sys/vnode.h>
#include <i405/i405.h>
#include <i405/i405_syscalls.h>
#include "fix_prog_headers.h"


/*
 * read or write a vnode
 * (compatability routine).
 */
int
ook_vn_rdwr(rw, vp, base, len, offset, seg, ioflag, aresid)
	enum uio_rw rw;
	struct vnode *vp;
	caddr_t base;
	int len;
	int offset;
	int seg;
	int ioflag;
	int *aresid;
{
	USES_NVOP_RDWR;
	struct uio auio;
	struct iovec aiov;
	int error;

#if NOTDEF
	if ((rw == UIO_WRITE) && isrofile(vp)) {
		return (EROFS);
	}
#endif

	aiov.iov_base = base;
	aiov.iov_len = len;
	auio.uio_iov = &aiov;
	auio.uio_iovcnt = 1;
	auio.uio_offset = offset;
	auio.uio_segflg = seg;
	auio.uio_resid = len;
	error = NVOP_RDWR(vp, &auio, rw, ioflag, NULL);  /* kernel will fill in u.u_cred */
	if (aresid)
		*aresid = auio.uio_resid;
	else
		if (auio.uio_resid)
			error = EIO;
	return (error);
}

