.TH INTGEN 1 "9 August 1993" "UCLA Stacking"
.\"
.\"
.\"
.SH NAME
intgen \- compile a UCLA Interface description file
.\"
.\"
.\"
.SH SYNOPSIS
.B intgen
.I [ -h | -c ]
.I [ --no-structures ]
.I [ --backvint ]
.I filename
.\"
.\"
.\"
.SH DESCRIPTION
.LP
.B intgen
translates a UCLA stackable layers interface description file
into C source and header information.
The
.I -h
or 
.I -c
option selects between generating
C header or source files;
one is required.
The interface definition file
(conventionally with a 
.I .int
extension)
is specified either on the command line or is taken from standard input.
.\"
.LP
.\"
The
.I --no-structures
option prevents writing structure definitions.
The intgen code ``STRUCTURE foo { ... }''
will generate RPC information but not a
``struct foo { }'' if this option is specified.
(By default it generates both.)
This option is useful if you're specifying data types declared
elsewhere strictly for RPC purposes.
.\"
.LP
.\"
The
.I --backvint
option instructs intgen to generate code translating
the old vnode interface into the new stackable layers interface.
This option is required only on systems where most of the kernel
runs the old interface while certain file systems are converted
to use the new interface, and then only for system standard vnode
operations.
.\"
.\"
.\"
.SH "LANGUAGE CONSTRUCTS"
A 
.I .int
file resembles a C header file
with a few annotations.
Trivial differences are the replacement of TYPEDEF for typedef
and STRUCTURE for struct.
In addition, 
.B intgen
supports tagged unions,
several types of arrays,
a facility to declare primitive data types,
and, of course, operation declarations.
.LP
.B TYPEDEF
declares two types as equivalent.
For example,
.LP
.nf
.RS
.DT
.ft B
TYPEDEF u_short uid_t;
.ft R
.RE
.fi
.LP
will make
.B uid_t
a synonym for u_short.
If
.B --no-structure
is not defined,
a
.B TYPEDEF
command will generate a corresponding
C-typedef in the header file
and any necessary RPC support procedures.
.LP
.B PRIMITIVE
declares a ``primitive'' type,
a type implemented directly by low-level RPC services.
For example,
.LP
.nf
.RS
.DT
.ft B
PRIMITIVE {
    i405_xdr_struct_ucred,
    i405_utok_struct_ucred,
    i405_alloc_yes,
    i405_xdr_struct_ucred
} struct ucred;
.ft R
.RE
.fi
.LP
declares ``struct ucred'' as a primitive operation.
Each identifier in the curly braces is a function
to be called for the appropriate RPC protocol.
A list of RPC protocols can be found in 
.BR "<i405/i405_transport.h>" .
.B PRIMITIVE
should be required only for backwards compatibility
and providing very basic data types.
This declaration generates only RPC information.
.LP
.B STRUCTURE
declares standard structures.
As an example,
.LP
.nf
.RS
.DT
.ft B
STRUCTURE
timeval {
    long tv_sec;  /* seconds */
    long tv_usec; /* and microseconds */
};
.ft R
.RE
.fi
.LP
declares a simple structure.
The header file will declare a corresponding C-struct
while the source will contain necessary RPC information.
.LP
.B OPERATION
declares new operations.
It is used just as 
.B STRUCTURE
except that each field is proceeded by a
.IR direction .
Directions are either
.BR IN ,
.BR OUT ,
or
.BR INOUT ,
specifying in which direction the data will be copied
when it moves between address spaces.
Operations
must conclude with the
operation's UUID as generated
by the program
.BR make_nca_uuid (1).
.LP
A sample operation might be:
.LP
.nf
.RS
.DT
.ft B
OPERATION
nvop_rdwr {
    IN struct vnode *a_vp;
    INOUT struct uio *a_uiop;
    IN int a_rw;
    IN int a_ioflag;
    IN struct ucred *a_cred;
} uuid(2ac782a70000.02.83.b3.c0.45.00.00.00);
.ft R
.RE
.fi
.LP
An operation declaration generates several relevant
data structures in the header file.
Operation foo will generate
.BR "struct foo_args" .
A pointer to this sort of structure will be passed to 
a file system layer's function implementing this operation.
In addition, a macro
.B FOO(FOOARG1, FOOARG2, ...)
will be created to invoke the operation.
This macro can be treated as an ordinary function call.
Any function invoking the operation must place a macro
.B USES_FOO
with its local variable declarations.
.LP
An additional modifier,
.BR NOTRANSLATE ,
can be specified for operation arguments of type ``struct vnode *''.
Ordinarily vnodes are mapped as they pass down the stack.
The
.B NOTRANSLATE
flag specifies that a particular argument not be mapped
and should be used when the vnode is defined by the operation
as a generic vnode instead of a layer-specific vnode.
.LP
Unions can be declared with the 
.B UNION
declaration.
All unions must be tagged;
each branch of the union must specify a tag.
For example,
.LP
.nf
.RS
.DT
.ft B
UNION i405_union_test_union {
CASE i405_union_test_one_tag: struct i405_union_test_one one;
CASE i405_union_test_two_tag: struct i405_union_test_two two;
};
.ft R
.RE
.fi
.LP
declares a union with two possible tag values.
Each tag associates a tag with a single data structure
(typically a struct).
For a
.B UNION
foo the header file will declare
``struct foo_union''
with two elements: an integer ``tag''
and a C-union ``un'' containing
each possible union option.
.LP
Several kinds of arrays are supported,
each with a slightly different style of usage.
Because arrays are not directly supported in the C language,
a set of macros assist in accessing arrays.
For an array
.B ary
declared 
in structure
.BR str ,
the macro 
.B struct_str_ary
will return the array;
while the macros
.B struct_str_ary_elems
and 
.B struct_str_ary_space
access the number of used elements
and the amount of allocated space (in elements).
(Note that the
.B _space
macro is defined only for variable-length arrays.)
.LP
.BR "IMPLICIT ARRAY" s
are have a statically fixed length.
Example declarations are:
.LP
.nf
.RS
.DT
.ft B
IMPLICIT ARRAY (int, 2) b;
IMPLICIT ARRAY (char, BUFSIZ) c;
.ft R
.RE
.fi
.LP
Declaring
.B b
as an array of two integers.
.LP
.BR "FIXED ARRAY" s
have a length defined by the caller.
A sample declaration is:
.LP
.nf
.RS
.DT
.ft B
OPERATION foo {
    FIXED ARRAY (struct replicalist) b;
} uuid(...);
.ft R
.RE
.fi
.LP
This fixed array will have a macro
.B foo_b_elems
declared.
.LP
Finally, in a
.B VARIABLE ARRAY
the caller pre-allocates a buffer
.B foo_c_space
elements long
and the callee
will fill in the array and set
.B foo_c_elems
with the number of used elements.
If the caller's buffer is too small,
the callee should set
.B foo_c_elems 
to the real number of elements,
fill in the first
.B foo_c_space 
elements of the array,
and return no error.
The caller is responsible to detect this error condition.
.LP
In addition to these data types,
several declarations allow code to be placed
directly into the header and source files.
.B BEGIN VERBATIM C
and
.B END VERBATIM C
tag a block to be copied into the source file;
corresponding
.B H
declarations exist for header files.
It is often required to insert:
.LP
.nf
.RS
.DT
.ft B
BEGIN VERBATIM C
#include <i405/test_interface.h>
END VERBATIM C
.ft R
.RE
.fi
.LP
in the test_interface.int to include the proper header in the source file.
A
.B BEGIN COMMENT
allows comment blocks which are swallowed.
.\" BEGIN VERBATIM {C,H}/BEGIN COMMENT
.\"
.SH "ADDING A NEW OPERATION"
There are several steps to adding a new operation to some file system.
.LP
First, create a
.B .int
file.
Typically the interface for the foo file system
will be called foo_interface.int.
.LP
From this interface definition file generate
source and header files.  This is typically done with
.LP
.nf
.RS
.DT
.ft B
intgen -h foo_interface.int >foo_interface.h
intgen -c foo_interface.int >foo_interface.c
.ft R
.RE
.fi
.LP
Alternatively the Ficus Imakefile target
.B "I405InterfaceTarget(foo_interface)" 
can be used to specify these commands in an 
.BR Imakefile .
.LP
Next, the source file 
.B foo_interface.c
should be added to the kernel by editing
.BR conf.common/files.cmn .
.LP
The operation needs to be added to the global table of operations.
This is found in the file
.BR i405/i405_op_conf.c ;
this file will need to be changed in two places.
The inserted code will resemble:
.LP
.nf
.RS
.DT
.ft B
extern struct vnodeop_desc newop_desc;
...
struct vnodeop_desc *i405_op_descs[] = {
    ...
    &newop_desc,
    ...
};
.ft R
.RE
.fi
.LP
Finally you will need to add your operation to your particular
file system layer.
Add the operation to your layer's list of vnode operations
(typically in
.BR foo_vnodeops.c )
and the add code implementing the operation.
An example would be:
.LP
.nf
.RS
.DT
.ft B
...
static int foo_newop();
...
struct vnodeopv_entry_desc foo_vnodeop_entries[] = {
    ...
    { &newop_desc, foo_newop },
    ...
};
...
static int
foo_newop(ap)
	struct newop_args *ap;
{
    /* your code goes here */
    return 0;
}
.ft R
.RE
.fi
.LP
.\"
.\"
.\"
.SH "SEE ALSO"
.BR make_nca_uuid (1)
.\"
.SH BUGS
.LP
The C-pre-processor is
.I not
run on the 
.B .int
file, but on the generated header and source files.
This may produce unexpected behavior.
.LP
This manual page describes intgen version 2.
This program is currently installed as intgen2, not
as intgen.
.\" Local IspellWords:  INTGEN intgen backvint int foo struct uid xdr ucred utok
.\" Local IspellWords:  alloc timeval tv usec INOUT operation's UUID nca uuid
.\" Local IspellWords:  rdwr vp uio uiop rw ioflag cred ac args layer's FOOARG
.\" Local IspellWords:  NOTRANSLATE ary str elems BUFSIZ replicalist pre cmn
.\" Local IspellWords:  callee Imakefile InterfaceTarget conf op vnodeop desc ap
.\" Local IspellWords:  newop descs vnodeopv nvop un IspellWords
.\" Local IspellWords:  
