
/*
 * newvint_init.c
 *
 * This has all the configuration stuff for the new interface.
 * It compiles in and out of the kernel, handling things either way.
 * 
 * Define "BACKVINT" to fill in the first ops with ops that will
 * translate the old interface to the new.
 */



#include <sys/param.h>
#include <sys/time.h>
#include <sys/vnode.h>
#include <sys/nvnode.h>


#if 0
#define DODEBUG(A) A
#else
#define DODEBUG(A)
#endif


#if defined(BACKVINT) && defined(OUTOFKERNEL)
Backwards compatibility is not supported in the out-of-kernel interface.
#endif



extern struct vnodeop_defn *vnodeop_defns[];
				/* a list of lists of vnodeops defns */
extern struct vnodeop_desc *vnodeop_descs[];
				/* and the operations they perform */
/*
 * This code doesn't work if the defn is **vnodop_defns with cc.
 * The problem is because of the compiler sometimes putting in an
 * extra level of indirection for arrays.  It's an interesting
 * "feature" of C.
 */
extern int num_operations;
#ifdef BACKVINT
extern int num_back_operations;
#endif

typedef (*PFI)();   /* the standard Pointer to a Function returning an Int */



#ifdef BACKVINT
int	oldtonew_open(),
	oldtonew_close(),
	oldtonew_rdwr(),
	oldtonew_ioctl(),
	oldtonew_select(),
	oldtonew_getattr(),
	oldtonew_setattr(),
	oldtonew_access(),
	oldtonew_lookup(),
	oldtonew_create(),
	oldtonew_remove(),
	oldtonew_link(),
	oldtonew_rename(),
	oldtonew_mkdir(),
	oldtonew_rmdir(),
	oldtonew_readdir(),
	oldtonew_symlink(),
	oldtonew_readlink(),
	oldtonew_fsync(),
	oldtonew_inactive(),
	oldtonew_lockctl(),
	oldtonew_fid(),
	oldtonew_getpage(),
	oldtonew_putpage(),
	oldtonew_map(),
	oldtonew_dump(),
	oldtonew_cmp(),
	oldtonew_realvp();

struct vnodeops oldtonew_vnodeops = {
	oldtonew_open,
	oldtonew_close,
	oldtonew_rdwr,
	oldtonew_ioctl,
	oldtonew_select,
	oldtonew_getattr,
	oldtonew_setattr,
	oldtonew_access,
	oldtonew_lookup,
	oldtonew_create,
	oldtonew_remove,
	oldtonew_link,
	oldtonew_rename,
	oldtonew_mkdir,
	oldtonew_rmdir,
	oldtonew_readdir,
	oldtonew_symlink,
	oldtonew_readlink,
	oldtonew_fsync,
	oldtonew_inactive,
	oldtonew_lockctl,
	oldtonew_fid,
	oldtonew_getpage,
	oldtonew_putpage,
	oldtonew_map,
	oldtonew_dump,
	oldtonew_cmp,
	oldtonew_realvp,
};
#define NUM_OLD_VOPS (sizeof (oldtonew_vnodeops)/sizeof(PFI))
#endif


#define UUID_HASH_BUCKETS 0x20
#define UUID_HASH_MASK (UUID_HASH_BUCKETS-1)
#define UUID_TO_HASH(UUID) (UUID & UUID_HASH_MASK)

struct vnodeop_desc *uuid_hash[UUID_HASH_BUCKETS];


/*
 * Now does hashing!
 */
struct vnodeop_desc *
uuid_to_desc (uuid)
	int uuid;
{
	int i;
	struct vnodeop_desc *vdp = NULL, *nvdp;
	int hash = UUID_TO_HASH(uuid);

	/*
	 * First check the hash chain.
	 */
	vdp = uuid_hash[hash];
	while (vdp)
		if (vdp->vdesc_uuid == uuid)
			return vdp;
		else vdp = vdp->vdesc_uuidhash;

	/*
	 * Hash miss.  See if it's there with a linear search.
	 */
	for (i=0; vnodeop_descs[i]; i++)
		if (vnodeop_descs[i]->vdesc_uuid == uuid) {
			/*
			 * Found it.  Add to end of hash chain.
			 */
			nvdp = vnodeop_descs[i];
			nvdp->vdesc_uuidhash = NULL;
			if (uuid_hash[hash]==NULL) {
				uuid_hash[hash] = nvdp;
				return nvdp;
			}
			else {
				vdp = uuid_hash[hash];
				while (vdp->vdesc_uuidhash)
					vdp = vdp->vdesc_uuidhash;
				return vdp->vdesc_uuidhash = nvdp;
			};
		};
	return NULL;
}



#if 0
struct vnodeop_desc *
offset_to_desc (offset)
	int offset;
{
#ifdef BACKVINT
	offset -= NUM_OLD_VOPS;
#endif	
	return vnodeop_descs[offset];
}
#endif




void
vnode_client_init()
{
	int i, j, k;
	struct vnodeop_defn *defnp;

	/*
	 * Allocate the dynamic vectors and fill them in.
	 */
	for (i=0; vnodeop_defns[i]; i++)
		for (j=0; vnodeop_defns[i][j].vdefn_routine; j++) {
			defnp = &vnodeop_defns[i][j];

			/*
			 * Allocate and init the vector, if it needs it.
			 * Also handle backwards compatibility.
			 */
			if (*defnp->vdefn_vector == NULL) {
				*defnp->vdefn_vector = (PFI*)
					kmem_zalloc(num_operations*sizeof(PFI));
#ifdef BACKVINT
				bcopy(&oldtonew_vnodeops, *defnp->vdefn_vector,
				      NUM_OLD_VOPS*sizeof(PFI));
#endif
			};
			/*
			 * Sanity check:  is this operation listed
			 * in the list of operations?  We check this
			 * by seeing if its offest is zero.  Since
			 * the default routine should always be listed
			 * first, it should be the only one with a zero
			 * offset.  Any other operation with a zero
			 * offset is probably not listed in vnodeop_descs,
			 * and so is probably an error.
			 */
			if (defnp->vdefn_desc->vdesc_offset == 0 &&
				    defnp->vdefn_desc->vdesc_offset !=
				    	VOFFSET(nvop_default)) {
				printf ("error: operation %s not listed in vnodeop_descs.\n", defnp->vdefn_desc->vdesc_name);
				panic ("bad operation");
			};
			/*
			 * Fill in this entry.
			 */
			(*defnp->vdefn_vector)
				[defnp->vdefn_desc->vdesc_offset] =
					defnp->vdefn_routine;
		};

	/*
	 * Finally, go back and replace unfilled routines
	 * with their default.  (Sigh, an O(n^3) algorithm.  I
	 * could make it better, but that'd be work, and n is small.)
	 */
	for (i=0; vnodeop_defns[i]; i++)
		for (j=0; vnodeop_defns[i][j].vdefn_routine; j++) {
			defnp = &vnodeop_defns[i][j];
			for (k=0; k<num_operations; k++)
				if ((*defnp->vdefn_vector)[k] == NULL)
					(*defnp->vdefn_vector)[k] = 
						(*defnp->vdefn_vector)
							[VOFFSET(nvop_default)];
			/* love that indention... */
		};

}




#ifdef OUTOFKERNEL


/*
 * Turn this variable on to see every in-to-kernel vnodeop printed.
 */
int user_new_vnodeop_debug=0;

/*
 * User level interface to the new vnode operations.
 * NEEDSWORK: Bad style, uses magic syscall numbers.
 *
 * We used to dredge actual offsets out of the kernel,
 * but this became too big of a pain, so now we use uuids
 * between the user and kernel levels.
 */

int
user_new_vnodeop(vp, off, ap)
	struct vnode *vp;
	int off;
	struct nvop_generic_args *ap;
{
	struct nvop_generic_args **app = &ap;  /* YAH--yet another hack */
	int error;
	extern int errno;

	if (user_new_vnodeop_debug)
		printf ("calling vop, vp=%x, off=%d(%s), app=%x", vp, off,
			ap->a_desc->vdesc_name,
			app);
	error = syscall (224, vp, ap->a_desc->vdesc_uuid, app);

	if (user_new_vnodeop_debug)
		printf (".\n");
	return (error == -1) ? errno : 0;
}

#endif



void
vnode_interface_init()
{
	int num_ops = 0;
	int i, j;

	/*
	 * Set all vnode vectors to a well known value.
	 */
	for (i=0; vnodeop_defns[i]; i++)
		for (j=0; vnodeop_defns[i][j].vdefn_routine; j++) {
			*vnodeop_defns[i][j].vdefn_vector = NULL;
		};

#ifdef BACKVINT
	/*
	 * Leave space at the beginning for old compatibility functions.
	 */
	num_ops = num_back_operations = NUM_OLD_VOPS;
	DODEBUG(printf ("ops 0-%d reserved for compatibility\n", num_ops-1));
#else
	num_ops = 0;
#endif


	/*
	 * Figure out how many ops there are by counting the table,
	 * and assign each its offset.
	 */
	for (i=0; vnodeop_descs[i]; i++) {
		vnodeop_descs[i]->vdesc_offset = num_ops;
/*OLD:		*(vnodeop_descs[i]->vdesc_offsetp) = num_ops; */
		num_ops++;
	};

	num_operations = num_ops;
     
	vnode_client_init();   /* finish the job */
}





