
/*
 * newvint_conf.c
 *
 * This is the user-changable configuration file for the new interface.
 * It parallels vfs_conf.c, but for the vnode interface rather than the
 * vfs inteface.
 *
 */

#include <sys/param.h>
#include <sys/time.h>
#include <sys/vnode.h>
#include <sys/nvnode.h>


extern struct vnodeop_desc
	nvop_open_desc,
	nvop_close_desc,
	nvop_rdwr_desc,
	nvop_ioctl_desc,
	nvop_select_desc,
	nvop_getattr_desc,
	nvop_setattr_desc,
	nvop_access_desc,
	nvop_lookup_desc,
	nvop_create_desc,
	nvop_remove_desc,
	nvop_link_desc,
	nvop_rename_desc,
	nvop_mkdir_desc,
	nvop_rmdir_desc,
	nvop_readdir_desc,
	nvop_symlink_desc,
	nvop_readlink_desc,
	nvop_fsync_desc,
	nvop_inactive_desc,
	nvop_lockctl_desc,
	nvop_fid_desc,
	nvop_getpage_desc,
	nvop_putpage_desc,
	nvop_map_desc,
	nvop_dump_desc,
	nvop_cmp_desc,
	nvop_realvp_desc,
/* Ficus ops */
	fvop_debug_desc,
	fvop_orphan_desc,
	fvop_getfattrs_desc,
	fvop_setfattrs_desc,
	fvop_newcreate_desc,
#ifdef NEWOFS
	nvop_newopen_desc,
	nvop_newfid_desc,
	nvop_purge_cache_desc,
	nvop_newfid_to_vp_desc,
#endif
#ifdef CONSISTENTFS
	nvop_get_token_desc,
#endif
/* and the default */
	nvop_default_desc;



/*
 * This lists all vnodeoperations supported.  It's used in initialization
 * at compile time.  Position in this null-terminated list
 * determines vnode operation offset (possibly with compatibilty allowed).
 */

struct vnodeop_desc *vnodeop_descs[] = {
	&nvop_default_desc,   /* must be first */
	&nvop_open_desc,
	&nvop_close_desc,
	&nvop_rdwr_desc,
	&nvop_ioctl_desc,
	&nvop_select_desc,
	&nvop_getattr_desc,
	&nvop_setattr_desc,
	&nvop_access_desc,
	&nvop_lookup_desc,
	&nvop_create_desc,
	&nvop_remove_desc,
	&nvop_link_desc,
	&nvop_rename_desc,
	&nvop_mkdir_desc,
	&nvop_rmdir_desc,
	&nvop_readdir_desc,
	&nvop_symlink_desc,
	&nvop_readlink_desc,
	&nvop_fsync_desc,
	&nvop_inactive_desc,
	&nvop_lockctl_desc,
	&nvop_fid_desc,
	&nvop_getpage_desc,
	&nvop_putpage_desc,
	&nvop_map_desc,
	&nvop_dump_desc,
	&nvop_cmp_desc,
	&nvop_realvp_desc,
/* Ficus vnodeops */
	&fvop_debug_desc,
	&fvop_orphan_desc,
	&fvop_getfattrs_desc,
	&fvop_setfattrs_desc,
	&fvop_newcreate_desc,
#ifdef NEWOFS
	&nvop_newopen_desc,
	&nvop_newfid_desc,
	&nvop_purge_cache_desc,
	&nvop_newfid_to_vp_desc,
#endif
#ifdef CONSISTENTFS
	&nvop_get_token_desc,
#endif
	NULL
};


