
/*
 * newvint_conf.c
 *
 * This is the user-changable configuration file for the new interface.
 * It parallels vfs_conf.c, but for the vnode interface rather than the
 * vfs inteface.
 *
 */

#include <sys/param.h>
#include <sys/time.h>
#include <sys/vnode.h>
#include <sys/nvnode.h>



#ifdef NFSCLIENT
extern struct vnodeop_defn nfs_vnodeop_decls[];
#endif

#ifdef UFS
extern struct vnodeop_defn ufs_vnodeop_decls[];
#endif


#ifdef FICUS
extern struct vnodeop_defn flfs_vnodeop_decls[];
extern struct vnodeop_defn flfs_raw_vnodeop_decls[];
extern struct vnodeop_defn flfs_control_vnodeop_decls[];
extern struct vnodeop_defn fpfs_vnodeop_decls[];
extern struct vnodeop_defn fpfs_queue_vnodeop_decls[];
extern struct vnodeop_defn fpfs_unnamed_vnodeop_decls[];
extern struct vnodeop_defn fpfs_norepl_vnodeop_decls[];
#endif

#ifdef NULLFS
extern struct vnodeop_defn null_vnodeop_decls[];
#endif

#ifdef NEWOFS
extern struct vnodeop_defn newo_vnodeop_decls[];
#endif

#ifdef MEASUREFS
extern struct vnodeop_defn measure_vnodeop_decls[];
#endif

#ifdef SHRINKFIDFS
extern struct vnodeop_defn shrinkfid_vnodeop_decls[];
#endif

#ifdef CONSISTENTFS
extern struct vnodeop_defn consistent_vnodeop_decls[];
#endif

#if 0
#ifdef LOFS
extern struct vnodeop_defn lo_vnodeop_decls[];
#endif

extern struct vnodeop_defn fifo_vnodeop_decls[];
extern struct vnodeop_defn spec_vnodeop_decls[];
#endif 0


/*
 * vnodeop_defns provides a list of routines for each vnodeops vector.
 * It's null terminated.
 *
 * Out-of-kernel, someone else (more knowlegable about what file
 * systems live in this address space) must specify this table.
 */


struct vnodeop_defn *vnodeop_defns[] = {

#ifdef NFSCLIENT
	nfs_vnodeop_decls,
#endif

#ifdef UFS
	ufs_vnodeop_decls,
#endif

#ifdef FICUS
	flfs_vnodeop_decls,
	flfs_raw_vnodeop_decls,
	flfs_control_vnodeop_decls,
	fpfs_vnodeop_decls,
	fpfs_queue_vnodeop_decls,
	fpfs_unnamed_vnodeop_decls,
	fpfs_norepl_vnodeop_decls,
#endif

#ifdef NULLFS
	null_vnodeop_decls,
#endif

#ifdef NEWOFS
	newo_vnodeop_decls,
#endif

#ifdef MEASUREFS
	measure_vnodeop_decls,
#endif

#ifdef SHRINKFIDFS
	shrinkfid_vnodeop_decls,
#endif

#ifdef CONSISTENTFS
	consistent_vnodeop_decls,
#endif

#if 0
#ifdef LOFS
	lo_vnodeop_decls,
#endif
	fifo_vnodeop_decls,
	spec_vnodeop_decls,
#endif 0
	NULL
};    /* a list of lists of vnodeops defns */

#endif /* OUTOFKERNEL */



int num_operations;
#ifdef BACKVINT
int num_back_operations;
#endif



