 
/*
 * newvint_xdr_special.c
 *
 *
 * Vnodes are given to the user level as file descriptiors.
 * This means that when a user program crashes, its vnodes
 * will be released.
 *
 * NEEDSWORK: uio in to OTW version isn't implemented.
 * (Actually, it is, but it returns useless pointer information.)
 *
 * Note: Define SECURE_NEWVINT to make the vnode interface "secure".
 * This means disallowing user-supplied credentials for non-superusers.
 * (This isn't defined now because it's a pain to have to run
 * the Ficus OOK daemons as root.)
 */
 

#include <sys/param.h>
#include <sys/file.h>
#include <sys/errno.h>
#include <sys/uio.h>
#include <sys/ucred.h>
#include <sys/vfs.h>
#include <sys/time.h>
#include <sys/vnode.h>
#include <sys/nvnode.h>
#include <rpc/types.h>
#include <rpc/xdr.h>
#include <nfs/nfs.h>
#include <nfs/export.h>
#include <nfs/rnode.h>   /* for vtofh */
#include <newvint/xxdr.h>
#include <sys/user.h>

#define DTYPE_OURS 'j'  /* not quite DTYPE_VNODE */


#if 0
#define DODEBUG(A) A
#else
#define DODEBUG(A)
#endif



int
my_vno_close(fp)
	register struct file *fp;
{
	register struct vnode *vp;

        DODEBUG(printf ("my_vno_close called on fp=%x, vp=%x\n", fp,
		fp->f_data));
	vp = (struct vnode *)fp->f_data;
#ifndef OUTOFKERNEL
	/*
	 * my_vno_close is never called OOK, but this makes a compile error.
	 */
 	VN_RELE(vp);
#endif
	return (0);
}


struct fileops my_vnodefops = {
	NULL, NULL, NULL, my_vno_close
#if 0
	vno_rw,
	vno_ioctl,
	vno_select,
	vno_close
#endif
};




#ifndef OUTOFKERNEL

struct vnode *
fdtovp(fd)
	int fd;
{
	struct file *fp;

	DODEBUG(printf ("fdtovp: fd=%d\n", fd));

	fp = getf(fd);
	if (fp==NULL) {
		u.u_error = EBADF;
		return NULL;
	};

	if (fp->f_type!=DTYPE_OURS && fp->f_type!=DTYPE_VNODE) {
		u.u_error = EINVAL;
		return NULL;
	};
		
	DODEBUG(printf ("   -> vp=%x\n", fp->f_data));

        /* get the vp from f_data */
        return (struct vnode *)fp->f_data;
}


int
vptofd(vp)
	struct vnode *vp;
{
	struct file *fp;
        int indx;
	extern struct fileops my_vnodefops;

	/*
	 * See if the user already holds a vnode like this.
	 */
	for (indx=0; indx<NOFILE; indx++)
		if (fp=u.u_ofile[indx])
			if (fp->f_count!=0 && (fp->f_type==DTYPE_OURS || fp->f_type==DTYPE_VNODE) && (struct vnode*)fp->f_data==vp) {
				/*
				 * fp->f_count++;
				 * I don't think we want to do this.
				 */
				DODEBUG(printf ("File %d re-used.\n", indx));
				break;
			};

        /*
         * allocate a user file descriptor and file table entry.
         */
	if (indx>=NOFILE) {
		fp = falloc();
		if (fp == NULL) {
			printf ("vptofd: Out of file descriptors for process %d.\n", u.u_procp->p_pid);
			u.u_error = EMFILE;
			return -1;
		};
		indx = u.u_r.r_val1;     /* this is bullshit (according to Sun) */

		/*
		 * Fill in the file struct.
		 */
		DODEBUG(printf ("New file entry %d allocated.\n", indx));
		fp->f_flag = 0 /*filemode & FMASK*/;
		fp->f_type = DTYPE_OURS;   /* not DTYPE_VNODE */
		fp->f_data = (caddr_t)vp;
		fp->f_ops = &my_vnodefops;
	};
 
	DODEBUG(printf ("vptofd: vp %x -> uvp %d\n", vp, indx));

	return indx;
}




/*
 * Handle inactive specially (to free the fd).
 * This code copied from "close".
 */
int
user_inactive(uvp, ap)
	int uvp;
	struct nvop_inactive_args **ap;
{
        struct file *fp;
        int err;
	u_char *pf;

	DODEBUG(printf ("user_inactive: uvp %d -> \n", uvp));
	fp = getf(uvp);
	DODEBUG(printf ("   -> fp %x\n", fp));
	if (fp==NULL)
		return EBADF;
 
	/* Release all System-V style record locks, if any */
	(void) vno_lockrelease(fp);	/* WHAT IF error returned? */

	/*
	 * Taken directly from close
	 */
	pf = (u_char *)&u.u_pofile[uvp];
	u.u_ofile[uvp] = NULL;
	while (u.u_lastfile >= 0 && u.u_ofile[u.u_lastfile] == NULL)
		u.u_lastfile--;
	*pf = 0;
	closef(fp);

	return u.u_error;   /* those Sun people aren't very clean... */
}

#endif !OUTOFKERNEL



/*************************************************************************
 *
 * following is code to map between vp's and fh's.
 */

bool_t
xdr_fhtovp(xdrs, vp_p)
	XDR *xdrs;
        struct vnode **vp_p;
{
	fhandle_t fh;
	struct exportinfo *exi;
	extern struct vnode *fhtovp();

	/*
	 * We assume that vp_p is going to be an rnode.
	 */
	if (!xdr_fhandle(xdrs, &fh))
		return FALSE;

	/*
	 * NEEDSWORK: We'd like the exi to be put in args in
	 * rfs_dispatch, but that happens _after_ the arguments
	 * are serialized.  But right now, we're in the middle
	 * of serializing.  The hack is to figure them out now,
	 * on the fly.  Of course, I ignore all the authentication
	 * stuff done in the real code.  This needs to be fixed.
	 */
	exi = NVOP_PUBLIC(xdrs)->exi;
	if (exi==NULL) {
		exi = findexport(&fh.fh_fsid, (struct fid *) &fh.fh_xlen);
#ifndef OUTOFKERNEL
		if (exi == NULL)
			return FALSE;
#endif
		NVOP_PUBLIC(xdrs)->exi = exi;
				/* cache for possible addtl use */
	};
	*vp_p = fhtovp(&fh, NVOP_PUBLIC(xdrs)->exi);
	if (*vp_p == NULL)
		return FALSE;

	return TRUE;
}
 
 
bool_t
xdr_vptofh(xdrs, vp_p)
	XDR *xdrs;
        struct vnode **vp_p;
{
        struct file *fp;
        int indx;
	extern struct fileops my_vnodefops;
	bool_t stat;

	DODEBUG(printf ("in vptofh, vp=%x\n", *vp_p));
	/*
	 * We assume that *vp_p is an nfs rnode.
	 */
	return xdr_fhandle(xdrs, vtofh(*vp_p));
}

/*
 * Unlike xdr_fdvpfree, fhvpfree has to do the vn_rele.
 * This is because file handles each hold their own reference
 * to the vnode.  When the file handle goes away, we must free
 * the reference.  Of course, we blow the file handle away as
 * soon as we map it to the vnode.  Fortunatly, this is done
 * in the nfs_bypass code, where we know all vp's came from file
 * handles.  We therefore make sure that fhvpfree gets 
 * later to do our dirty work.  -JSH 2-Nov-90
 */
bool_t
xdr_fhvpfree(xdrs, vp_p)
	XDR *xdrs;
        struct vnode **vp_p;
{
	if (*vp_p) {
		VN_RELE(*vp_p);
		*vp_p = NULL;
	};
	return TRUE;
}


/*
 * For xdr_fhtovpp, we just build an rnode around things.
 */
bool_t
xdr_fhtovpp(xdrs, vpp_p)
	XDR *xdrs;
        struct vnode ***vpp_p;
{
	fhandle_t fh;
	extern struct vnode *makenfsnode();

	if (*vpp_p == NULL)	/* Sanity check. */
		panic ("xdr_fhtovpp: vpp_p is null.");

	/*
	 * vpp_p will be an rnode.  First we get the fh.
	 */
	if (!xdr_fhandle(xdrs, &fh))
		return FALSE;

	/*
	 * Now we make the nfs node (with NULL attribute).
	 */
	**vpp_p = makenfsnode (&fh, NULL, NVOP_PUBLIC(xdrs)->vfsp);

	return **vpp_p==NULL ? FALSE : TRUE;
}
 
 
bool_t
xdr_vpptofh(xdrs, vpp_p)
	XDR *xdrs;
        struct vnode ***vpp_p;
{
	fhandle_t fh;
        struct file *fp;
        int indx;
	extern struct fileops my_vnodefops;


	/* Make the file handle to send out. */
#ifndef OUTOFKERNEL
	if (NVOP_PUBLIC(xdrs)->exi == NULL)   /* more sanity */
		panic ("xdr_vpptofh: exi is null.");
#endif
	makefh(&fh, **vpp_p, NVOP_PUBLIC(xdrs)->exi);

	/* Send it. */
	return xdr_fhandle(xdrs, &fh);
}

/*
 * We do the same thing as fhvpfree.
 */
bool_t
xdr_fhvppfree(xdrs, vpp_p)
	XDR *xdrs;
        struct vnode ***vpp_p;
{
	if (**vpp_p) {
		VN_RELE(**vpp_p);
		**vpp_p = NULL;
		mem_free (*vpp_p, sizeof(struct vnode**));
					/* free the space for the pointer */
	};
	return TRUE;
}



	
/*************************************************************************
 *
 * following is code to map between vp's and fd's.
 */


#ifndef OUTOFKERNEL
 
bool_t
xdr_fdtovp(xdrs, kvp)
	XDR *xdrs;
        struct vnode **kvp;
{
        int uvp;   /* in user space */
        struct file *fp;
        int err;

	if (!XDR_GETLONG(xdrs, &uvp))
		return FALSE;

	*kvp = fdtovp(uvp);

        return TRUE;
}
 
 
bool_t
xdr_vptofd(xdrs, vp_p)
	XDR *xdrs;
        struct vnode **vp_p;
{
        int indx;

	indx = vptofd(*vp_p);

        /*
         * Send the user the vnode.
         */
	return XDR_PUTLONG(xdrs, &indx);
}

/*
 * fdvpfree doesn't do anything because for the user-to-kernel interface,
 * user processes are allowed to hold references to in-kernel vnodes.
 * The user presumably cleans these up by vop_inactive calls (perhaps
 * by the utok package).  These are also cleaned up when the user's
 * program terminates, becuase all file descriptors have their close
 * function called, and we have a special close function which will
 * release the vp.   -JSH 2-Nov-90
 */
bool_t
xdr_fdvpfree(xdrs, vp_p)
	XDR *xdrs;
        struct vnode **vp_p;
{
	*vp_p = NULL;
	return TRUE;
}

/*
 * For all the fd stuff, handling of vp's and vpp's is the same
 * (except for one level of indirection).  This is because of the
 * glorious fact that we hold ook references, making life infinitely
 * easier.
 */
bool_t
xdr_fdvpp_stuff(xdrs, vpp_p)
	XDR *xdrs;
        struct vnode ***vpp_p;
{
	extern bool_t xdr_struct_vnode_p();

	return xdr_alt_reference(xdrs, vpp_p,
				 sizeof(struct vnode **), xdr_struct_vnode_p);
}

#endif !OUTOFKERNEL


bool_t
xdr_struct_vnode_p(xdrs, vp_p)
        register XDR *xdrs;
	struct vnode **vp_p;
{
	int uvp;
	int err;

	switch (xdrs->x_op) {
	case XDR_FREE:
		return (*(NVOP_PUBLIC(xdrs)->vp_free))(xdrs, vp_p);
	case XDR_DECODE:
		return (*(NVOP_PUBLIC(xdrs)->otw_to_vp))(xdrs, vp_p);
	case XDR_ENCODE:
		return (*(NVOP_PUBLIC(xdrs)->vp_to_otw))(xdrs, vp_p);
	};
}


bool_t
xdr_struct_vnode_pp(xdrs, vpp_p)
        register XDR *xdrs;
	struct vnode ***vpp_p;
{
	int uvp;
	int err;

	/*
	 * Allocate space, if necessary
	 *
	 * This will be done on the caller side with XDR_ENCODE,
	 *  and the vpp_p should be some good value (where the caller
	 *  wants the result.
	 *
	 * It will also be done by the receiver side, in which case
	 *  vpp_p will point to a null value and so we will allocate
	 *  space.
	 */
	if (XXDR_SPACE(xdrs)) {
		if (*vpp_p == NULL) {
			*vpp_p=(struct vnode**)
				mem_alloc(sizeof(struct vnode**));
		};
		**vpp_p = NULL;   /* zero the vpp */
		/*
		 * Make things right for xdr user by skipping the pointer.
		 */
		XXDR_SKIP(xdrs, sizeof(struct vnode**));
		return TRUE;
	};

	/*
	 * We'll get here on to XDR_ENCODE the result vnode when
	 * returning values (OUT) on receiver side.  In that case
	 * we call vpptofh which returns the fh.  Then we'll be called
	 * with XDR_FREE, so we release the vnode.
	 *   Then the caller gets the results and calls us with
	 * XDR_DECODE to get the result.  We should not get called
	 * with XDR_FREE there, because the interface assumes that
	 * the caller provides all buffers and releases all references.
	 */
	switch (xdrs->x_op) {
	case XDR_FREE:
		return (*(NVOP_PUBLIC(xdrs)->vpp_free))(xdrs, vpp_p);
	case XDR_DECODE:
		return (*(NVOP_PUBLIC(xdrs)->otw_to_vpp))(xdrs, vpp_p);
	case XDR_ENCODE:
		return (*(NVOP_PUBLIC(xdrs)->vpp_to_otw))(xdrs, vpp_p);
	};
}


bool_t
xdr_struct_ucred(xdrs, cred)
	register XDR *xdrs;
	register struct ucred *cred;
{
	int i;

	DODEBUG(printf("xdr_struct_ucred: called\n"));
	if (!XXDR_LINEAR(xdrs))
		XXDR_SKIP(xdrs, sizeof(u_short));  /* reference count */
	DODEBUG(printf ("xdr_struct_ucred: @\n"));
	if (!xdr_alt_short(xdrs, &cred->cr_uid))
		return FALSE;
	DODEBUG(printf ("xdr_struct_ucred: a\n"));
	if (!xdr_alt_short(xdrs, &cred->cr_gid))
		return FALSE;
	DODEBUG(printf ("xdr_struct_ucred: b\n"));
	if (!xdr_alt_short(xdrs, &cred->cr_auid))
		return FALSE;
	DODEBUG(printf ("xdr_struct_ucred: c\n"));
	if (!xdr_alt_short(xdrs, &cred->cr_ruid))
		return FALSE;
	DODEBUG(printf ("xdr_struct_ucred: d\n"));
	if (!xdr_alt_short(xdrs, &cred->cr_rgid))
		return FALSE;
	DODEBUG(printf ("xdr_struct_ucred: e\n"));
	for (i=0; i<NGROUPS; i++)
		if (!xdr_alt_short(xdrs, &cred->cr_groups[i]))
			return FALSE;
#if 0   /* I'm not sure that this part works, so we just send them all. :-( */
		else if (cred->cr_groups[i] == 0)
			break;   /* 0 terminated list */
#endif
	DODEBUG(printf ("Cred: uid=%d, gid=%d, g1=%d, g2=%d, g3=%d\n",
		cred->cr_uid, cred->cr_gid,
		cred->cr_groups[0], cred->cr_groups[1], cred->cr_groups[2]));
	return TRUE;
}

bool_t
xdr_struct_ucred_p(xdrs, credp)
        register XDR *xdrs;
        struct ucred **credp;            /* the pointer to work on */
{
	struct ucred *ucred;
	register struct ucred *cred = *credp;
        register bool_t stat;
	bool_t creds_coming;

	DODEBUG(printf("xdr_struct_ucred_p called\n"));
	if (xdrs->x_op == XDR_FREE) {
		if (cred!=NULL) {
			crfree(cred);
			*credp = NULL;
		};
		return TRUE;
	} else if (XXDR_LINEAR(xdrs)) {   /* OTW, full creds must be sent */
		if (cred==NULL && xdrs->x_op == XDR_DECODE) {
			*credp = cred = crget();
		};
		return xdr_struct_ucred(xdrs,cred);
	} else { /* !LINEAR */
		DODEBUG(printf("xdr_struct_ucred_p: not linear\n"));
		/* get the user's pointer */
		if (!XDR_GETLONG(xdrs, &ucred))
			return FALSE;
		if (ucred==NULL) {
			if (xdrs->x_op == XDR_ENCODE)
				return FALSE;   /* user didn't alloc space for cred */
			else {   /* use default creds */
				DODEBUG(printf("xdr_struct_ucred_p: default cred\n"));
				*credp = cred = crgetcred();
				return TRUE;
			};
		};
#ifdef SECURE_NEWVINT
		/*
		 * With user available vnodes, if the user can set
		 * their own credentials, then they can masquerade
		 * as anyone.  To prevent this badness, we allow
		 * only privilaged users (the superuser) to pass in
		 * credentials.
		 *
		 * Note: If suser is not defined OOK, then emulate it!
		 * Don't change this code.
		 */
		if (xdrs->x_op == XDR_DECODE && !suser()) {
			u.u_error = EPERM;
			return FALSE;
		};
#endif SECURE_NEWVINT
		if (xdrs->x_op == XDR_DECODE && cred==NULL)
			*credp = cred = crget();
		XXDR_PUSH(xdrs, ucred);
		stat = xdr_struct_ucred(xdrs, cred);
		DODEBUG(printf ("ucred_p: uid=%d, gid=%d\n", cred->cr_uid, cred->cr_gid));
		XXDR_POP(xdrs);
		return stat;
	};
/*	panic ("xdr_struct_ucred_p: bad deal.\n"); */
}


 
bool_t
xdr_struct_iovec_p(xdrs, pp, count)
	register XDR *xdrs;
	struct iovec **pp;		/* the pointer to work on */
	u_int count;			/* number of iovecs to get */
{
	caddr_t userp;
	register struct iovec *loc = *pp;
	bool_t needpop=FALSE, stat;
	int i;

	if (xdrs->x_op == XDR_FREE) {
		if (loc) {
			mem_free(loc, count*sizeof(struct iovec));
			*pp = NULL;
		};
		return TRUE;
	}

	if (loc == NULL && xdrs->x_op==XDR_DECODE) {
		*pp = loc = (struct iovec*) mem_alloc(count*sizeof(struct iovec));
#ifndef KERNEL
		if (loc == NULL) {
			(void) fprintf(stderr,
				       "xdr_reference: out of memory\n");
			return (FALSE);
		}
#endif
	}
	if (!XXDR_LINEAR(xdrs)) {
		if (!XDR_GETLONG(xdrs, &userp))
			return FALSE;
		needpop = TRUE;
		if (!XXDR_PUSH(xdrs, userp))
			return FALSE;
	};

	stat = TRUE;

	for (i=0; i<count; i++) {
		if (!xdr_long(xdrs, &loc[i].iov_base)) {
			stat = FALSE;
			goto out;
		};
		DODEBUG(printf ("base=%x\n", loc[i].iov_base));
		if (!xdr_int(xdrs, &loc[i].iov_len)) {
			stat = FALSE;
			goto out;
		};
		DODEBUG(printf ("len=%x\n", loc[i].iov_len));
	};

out:	
	if (needpop)
		XXDR_POP(xdrs);
	return (stat);
}

bool_t
xdr_struct_uio(xdrs, uio)
        register XDR *xdrs;
        struct uio *uio;            /* the pointer to work on */
{
	u_int loc;

	if (XXDR_LINEAR(xdrs)) {
		 /*
		  * The pointers in the uio are meaningless in a serial world.
		  * This routine needs to be re-done for that case.
		  */
		return FALSE;
	};

	/*
	 * An unfortunate hack: the count should come before
	 * the reference, ideally.
	 */
	if (!XXDR_LINEAR(xdrs)) {
		loc = XDR_GETPOS(xdrs);
		XXDR_SKIP(xdrs, sizeof(struct iovec*));
	};
	if (!xdr_int(xdrs, &uio->uio_iovcnt))
		return FALSE;
	if (!xdr_long(xdrs, &uio->uio_offset))
		return FALSE;
	if (xdrs->x_op == XDR_FREE || XXDR_IN(xdrs)) {
		if (!xdr_alt_short(xdrs, &uio->uio_segflg))
			return FALSE;
		if (!xdr_alt_short(xdrs, &uio->uio_fmode))
			return FALSE;
	} else {
		XXDR_SKIP(xdrs, sizeof(short) + sizeof(short));
	};
	if (!xdr_int(xdrs, &uio->uio_resid))
		return FALSE;
	if (xdrs->x_op == XDR_FREE || XXDR_IN(xdrs)) {
		if (!XXDR_LINEAR(xdrs)) {
			XDR_SETPOS(xdrs, loc);   /* jump back to the iovec reference */
		};
		if (!xdr_struct_iovec_p(xdrs, &uio->uio_iov,
				  uio->uio_iovcnt))
			return FALSE;
	};
	DODEBUG(printf ("iovec at %x\n", uio->uio_iov));
	return TRUE;
}

bool_t
xdr_struct_uio_p(xdrs, pp)
        register XDR *xdrs;
        struct uio **pp;            /* the pointer to work on */
{
	return xdr_alt_reference(xdrs, pp, sizeof(struct uio), xdr_struct_uio);
}

bool_t
xdr_struct_timeval(xdrs, p)
        register XDR *xdrs;
        struct timeval *p;            /* the pointer to work on */
{
	return (xdr_long(xdrs, &p->tv_sec) &&
		xdr_long(xdrs, &p->tv_usec)) ? TRUE : FALSE;
}

bool_t
xdr_struct_vattr(xdrs, p)
        register XDR *xdrs;
        struct vattr *p;            /* the pointer to work on */
{
	if (!xdr_int(xdrs, &p->va_type))
		return FALSE;
	if (!xdr_alt_u_short(xdrs, &p->va_mode))
		return FALSE;
	if (!xdr_alt_short(xdrs, &p->va_uid))
		return FALSE;
	if (!xdr_alt_short(xdrs, &p->va_gid))
		return FALSE;
	if (!xdr_long(xdrs, &p->va_fsid))
		return FALSE;
	if (!xdr_long(xdrs, &p->va_nodeid))
		return FALSE;
	if (!xdr_alt_short(xdrs, &p->va_nlink))
		return FALSE;
	if (!xdr_u_long(xdrs, &p->va_size))
		return FALSE;
	if (!xdr_long(xdrs, &p->va_blocksize))
		return FALSE;
	if (!xdr_struct_timeval(xdrs, &p->va_atime))
		return FALSE;
	if (!xdr_struct_timeval(xdrs, &p->va_mtime))
		return FALSE;
	if (!xdr_struct_timeval(xdrs, &p->va_ctime))
		return FALSE;
	if (!xdr_alt_short(xdrs, &p->va_rdev))   /* dev_t */
		return FALSE;
	if (!xdr_long(xdrs, &p->va_blocks))
		return FALSE;
	return TRUE;
}

bool_t
xdr_struct_vattr_p(xdrs, pp)
        register XDR *xdrs;
        struct vattr **pp;            /* the pointer to work on */
{
	return xdr_alt_reference(xdrs, pp, sizeof(struct vattr), xdr_struct_vattr);
}




/*
 * A short xdr that takes 16 bits when using xdr_user.
 */
bool_t
xdr_alt_short(xdrs, sp)
	XDR *xdrs;
{
	if (XXDR_LINEAR(xdrs)) {
		return xdr_short(xdrs, sp);
	}
	else switch (xdrs->x_op) {
	case XDR_ENCODE: return XDR_PUTBYTES(xdrs, sp, sizeof(short)); break;
	case XDR_DECODE: return XDR_GETBYTES(xdrs, sp, sizeof(short)); break;
	};
	return TRUE;
}
 
 
bool_t
xdr_alt_reference(xdrs, pp, size, proc)
	register XDR *xdrs;
	caddr_t *pp;		/* the pointer to work on */
	u_int size;		/* size of the object pointed to */
	xdrproc_t proc;		/* xdr routine to handle the object */
{
	caddr_t userp;
	register caddr_t loc = *pp;
	register bool_t stat;
	bool_t needpop=FALSE;

	if (loc == NULL)
		switch (xdrs->x_op) {
		case XDR_FREE:
			return (TRUE);

		case XDR_DECODE:
			*pp = loc = (caddr_t) mem_alloc(size);
#ifndef KERNEL
			if (loc == NULL) {
				(void) fprintf(stderr,
				    "xdr_reference: out of memory\n");
				return (FALSE);
			}
			bzero(loc, (int)size);
#else
			bzero(loc, size);
#endif
			break;
	}

	if (xdrs->x_op!=XDR_FREE && !XXDR_LINEAR(xdrs)) {
		if (!XDR_GETLONG(xdrs, &userp))
			return FALSE;
		needpop = TRUE;
		if (!XXDR_PUSH(xdrs, userp))
			return FALSE;
	};

	stat = (*proc)(xdrs, loc /*, LASTUNSIGNED */);

	if (needpop)
		XXDR_POP(xdrs);

	if (xdrs->x_op == XDR_FREE) {
		mem_free(loc, size);
		*pp = NULL;
	}
	return (stat);
}


/*
 * handle freeing both in and out arrays.
 */
bool_t
xdr_free_array(xdrs, addrp, elemsp, spacep, elsize, elproc)
	register XDR *xdrs;
	caddr_t *addrp;
	u_int *elemsp, *spacep;
	u_int elsize;
	xdrproc_t elproc;
{
	int count;
	bool_t stat=TRUE;
	int i;
	register caddr_t target = *addrp;

	if (xdrs->x_op != XDR_FREE)
		panic ("xdr_free_array: xdrs->x_op is not XDR_FREE.\n");

	/*
	 * Now we xdr_free each element of array.
	 * We only do this for big arguments with their own sub-procedure.
	 * For little things without an elproc, we just skip all this stuff.
	 */
	if (elproc) {
		count = *elemsp;
		if (*spacep < count)
			count = *spacep;
		for (i = 0; (i < count) && stat; i++) {
			stat = (*elproc)(xdrs, target /*, LASTUNSIGNED */);
			target += elsize;
		};
	};

	/*
	 * the array needs freeing
	 */
	mem_free(*addrp, *spacep * elsize);
	*addrp = NULL;
	*elemsp = *spacep = 0;

	return stat;
}

/*
 * xdr_get_array_elems -- code used by both in & out array --
 *    used to get all the elements of the array.
 * This routine handles packed arrays of characters.
 * NEEDSWORK: It should also handle packed arrays of short int's.
 */
bool_t
xdr_get_array_elems(xdrs, target, count, elsize, elproc)
	XDR *xdrs;
	caddr_t target;
	int count;
	u_int elsize;
	xdrproc_t elproc;
{
	bool_t stat;
	int i;

	if (elsize < sizeof(int)) {
		if (elproc != NULL || elsize!=1)
			panic ("xdr_get_array_elems: sending short things without packing.");
		/*
		 * If we're getting charcater arrays, use xdr_opaque,
		 * since that will handle packing.
		 */
		stat = xdr_opaque (xdrs, target, elsize * count);
		target += elsize * count;
	} else {   /* handle normal things */
		for (i = 0; (i < count) && stat; i++) {
			stat = (*elproc)(xdrs, target /*, LASTUNSIGNED */);
			target += elsize;
		};
	};
	return stat;
}

/*
 * in_array is not really just for in arrays, it's for arrays
 * that only have an "elems" pointer.
 */
bool_t
xdr_in_array(xdrs, addrp, elemsp, elsize, elproc)   /* actually, any fixed-length array */
	register XDR *xdrs;
	caddr_t *addrp;
	u_int *elemsp;
	u_int elsize;
	xdrproc_t elproc;
{
	register caddr_t target = *addrp;
	u_int count, i;
	bool_t stat=TRUE;
	caddr_t userp;
	bool_t needpop=FALSE;

	/*
	 * Handle freeing.
	 */
	if (xdrs->x_op == XDR_FREE)
		return xdr_free_array(xdrs, addrp, elemsp, elemsp, elsize, elproc);

	/*
	 * First get the size:  if in, it's the max size.  if out, it's
	 * the real size.
	 */
	if (!xdr_u_int(xdrs, elemsp))
		return FALSE;

	/*
	 * If deserializing, we may need to allocate data.
	 */
	if (target == NULL) {
		target = *addrp = mem_alloc(*elemsp * elsize);
		bzero (target, *elemsp * elsize);
	};

	/*
	 * Take care of the non-linear case.
	 * (Do this here because if c is 0, we need to
	 * have skipped the pointer.
	 */
	if (!XXDR_LINEAR(xdrs)) {
		if (!XDR_GETLONG(xdrs, &userp))
			return FALSE;
		needpop = TRUE;
		if (!XXDR_PUSH(xdrs, userp))
			return FALSE;
	};
	
	/*
	 * get the elements.
	 */
	stat = xdr_get_array_elems(xdrs, target, *elemsp, elsize, elproc);

	if (needpop)
		XXDR_POP(xdrs);

	return (stat);
}

/*
 * out_array is for arrays that have both an elems and space components
 * that need to be maintained.
 *
 * When this kind of array is an "out" parameter, it's type should be SPACEOUT.
 *
 * Unfortunately, these "out" arrays are also sometimes just "in" params.
 * This can be detected by the (very non-intuitive) null *addrp without
 * XXDR_SPACE being set.
 */
bool_t
xdr_out_array(xdrs, addrp, elemsp, spacep, elsize, elproc)
	register XDR *xdrs;
	caddr_t *addrp;
	u_int *elemsp, *spacep;
	u_int elsize;
	xdrproc_t elproc;
{
	register caddr_t target = *addrp;
	u_int count, i;
	bool_t stat=TRUE;
	caddr_t userp;
	bool_t needpop=FALSE;

	DODEBUG(printf ("xdr_out_array: op=%d, mode=%d\n", xdrs->x_op, XXDR_GETMODE(xdrs));)
	/*
	 * Handle freeing (same code for both in and out arrays).
	 */
	if (xdrs->x_op == XDR_FREE)
		return xdr_free_array(xdrs, addrp, elemsp, spacep, elsize, elproc);

	/*
	 * If we're in SPACE mode, we just allocate space for the data.
	 */
	if (XXDR_SPACE(xdrs)) {
		int space;

		XXDR_SKIP(xdrs, sizeof(u_int));  /* skip elems */
		if (!xdr_u_int(xdrs, spacep))
			return FALSE;
		space = *spacep * elsize;
		DODEBUG(printf ("xdr_out_array: %d space (spacep=%x)\n", space, spacep);)
		/*
		 * If deserializing, we may need to allocate data.
		 */
		if (target == NULL) {
			if (xdrs->x_op != XDR_DECODE)
				printf ("xdr_out_array: Warning.  Space without decode and null pointer.\n");
			if (space > 64*1024) {
				printf ("xdr_out_array: Warning.  Request for %x bytes in a message, operation aborted.\n", space);
				/* NEEDSWORK: should a log, not a printf */
				return FALSE;
			};
			target = *addrp = mem_alloc(space);
			bzero (target, space);
		};
		XXDR_SKIP(xdrs, sizeof(caddr_t));  /* skip pointer */
		return TRUE;
	};

	/*
	 * Get the real number of used elements.
	 */
	if (!xdr_u_int(xdrs, elemsp))
		return FALSE;
	XXDR_SKIP(xdrs, sizeof(u_int));   /* skip space */
	DODEBUG(printf ("xdr_out_array: %d elems\n", *elemsp);)

	/*
	 * If target is null, then we must be dealing with an "IN"
	 * array.  We should allocate space and set up spacep.
	 */
	if (target==NULL) {
		*spacep = *elemsp;
		target = *addrp = mem_alloc(*spacep * elsize);
		if (target==NULL) {
			u.u_error = ENOMEM;
			return FALSE;
		};
		bzero (target, *spacep * elsize);
	};

	/*
	 * Take care of the non-linear case.
	 * (Do this here because if *elemsp is 0, we need to
	 * have skipped the pointer.
	 */
	if (!XXDR_LINEAR(xdrs)) {
		if (!XDR_GETLONG(xdrs, &userp))
			return FALSE;
		needpop = TRUE;
		if (!XXDR_PUSH(xdrs, userp))
			return FALSE;
	};
	
	/*
	 * now we xdr each element of array, but only
	 * if we're handling big, hefty elements.
	 * With dainty characters, we handle them in one fell
	 * swoop.
	 */
	count = *elemsp;
	if (*spacep < count)
		count = *spacep;
	stat = xdr_get_array_elems(xdrs, target, count, elsize, elproc);

	if (needpop)
		XXDR_POP(xdrs);

	return (stat);
}

 
/*
 * NEEDSWORK: a short term hack.
 */
bool_t
xdr_struct_pathname_p(xdrs, pp)
        register XDR *xdrs;
        struct pathname **pp;            /* the pointer to work on */
{
	switch (xdrs->x_op) {
	case XDR_FREE:
		return TRUE;
	case XDR_DECODE:
		*pp = NULL;
		/* fall through */
	case XDR_ENCODE:
		XXDR_SKIP(xdrs, sizeof(struct pathname *));
		return TRUE;
	default:
		panic ("xdr_struct_pathname_p");
	};
}

bool_t
xdr_struct_flock_p()
{
	return FALSE;
}
 
bool_t
xdr_struct_fid_pp(xdrs, ppp)
        register XDR *xdrs;
        struct fid **ppp;            /* the pointer to work on */
{
	return FALSE;
#if 0
	int ufidpp;
	int err;

	/*
	 * Allocate space, if necessary
	 *
	 * This will be done on the caller side with XDR_ENCODE,
	 *  and the vpp_p should be some good value (where the caller
	 *  wants the result.
	 *
	 * It will also be done by the receiver side, in which case
	 *  vpp_p will point to a null value and so we will allocate
	 *  space.
	 */
	if (XXDR_SPACE(xdrs)) {
		if (*ppp == NULL) {
			*ppp=(struct vnode**)
				mem_alloc(sizeof(struct fid**));
		};
		**ppp = NULL;   /* zero the vpp */
		/*
		 * Make things right for xdr user by skipping the pointer.
		 */
		XXDR_SKIP(xdrs, sizeof(struct fid**));
		return TRUE;
	};

	/*
	 * We'll get here on to XDR_ENCODE the result vnode when
	 * returning values (OUT) on receiver side.  In that case
	 * we call vpptofh which returns the fh.  Then we'll be called
	 * with XDR_FREE, so we release the vnode.
	 *   Then the caller gets the results and calls us with
	 * XDR_DECODE to get the result.  We should not get called
	 * with XDR_FREE there, because the interface assumes that
	 * the caller provides all buffers and releases all references.
	 */
	switch (xdrs->x_op) {

	case XDR_FREE:
		if (**ppp) {
			freefid(**ppp);
			**ppp = NULL;
			mem_free (*ppp, sizeof(struct fid**));
					/* free the space for the pointer */
		};
		return TRUE;

	case XDR_DECODE:   /* decode the fid */
	case XDR_ENCODE:   /* send the fid out */
		/* This code will not work without XDR on both sides,
		 * because the fid is a variable length thing
		 * and the client has no idea of how much space to
		 * reserve.  -JSH 7-Feb-91
		 */		 
		return FALSE;
	};
	return FALSE;
#endif
}
 
bool_t
xdr_struct_page_p()
{
	return FALSE;
}
 
bool_t
xdr_struct_seg_p()
{
	return FALSE;
}
 
bool_t
xdr_struct_as_p()
{
	return FALSE;
}

bool_t
xdr_u_int_p()
{
	return FALSE;
}

bool_t
xdr_addr_t()
{
	return FALSE;
}

bool_t
xdr_char_p(xdrs, cpp)
	XDR *xdrs;
	char **cpp;
{
	bool_t stat;

	if (XXDR_LINEAR(xdrs))
		return xdr_string(xdrs, cpp, NEWVINT_MAXSTRLEN);

	switch (xdrs->x_op) {
	case XDR_FREE:
		if (*cpp) {
			mem_free(*cpp, NEWVINT_MAXSTRLEN);
			*cpp = NULL;
		};
		return (TRUE);
	case XDR_DECODE:
		*cpp = (caddr_t) mem_alloc(NEWVINT_MAXSTRLEN);
		return XXDR_GETSTRING(xdrs, cpp, NEWVINT_MAXSTRLEN);
	case XDR_ENCODE:
		return FALSE;
	default:
		panic ("xdr_char_p");
	}
}

