
/*
 * nfs_bypass.c
 * implements a bypass routine for nfs
 * (does not necessarilarly provide stateless semantics, though)
 *
 */



#ifdef NEWVINT

#include <sys/param.h>
#include <sys/user.h>
#include <sys/file.h>
#include <sys/uio.h>
#include <sys/ucred.h>
#include <sys/dnlc.h>   /* for NOCRED */
#include <sys/vfs.h>
#include <sys/vnode.h>
#include <sys/nvnode.h>

#include <netinet/in.h>   /* for rcp/svc.h */
#include <rpc/types.h>
#include <rpc/auth.h>
#include <rpc/auth_unix.h>
#include <rpc/xdr.h>
#include <rpc/svc.h>

#include <newvint/xxdr.h>
#include <netinet/in.h>
#include <nfs/nfs.h>
#include <nfs/nfs_clnt.h>

#include <sys/syslog.h>
#ifdef OUTOFKERNEL
#define log syslog
#endif

#if 0
#define DODEBUG(A) A
#else
#define DODEBUG(A)
#endif



#define PUBLIC_IN 1
#define PUBLIC_OUT 0








bool_t xdr_bypassargs(), xdr_bypassres(), xdr_bypassfree();
int rfs_bypass();

extern struct vnodeop_desc *uuid_to_desc();


int nfs_bug_bypass = 0;   /* hack this to show bypassed calls */


int
dummy_freefunc()
{
}

static void
setup_public(xp, xdrs, exi, vfsp, inout)
	struct nvop_xdr_public *xp;
	XDR *xdrs;
	struct exportinfo *exi;
	struct vfs *vfsp;
	int inout;
{
	xp->xdrs = xdrs;
	xp->freefunc = dummy_freefunc;
	xp->mode = (inout==PUBLIC_IN) ? xfer_in : xfer_out;

	xp->otw_to_vp = xdr_fhtovp;
	xp->vp_to_otw = xdr_vptofh;
	xp->vp_free = xdr_fhvpfree;
	xp->exi = exi;

	xp->otw_to_vpp = xdr_fhtovpp;
	xp->vpp_to_otw = xdr_vpptofh;
	xp->vpp_free = xdr_fhvppfree;
	xp->vfsp = vfsp;
}




#ifdef NFSSERVER

/*
 * rfs_handle_bypass is called from rfs_dispatch as soon as we
 * know we've got a bypass call.  This is necessary because
 * dispatch assumes some things about the structure of the
 * arguments that bypass routines don't agree with.
 *
 * Warning: NFS fakes up u.u_cred.  This code doesn't.
 * Of course, new routines shouldn't do anything so gross
 * as accessing a global variable.  What do you think the
 * cred parameter is for?
 */
int
rfs_handle_bypass(req, xprt)
	struct svc_req *req;
	SVCXPRT *xprt;
{
	struct nfsbypassstruct *args = NULL;
	int error=0;
	extern int rfssize;
	struct ucred *oldcr, *newcr;
	struct nvop_generic_args *ap;
	struct vnodeop_desc *descp;

	DODEBUG(printf ("rfs_handle_bypass called\n"));
	/*
	 * Allocate args struct and deserialize into it.
	 */
	args = (struct nfsbypassstruct *)rfsget();
	bzero((caddr_t)args, (u_int)rfssize);
	if ( ! SVC_GETARGS(xprt, xdr_bypassargs, args)) {
		svcerr_decode(xprt);
		error++;
		log (LOG_ERR, "rfs_handle_bypass: bad getargs\n");
		goto done;
	}

/*	svstat.reqs[RFS_BYPASS]++; */

	/*
	 * So here's the deal.  We're here, and all the arguments
	 * are ours.  But u.u_cred is still whoever started the server.
	 * Of course, file systems shouldn't look at the u structure,
	 * but what does Sun care?  New file systems shouldn't, though!
	 * But to make the ufs work, here we fill in the u.u_cred
	 * with whatever credentials were passed in with the operation.
	 */
	ap = args->ap;      /* find the arguments */
	descp = ap->a_desc;
	oldcr = u.u_cred;   /* remember the old creds */
	if (descp->vdesc_cred_offset == -1) {
		newcr = oldcr;   /* just use u.u_cred anyway */
		printf ("Warning: rfs_handle_bypass: bypass operation with no credentials.\n");
	} else {
		newcr = *XtOffsetTo(struct ucred**,
				    descp->vdesc_cred_offset, ap);
	};
	crhold(newcr);
	u.u_cred = newcr;

	/*
	 * Call service routine with arg struct and results struct
	 */
	rfs_bypass(args, args, NULL, req);

done:

	/*
	 * Serialize and send results struct
	 */
	if (!error)
		if (!svc_sendreply(xprt, xdr_bypassres, (caddr_t)args)) {
			log(LOG_ERR, "rfs_handle_bypass: bad sendreply\n");
			error++;
		}

	/*
	 * Free args/results struct
	 */
	if (args != NULL) {
		/*
		 * This doesn't do anything, so it's taken out.
		 * It's just here to remind us where it goes
		 * in the real nfs code.
		 *
		 * rfs_bypassfree(args);
		 */
		/*
		 * This should go in rfs_bypassfree, but there we've
		 * lost the xprt.
		 */
		if (!SVC_FREEARGS(xprt, xdr_bypassfree, (caddr_t)args)) {
			log(LOG_ERR, "rfs_handle_bypass: bad svc_freeargs\n");
			error++;
		};

		rfsput((struct rfsspace *)args);
	}
/*	svstat.nbadcalls += error; */
	
	/*
	 * Clean up this credentials mess.
	 */
	crfree (newcr);
	u.u_cred = oldcr;
}




/*
 * rfs_bypass called on the server site to take care of the
 * bypassed command.
 */
int   /* should be void */
rfs_bypass(args, res, exi, req)
	struct nfsbypassstruct *args, *res;
	struct exportinfo *exi;
	struct svc_req *req;
{
	int error;
	struct nvop_generic_args *ap = args->ap;

	if (nfs_bug_bypass)
		printf ("rfs_bypass: operation <%s> args=%x\n", ap->a_desc->vdesc_name, ap);

	error = VCALL(ap->a_un.a_vp, ap->a_desc->vdesc_offset, ap);

	if (nfs_bug_bypass)
		printf ("rfs_bypass: returns %d\n", error);

	res->result = puterrno(error);
}







/*
 * xdr_bypassargs called on each site to serialize the arguments.
 * On the client, this is called in CLNT_CALL in rfscall called from nfs_bypass.
 * On the server this is called indirectly through rfsdisptab in rfs_dispatch.
 *
 * NEEDSWORK: I hope we don't have to save/restore the x_public stuff.
 * (For both args and res).
 */
bool_t
xdr_bypassargs(xdrs, args)
	XDR *xdrs;
	struct nfsbypassstruct *args;
{
	struct vnodeop_desc *vdescp;
	int uuid;

	DODEBUG(printf ("xdr_bypassargs called, op=%d\n", xdrs->x_op));
	/*
	 * Send the arguments by setting the "in" flag.
	 */
	setup_public(&args->public, xdrs, NULL, NULL, PUBLIC_IN);
	xdrs->x_public = (caddr_t)&args->public;

	/*
	 * Another gross hack.  Since args&res are the same,
	 * we only want to free them once.
	 * Therefore this code ignores frees,
	 * leaving the call to xdr_bypassres to handle the task.
	 */
	if (xdrs->x_op==XDR_FREE)
		return TRUE;

	/*
	 * First we handle the uuid of this operation, so that
	 * the remote site knows what we're up to.
	 */
	switch (xdrs->x_op) {
	case XDR_ENCODE:
		vdescp = args->ap->a_desc;
		/* send the uuid */
		if (!xdr_int(xdrs, &vdescp->vdesc_uuid))
			return FALSE;
		break;
	case XDR_DECODE:
		/*
		 * Receive and decode the uuid.
		 * It gets stuffed in the structure by xdr_nvop_foo.
		 */
		if (!xdr_int(xdrs, &uuid))
			return FALSE;
		if (NULL==(vdescp = uuid_to_desc(uuid))) {
			u.u_error = EOPNOTSUPP;
			log (LOG_WARNING, "xdr_bypassargs: newvop uuid %d unrecognized.\n", uuid);
			return FALSE;
		};
		break;
	default:
		panic ("xdr_bypassargs");
	};

	/*
	 * To make this stuff work on both server and client,
	 * we have to put a double indirect pointer as the arg.
	 * Sigh.  But this way, rfs_dispatch works unhacked.
	 */
	return xdr_alt_reference(xdrs, &args->ap,
				    vdescp->vdesc_size, vdescp->vdesc_xdr);
}




/*
 * nfsbypassfree (should be rfsbypassfree) is called on the server
 * after the call has been completed.  It should free the memory
 * allocated for the xdr response.  It's called with the results
 * pointer.
 *
 * Fortunatly, the args and the res are actually the same stuff,
 * so this one free takes care of them all.
 */
int
rfs_bypassfree(res)
	struct nfsbypassstruct *res;
{
	log (LOG_DEBUG, "rfs_bypassfree called.\n");
	/*
	 * I think svc_sendreply takes care of freeing.
	 * If not, the bug will become apparent as the machine hangs
	 * after heavy use.  (Buyer beware.)
	 */
}
#endif




#if defined(NFSSERVER) || defined(NFSCLIENT)

/*
 * xdr_bypassargs called on each site to serialize the results.
 * On the client, this is called in CLNT_CALL in rfscall called from nfs_bypass.
 * On the server this is called indirectly through rfsdisptab in rfs_dispatch.
 */
bool_t
xdr_bypassres(xdrs, res)
	XDR *xdrs;
	struct nfsbypassstruct *res;
{
	struct vnodeop_desc *vdescp;
	struct vnode **vp1p;
	struct vfs *vfsp;

	DODEBUG(printf ("xdr_bypassres called, op=%d, ap=%x, result=%d\n",
			xdrs->x_op, res->ap, res->result));

	/*
	 * Send the arguments by clearing the "in" flag.
	 * (First figure out the vfsp.  We sure hope that the
	 * arguments have left a vnode pointer in there where we expect
	 * it.)
	 */
	vp1p=XtOffsetTo(struct vnode**,
			res->ap->a_desc->vdesc_vp1_offset, res->ap);
	if (vp1p!=NULL)
		vfsp = (*vp1p)->v_vfsp;
	else panic ("xdr_bypassres: vp1p has no vfsp.");
	DODEBUG(printf ("xdr_bypassres: stolen vfsp is %x\n", vfsp));
	setup_public(&res->public, xdrs, res->public.exi, vfsp, PUBLIC_OUT);
	xdrs->x_public = (caddr_t)&res->public;

	DODEBUG(printf ("xbr: a, error=%d\n", res->result));
	/*
	 * First send/receive the error code.
	 */
	if (!xdr_int(xdrs, &res->result))
		return FALSE;
	if (res->result) {
		log (LOG_WARNING, "xdr_bypassres: error code %d received, result args not sent\n", res->result);
		return TRUE;   /* error code implies no rest of stuff */
	};


	DODEBUG(printf ("xbr: b\n"));
	/*
	 * To make this stuff work on both server and client,
	 * we have to put a double indirect pointer as the arg.
	 * Sigh.  But this way, rfs_dispatch works unhacked.
	 */

	vdescp = res->ap->a_desc;
	return xdr_alt_reference(xdrs, &res->ap,
				    vdescp->vdesc_size, vdescp->vdesc_xdr);
}



bool_t
xdr_bypassfree(xdrs, res)
	XDR *xdrs;
	struct nfsbypassstruct *res;
{
	struct vnodeop_desc *vdescp;

	if (xdrs->x_op!=XDR_FREE)
		panic("xdr_bypassfree: x_op!=XDR_FREE\n");

	/*
	 * The public section should already be set up, so
	 * we leave it alone.
	 *
	 * setup_public(&res->public, xdrs, NULL, NULL, PUBLIC_OUT);
	 */
	xdrs->x_public = (caddr_t)&res->public;

	vdescp = res->ap->a_desc;
	return xdr_alt_reference(xdrs, &res->ap,
				    vdescp->vdesc_size, vdescp->vdesc_xdr);
}

#endif



#ifdef NFSCLIENT

/*
 * nfs_bypass is called on the client side to send a bypassed call.
 * (Pretty simple, ain't it?)
 */
int
nfs_bypass(ap)
	struct nvop_generic_args *ap;
{
        XDR xdrs;
	struct vnode *vp;
	int error;
	struct nfsbypassstruct args, res;
	extern int nfs_remove();
	struct vnodeop_desc *descp = ap->a_desc;

	if (ap->a_desc->vdesc_flags & VDESC_INVPP)
		panic ("nfs_bypass doesn't handle invpp.\n");

	if (nfs_bug_bypass)
		printf ("nfs_bypass called for op <%s>\n",
			ap->a_desc->vdesc_name);

	/*
	 * Set up the structures.
	 */
	args.ap = res.ap = ap;

	/*
	 * Sigh.  We must get u.u_cred, since there's no consistent
	 * place for creds in the call.  (Hint for new protocol designers!)
	 */
	error = rfscall(vtomi(ap->a_un.a_vp), RFS_BYPASS, xdr_bypassargs,
			&args, xdr_bypassres, &res, u.u_cred);
	DODEBUG(printf ("rfscall returns to nfs_bypass\n"));

	if (!error)
		error = res.result;

	/*
	 * If a vpp is returned, because of the kludginess
	 * of this whole NFS overloading stuff, there's no
	 * way to set the vnode type when the vpp is created.
	 * To get around this, if a vpp is returned,
	 * after the operation has completed, we do a getattr
	 * on the file and set its type correctly.
	 * -JSH 8-Feb-91
	 */
	if (!error && descp->vdesc_vpp_offset != VDESC_NO_OFFSET) {
		struct vnode ***vppp=XtOffsetTo(struct vnode***,
				 descp->vdesc_vpp_offset,ap);
		struct ucred *cred =
			XtOffsetTo(struct ucred*, descp->vdesc_cred_offset,ap);
		struct vattr va;

		DODEBUG(printf ("nfs_bypass: fixing vnode type.\n"));
		error = nfs_getattr_otw(**vppp, &va, cred);
		DODEBUG(printf ("nfs_getattr_otw: error=%d, type=%d\n", error, va.va_type));
		if (!error) {   /* set the type and return */
			(**vppp)->v_type = va.va_type;
		};
	};

	DODEBUG(printf ("nfs_bypass, returning %d\n", error));
	return error;
}

#endif



#else   /* ifdef NEWVINT */
This works only under the new interface.
#endif  /* def NEWVINT */




