 
/*
 * xdr_user.h, XDR implementation for user->kernel space.
 *
 * The same, but different.
 *
 * x_base is base of stack of pointers.
 * x_private is current pointer.
 * x_handy is size of stack.
 *
 * xdruser_{push,pop} handle pushing and poping past pointers.
 * Things aren't so "serial" any more.
 *
 * NEEDSWORK: Error handling.
 *
 */
 
#ifdef KERNEL
#include <sys/param.h>
#endif
 
#include <rpc/types.h>
#include <rpc/xdr.h>
#include <netinet/in.h>
#include <sys/syslog.h>


#if 0
#define DODEBUG(A) A
#else
#define DODEBUG(A)
#endif

 
#define X_USERPRIVATE(XDRS) ((caddr_t*)XDRS->x_private)
#define X_USERBASE(XDRS) ((caddr_t*)XDRS->x_base)


struct xdr_ops *xdruser_ops;
void xdruser_fill_ops();


/*
 * The procedure xdruser_create initializes a stream descriptor for a
 * memory buffer.
 */
void
xdruser_create(xdrs, addr, buf, size, op)
        register XDR *xdrs;
        caddr_t addr;
        u_int size;
        enum xdr_op op;
{
        xdrs->x_op = op;
	if (!xdruser_ops)
		xdruser_fill_ops();
        xdrs->x_ops = xdruser_ops;
	xdrs->x_handy = size;
	X_USERPRIVATE(xdrs) = X_USERBASE(xdrs) = (caddr_t*)buf;
/*	X_USERPRIVATE(xdrs) = X_USERBASE(xdrs) = 
 *		(caddr_t*)mem_alloc(sizeof(caddr_t)*xdrs->x_handy);
 */
	*X_USERPRIVATE(xdrs) = addr;
	xdrs->x_public = 0;
}
 
/*ARGSUSED*/
static void
xdruser_destroy(xdrs)
        XDR *xdrs;
{
/*	mem_free(X_USERBASE(xdrs), sizeof(caddr_t)*xdrs->x_handy); */
}
 
static bool_t
xdruser_getlong(xdrs, lp)
        register XDR *xdrs;
        long *lp;
{
	int error = copyin(*X_USERPRIVATE(xdrs), lp, sizeof(long));
	*X_USERPRIVATE(xdrs) += sizeof(long);
        return (error==0);
}
 
static bool_t
xdruser_putlong(xdrs, lp)
        register XDR *xdrs;
        long *lp;
{
 	int error = copyout(lp, *X_USERPRIVATE(xdrs), sizeof(long));
	*X_USERPRIVATE(xdrs) += sizeof(long);
        return (error==0);
}
 
static bool_t
xdruser_getbytes(xdrs, addr, len)
        register XDR *xdrs;
        caddr_t addr;
        register u_int len;
{
	int error = copyin(*X_USERPRIVATE(xdrs), addr, len);
	*X_USERPRIVATE(xdrs) += len;
        return (error==0);
}
 
static bool_t
xdruser_putbytes(xdrs, addr, len)
        register XDR *xdrs;
        caddr_t addr;
        register u_int len;
{
	int error = copyout(addr, *X_USERPRIVATE(xdrs), len);

	DODEBUG(printf ("xdruser_putbyptes from %x to %x len %d, error=%d\n", addr, *X_USERPRIVATE(xdrs), len, error));
	*X_USERPRIVATE(xdrs) += len;
        return (error==0);
}
 
static u_int
xdruser_getpos(xdrs)
        register XDR *xdrs;
{
	return (u_int)(*X_USERPRIVATE(xdrs));
}
 
static bool_t
xdruser_setpos(xdrs, pos)
        register XDR *xdrs;
        caddr_t pos;
{
	*X_USERPRIVATE(xdrs) = pos;
        return TRUE;
}
 
/*ARGSUSED*/
static long *
xdruser_inline(xdrs, len)
        register XDR *xdrs;
        int len;
{
        return FALSE;
}
 
bool_t
xdruser_push(xdrs, addr)
        register XDR *xdrs;
        caddr_t addr;
{
	if (X_USERPRIVATE(xdrs)-X_USERBASE(xdrs) >= xdrs->x_handy) {
		log (LOG_WARNING, "xdruser_push: stack overflow.\n");
		return FALSE;
	};
	X_USERPRIVATE(xdrs)++;
	*X_USERPRIVATE(xdrs) = addr;
	return TRUE;
}
 
bool_t
xdruser_pop(xdrs)
        register XDR *xdrs;
{
	if (X_USERPRIVATE(xdrs)-X_USERBASE(xdrs) <= 0) {
		log (LOG_WARNING, "xdruser_pop: stack underflow.\n");
		return FALSE;
	};
	X_USERPRIVATE(xdrs)--;
	return TRUE;
}

bool_t
xdruser_skip(xdrs, s)
	register XDR *xdrs;
	int s;
{
	*X_USERPRIVATE(xdrs) += s;
	return TRUE;
}


 
bool_t
xdruser_getstring(xdrs, cpp, size)
        register XDR *xdrs;
	char **cpp;
	int size;
{
	char *ucp;
	int realsize, error;

	if (!XDR_GETLONG(xdrs, &ucp))
		return FALSE;
	if (*cpp == NULL) {   /* we assume that memory is allocated */
		panic ("xdruser_getstring: string is null.");
	};
	error = copyinstr(ucp, *cpp, size, &realsize);
	return (error==0);
}
 

void
xdruser_fill_ops()
{
        if (xdruser_ops == NULL) {
		xdruser_ops =
			(struct xdr_ops*)mem_alloc(sizeof(struct xdr_ops));
                xdruser_ops->x_getlong = xdruser_getlong;
                xdruser_ops->x_putlong = xdruser_putlong;
                xdruser_ops->x_getbytes = xdruser_getbytes;
                xdruser_ops->x_putbytes = xdruser_putbytes;
                xdruser_ops->x_getpostn = xdruser_getpos;
                xdruser_ops->x_setpostn = xdruser_setpos;
                xdruser_ops->x_inline = xdruser_inline;
                xdruser_ops->x_destroy = xdruser_destroy;
        }
}


