
/*
 * svcm.h
 *
 *	@(#)svcm.h	1.8 (UCLA Ficus Project) 1/9/95
 *
 * Copyright (c) 1993 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef _i405_svcm_svcm_h
#define _i405_svcm_svcm_h


struct svcm_lock {
 	u_int wanted:1,			/* someone wants to lock */
		unstable:1;		/* page owership list is being changed,
					 * (debugging only) */
	int lockcount;			/* count of recursive holdings */
	long locker;			/* who owns the lock */
	int refcnt;			/* reference count on this object */
};
#define svcm_lock_hold(lock) { (lock)->refcnt++; }
#define svcm_lock_rele(lock) { \
		if (--(lock)->refcnt <= 0) { \
			svcm_lock_destroy((lock)); \
		}; \
	}			       

/*
 * The following macros aid in checking locks
 * and are all conditional on SVCM_LOCK_CHECK.
 */
#ifdef SVCM_LOCK_CHECK
#define SVCM_LOCK_ASSERT(assertion) ASSERT(assertion)
#define SVCM_LOCK_CHECK_DECL int oldlockcount;
#define SVCM_LOCK_CHECK_CHECKPOINT(lock) do { oldlockcount = (lock)->lockcount; } while (0)
#define SVCM_LOCK_CHECK_VERIFY(lock) do { if (oldlockcount != (lock)->lockcount) svcm_lock_check_fail(__FILE__,__LINE__); } while (0)
#else /* ! LOCK_CHECK */
#define SVCM_LOCK_ASSERT(assertion)
#define SVCM_LOCK_CHECK_DECL
#define SVCM_LOCK_CHECK_CHECKPOINT(lock)
#define SVCM_LOCK_CHECK_VERIFY(lock)
#endif /* LOCK_CHECK */


/*
 * The following macros aid the stack-friendly-ops-only
 * version.
 */
#ifdef SVCM_SFO
#define SVCM_SFO_NOT_CONVERTED assfail("svcm_sfo_not_converted", __FILE__, __LINE__)
#else /* ! SVCM_SFO */
#define SVCM_SFO_NOT_CONVERTED  /* do nothing */
#endif /* SVCM_SFO */


enum svcm_obj_classes {
	SVCM_CLASS_DATA_PAGES,
	SVCM_CLASS_DATA_LOCKS,
	SVCM_CLASS_DNLC,
	SVCM_CLASS_ATTRIBUTES,
	/* insert new classes before here */
	SVCM_OBJ_CLASSES   /* not a class, but one beyond the last class */
};

struct svcm_class_ops {
	int (*svcm_init)();
	int (*svcm_register)();
};


struct svcm_name {
	struct svcm_name *prev, *next;  /* in the name hash */
	struct vnode *vp;		/* the name (vnode part) */
	struct svcm_lock *lock;		/* who holds it currently */
	int refcnt;
	int flags;
	caddr_t classes[SVCM_OBJ_CLASSES];
};
#define svcm_name_hold(name) { (name)->refcnt++; }
#define SVCM_NAME_DIEING 0x01
#define SVCM_NAME_REBORN 0x02

enum svcm_status {
	SVCM_STATUS_UNCACHED,     /* I don't cache; don't care about others. */
	SVCM_STATUS_NONCACHABLE,  /* I don't cache; others can't cache now. */
	SVCM_STATUS_SHARED,	  /* I cache (want callback); others can too. */
	SVCM_STATUS_EXCLUSIVE,	  /* I alone can cache and want callback. */
	SVCM_STATUS_WATCH,	  /* I don't cache, but I want all callbacks. */
};



extern struct svcm_class_ops svcm_class_ops[SVCM_OBJ_CLASSES];

#define SVCM_INIT(NAME, OWN_VP, OBJ_CLASS, CRED) \
	((*(svcm_class_ops[(int)(OBJ_CLASS)]).svcm_init)(NAME, OWN_VP, OBJ_CLASS, CRED))
#define SVCM_REGISTER(NAME, OWN_VP, OBJ_CLASS, OBJ_NAME_LEN, OBJ_NAME, STATUS, CRED) \
	((*(svcm_class_ops[(int)(OBJ_CLASS)]).svcm_register)(NAME, OWN_VP, \
		      OBJ_CLASS, OBJ_NAME_LEN, OBJ_NAME, STATUS, CRED))




/*
 * Pull in function declarations.
 */
#if defined(KERNEL) && !defined(STANDALONE_OOK)
#include <ficus/kernel_prototypes.h>
#endif
#include <i405/SVCM/svcm_main.h>
#include <i405/SVCM/svcm_byterange.h>
#include <i405/SVCM/svcm_namedobj.h>
#include <i405/SVCM/svcm_vm.h>
#include <i405/SVCM/svcm_subr.h>
#include <i405/SVCM/svcm_debug.h>
#include <i405/SVCM/svcm_interface.h>

#endif /* _i405_svcm_svcm_h */
