
/*
 * svcm_byterange.h
 *
 *	@(#)svcm_byterange.h	1.6 (UCLA Ficus Project) 9/26/94
 *
 * Copyright (c) 1992 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */


/*
 * NOTE:
 * The file is automatically maintained via the program updateheader.
 * While additions to this file will be maintained, there is no need to
 * edit the function prototypes for functions in i405_svcm_byterange.c
 * --- they will be automatically updated.
 */

#ifndef __i405_svcm_svcm_byterange_h
#define __i405_svcm_svcm_byterange_h

#include <sys/limits.h>
#define MAX_SEG_END LONG_MAX
#define MAX_SEG_LEN ((u_long)MAX_SEG_END + (u_long)1)

/*
 * B_PAGEFLIP should really follow the other B_ flags
 * in buf.h.
 */
#define B_PAGEFLIP	0x08000000	/* ove this page to the stack bottom */

struct svcm_byterange_name {
	off_t offset;
	u_long length;	/* should be unsigned version of off_t */
};


/*
 * Segments are the unit to keep track of things.
 * Segments are doulbly linked lists, NULL terminated at the ends.
#ifndef CIRCLE
 * NEEDSWORK: Circularly linked lists would be simplier.
#endif
 * There is always at least one segment present.
 */
struct svcm_seg {
	struct svcm_seg *prev, *next;
	off_t beg, end;
	struct vnode *own_vp;
	enum svcm_status status;
};

/*
 * A macro to compare segment ownership.
 */
#define SEG_MATCHES(SEG,VP) ((SEG)->own_vp == (VP))


/*
 * Debugging options:
 */

/*
 * JOHNH_SVCM_BYTERANGE_VERIFY:
 * Check the segment list for internal consistency each time it changes.
 */
/* #define JOHNH_SVCM_BYTERANGE_VERIFY */

/*
 * JOHNH_DATA_PAGE_LIST_VERIFY:
 * verify vp's pages vs. the segement list.
 * Very expsenive.
 */

/*
 * SVCM_BYTERANGE_KEEP_STATS:
 * Maintain statistics about memory usage and fast-path hits.
 */
#ifdef JOHNH4C
#define SVCM_BYTERANGE_KEEP_STATS
#endif /* JOHNH4C */



#include <i405/SVCM/svcm_byterange__ah.h>

#endif /* __i405_svcm_svcm_byterange_h */
