
/*-
 * svcm_debug.c
 *
 *	@(#)svcm_debug.c	1.3 (UCLA Ficus Project) 7/17/95
 *
 * Copyright (c) 1993 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */


#ifdef SVCM_TWO

#include <sys/param.h>
#include <sys/time.h>
#include <sys/systm.h>   /* for NOMEMWAIT() */
#include <sys/vnode.h>
#include <i405/i405.h>
#include <i405/SVCM/svcm.h>
#include <sys/errno.h>
#include <sys/buf.h>   /* for B_INVAL */
#include <sys/debug.h>  /* for ASSERT */
#include <vm/page.h>



/**********************************************************************
 *
 * PANIC SUPPORT
 *
 */

static int svcm_do_panic;

void
svcm_debug_panic(s)
	char *s;
{
	if (svcm_do_panic > 0)   /* avoid recursive panics */
		return;
	svcm_do_panic++;
	/* A debugger can change svcm_do_panic back to zero here. */
	if (svcm_do_panic > 0) {
		panic(s);
	};
}


/***********************************************************************
 *
 * BYTERANGE DEBUGGING CODE
 *
 */


#if 0
static void
svcm_dump_pages(name)   /* debug routine */
	struct svcm_name *name;
{
	struct svcm_seg *segi = name->seg;
	int c;

	c = '[';
	do {
		printf ("%c%x-%x:%x", c, segi->beg, segi->end, segi->ownvp);
		c = ',';
		segi = segi->next;
	} while (segi);
	printf ("]\n");
}
#endif


#ifdef JOHNH_SVCM_BYTERANGE_VERIFY
static void
svcm_verify_breakhere()
{
}

#define ASSERTF(EX,FLAG) if (!(EX)) { (FLAG)=1; printf("ASSERTION failed: %s, file: %s, line: %d\n", "EX", __FILE__, __LINE__); }
void
svcm_verify_seglist(base_seg)   /* debug routine */
	struct svcm_seg *base_seg;
{
	struct svcm_seg *segi = base_seg;
	int fail = 0;

	/*
	 * seglist must be
	 *   (1) complete (all bytes covered)
	 *   (2) properly linked
	 *   (3) uncompressable (every adjacent seg has different owner/status)
	 */

	ASSERTF(segi->prev == NULL, fail);			 /* (2) */
	ASSERTF(segi->beg == 0, fail);				 /* (1) */
	while (segi->next) {
		ASSERTF(segi->end+1 == segi->next->beg, fail);	 /* (1) */
		ASSERTF(segi == segi->next->prev, fail);	 /* (2) */
		ASSERTF(!(SEG_MATCHES(segi, segi->next->own_vp) && segi->status == segi->next->status), fail); /* (3) */
		segi = segi->next;
	};
	/* redundant with loop: ASSERTF(segi->next == NULL, fail);  (2) */
	ASSERTF(segi->end == MAX_SEG_END, fail);		 /* (1) */
	if (fail)
		svcm_verify_breakhere();
	if (fail) {
		static int already_been_here = 0;
		if (already_been_here++)
			return;
		panic("svcm_verify_seglist");
	};
}
#endif /* JOHNH_SVCM_BYTERANGE_VERIFY */



#ifdef JOHNH_DATA_PAGE_LIST_VERIFY
/*-
 *--------------------------------------------------------------------------
 *
 * svcm_verify_page_ownership -- verify that (offset,len) in name is owned by vp
 *
 * DESCRIPTION:
 *
 * RETURN VALUES:
 *      Returns zero/non-zero "if verified".
 *
 *--------------------------------------------------------------------------
 */
int
svcm_verify_page_ownership(own_vp, offset, len, name)
	struct vnode *own_vp;
	int offset, len;
	struct svcm_name *name;
{
	struct svcm_seg *base_seg = (struct svcm_seg *)name->classes[SVCM_CLASS_DATA_PAGES];
	struct svcm_seg *segi;
	off_t beg, end;

	if (!base_seg)
		return 0;

	beg = offset;
	end = offset + (len - 1);
	for (segi = base_seg;;) {
		if (beg >= segi->beg && beg <= segi->end)
			break;
		segi = segi->next;
		if (!segi)
			panic ("svcm_verify_page_ownership: seg (beg) not found.");
	};
	for (;;) {
		if (end <= segi->end)   /* we're done */
			break;
		if (! (SEG_MATCHES(segi, own_vp) && segi->status == SVCM_STATUS_EXCLUSIVE))
			return 0;
		segi = segi->next;
		if (!segi)
			panic ("svcm_verify_page_ownership: seg (end) not found.");
	};
	if (end >= segi->beg) {   /* get the last bit of the last seg */
		if (! (SEG_MATCHES(segi, own_vp) && segi->status == SVCM_STATUS_EXCLUSIVE))
			return 0;
	};

	return 1;
}




/*-
 *--------------------------------------------------------------------------
 *
 * svcm_verify_vp_page_ownership
 *
 * DESCRIPTION:
 *	Scan vp's page list and verify that each page is owned by vp
 *	in it's name.
 *
 *	Assumes that we're protected by our svcm lock.
 *
 * RETURN VALUES:
 *      None.  Panics on error.
 *
 *--------------------------------------------------------------------------
 */
void
svcm_verify_vp_page_ownership(vp, name)
	struct vnode *vp;
	struct svcm_name *name;
{
	struct page *pp;

	/* We only do files (UFS VDIR's get pages !) */
	if (vp->v_type != VREG)
		return;

	for (pp = vp->v_pages; pp;) {
		if (!svcm_verify_page_ownership(vp, pp->p_offset, PAGESIZE, name))
			svcm_debug_panic("svcm_verify_vp_page_ownership");
		pp = pp->p_vpnext;
		if (pp == vp->v_pages)
			break;
	};
}


/*-
 *--------------------------------------------------------------------------
 *
 * svcm_verify_seglist_data_pages
 *
 * DESCRIPTION:
 *	Verify (for all vnodes mentioned in the seglist)
 *	that they only have pages legal to them.
 *
 * RETURN VALUES:
 *      None.  Panics on error.
 *
 *--------------------------------------------------------------------------
 */
void
svcm_verify_seglist_data_pages(base_seg)
	struct svcm_seg *base_seg;
{
	/* not yet implemented */
}

#endif /* JOHNH_DATA_PAGE_LIST_VERIFY */


#endif /* SVCM_TWO */


