
/*-
 * svcm_namedobj.c
 *
 *	$Id: svcm_namedobj.c,v 1.12 1997/04/25 05:26:52 johnh Exp $ (UCLA Ficus Project) $Date: 1997/04/25 05:26:52 $
 *
 * Copyright (c) 1993 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */



#ifdef SVCM_TWO

#define USE_SVCM_DNLC

#include <sys/param.h>
#include <sys/time.h>
#include <sys/systm.h>   /* for NOMEMWAIT() */
#include <sys/vnode.h>
#include <i405/i405.h>
#include <i405/SVCM/svcm.h>
#include <sys/debug.h>
#include <ficus/ficus_subr.h>   /* for logging */



/**********************************************************************
 *
 * STATIC FUNCTION DELCARATIONS (sigh)
 *
 */

struct namelist_traversal_context;

static struct namelist * namelist_alloc __P((void));
static void namelist_free __P((struct namelist *namelist));
static void namelist_list __P((struct namelist *namelist, struct vnode *own_vp, int namedobj_len, struct svcm_namedobj_name *namedobj, enum svcm_status status));
static int namelist_unlist __P((struct namelist *namelist, struct namelist *elem, struct svcm_name *name, enum svcm_obj_classes obj_class, int namedobj_len, struct svcm_namedobj_name *namedobj, int flags, struct ucred *cred));
static int name_match __P((int o1_len, struct svcm_namedobj_name *o1, int o2_len, struct svcm_namedobj_name *o2));



/**********************************************************************
 *
 * NAMELIST STATISTICS
 *
 */

#ifdef JOHNH4C
#define SVCM_NAMEDOBJ_KEEP_STATS
#endif /* JOHNH4C */

struct svcm_namedobj_stats {
	/* Traversal statistics. */
	int sns_minimum_traversals;
	int sns_real_traversals;
	int sns_traversal_hop;
	/* Allocation statistics. */
	int sns_namelist_alloc;
	int sns_namelist_free;
	/* Fast path information. */
	int sns_register_fastpath_hit;
	int sns_register_fastpath_miss;
	int sns_svcm_enter_fastpath_hit;
	int sns_svcm_enter_fastpath_miss;
};

struct svcm_namedobj_stats svcm_namedobj_stats[SVCM_OBJ_CLASSES];

#ifdef SVCM_NAMEDOBJ_KEEP_STATS
#define SVCM_NAMEDOBJ_STATS_INC(CLASS,FIELD) ((svcm_namedobj_stats[CLASS].FIELD)++)
#define SVCM_NAMEDOBJ_STATS_ADD(CLASS,FIELD,COUNT) (svcm_namedobj_stats[CLASS].FIELD += (COUNT))
#else /* ! SVCM_KEEP_STATS */
#define SVCM_NAMEDOBJ_STATS_INC(CLASS,FIELD)
#define SVCM_NAMEDOBJ_STATS_ADD(CLASS,FIELD,COUNT)
#endif /* SVCM_KEEP_STATS */





/**********************************************************************
 *
 * NAMELIST MEMORY ALLOCATION
 *
 */

/*
 * Internal namelist data structures.
 */

struct namelist_header {
        int traversal_serial;
};

struct namelist_traversal_context {
	int traversal_serial;
	struct namelist *traversal;
};


struct namelist_elem {
	struct vnode *own_vp;
	enum svcm_status status;
	int name_len;
	struct svcm_namedobj_name name;
};

struct namelist {
	struct namelist *prev, *next;
	/* header vs. elem is distinguished by context. */
	union {
		struct namelist_header header;
		struct namelist_elem elem;
	} un;
};


static struct namelist *namelist_base = NULL;
#define SVCM_NAMELIST_INC 0x10

static struct namelist *
namelist_alloc()
{
	struct namelist *namelist =
		(struct namelist*)new_kmem_fast_zalloc(&namelist_base,
				   sizeof(struct namelist), SVCM_NAMELIST_INC, KMEM_SLEEP);
	SVCM_NAMEDOBJ_STATS_ADD(0, sns_namelist_alloc, SVCM_NAMELIST_INC);
	return namelist;
}

static void
namelist_free(namelist)
	struct namelist *namelist;
{
	SVCM_NAMEDOBJ_STATS_INC(0, sns_namelist_free);
	kmem_fast_free ((caddr_t*)&namelist_base, (caddr_t)namelist);
}



/**********************************************************************
 *
 * NAMELIST MANIUPLATION
 *
 */

/*-
 *--------------------------------------------------------------------------
 *
 * svcm_namedobj_init -- a generic namedobj initializer
 *
 * DESCRIPTION:
 *	Build a dummy node for a namedobj list.
 *
 *	For now, we keep things on a simple, doubly linked, circular list.
 *	NEEDSWORK: Consider fancier data structures as the list length
 *	grows.
 *
 * RETURN VALUES:
 *      Returns an errno value.
 *
 *--------------------------------------------------------------------------
 */
int
svcm_namedobj_init(name, own_vp, obj_class, cred)
	struct svcm_name *name;
	struct vnode *own_vp;
	int obj_class;
	struct ucred *cred;
{
	struct namelist **namelist_p =
		(struct namelist **)&name->classes[obj_class];
	struct namelist *header = namelist_alloc();

	if (*namelist_p)
		return 0;

	header->next = header->prev = header;
	header->un.header.traversal_serial = 0;

	*namelist_p = header;

	return 0;
}



/*-
 *--------------------------------------------------------------------------
 *
 * namelist_traverse_init -- set up a namelist for a traversal
 *
 * DESCRIPTION:
 *	Set up a namelist for a traversal.
 *
 * RETURN VALUES:
 *      None.
 *
 *--------------------------------------------------------------------------
 */
#define namelist_traverse_init(NAMELIST,CONTEXT) \
	(CONTEXT)->traversal_serial = (NAMELIST)->un.header.traversal_serial; \
	(CONTEXT)->traversal = (NAMELIST);

/*-
 *--------------------------------------------------------------------------
 *
 * namelist_traversal_valid -- check the if a traversal should be aborted
 *
 * DESCRIPTION:
 *	Set up a namelist for a traversal.
 *
 * RETURN VALUES:
 *      None.
 *
 *--------------------------------------------------------------------------
 */
#define namelist_traversal_valid(NAMELIST,CONTEXT) ((NAMELIST)->un.header.traversal_serial == (CONTEXT)->traversal_serial)



/*-
 *--------------------------------------------------------------------------
 *
 * namelist_traversal_void -- void any in-progress traversals
 *
 * DESCRIPTION:
 *
 * RETURN VALUES:
 *      None.
 *
 *--------------------------------------------------------------------------
 */
static int namelist_serials;
#define namelist_traversal_void(NAMELIST) \
	(NAMELIST)->un.header.traversal_serial = ++namelist_serials;



/*-
 *--------------------------------------------------------------------------
 *
 * namelist_traverse -- traverse a name list, iterating over all elements
 *
 * DESCRIPTION:
 *	Traverse a name list, iterating over all elements.
 *
 *	Abort if the traversal becomes invalid.
 *
 *	This function has been inlined for speed.
 *
 * RETURN VALUES:
 *      The next namelist element, or null when all elements have been read.
 *
 *--------------------------------------------------------------------------
 */
/* If good and not done, advance and return the next element, otherwise return null */
#define namelist_traverse(NAMELIST,CONTEXT) \
	(namelist_traversal_valid((NAMELIST),(CONTEXT)) && (CONTEXT)->traversal->next != (NAMELIST) ? ((CONTEXT)->traversal = (CONTEXT)->traversal->next) : NULL)


/*-
 *--------------------------------------------------------------------------
 *
 * namelist_list -- add an element to the list
 *
 * DESCRIPTION:
 *	As a side effect all list traversals are reset.
 *
 * RETURN VALUES:
 *      None.
 *
 *--------------------------------------------------------------------------
 */
static void
namelist_list(namelist, own_vp, namedobj_len, namedobj, status)
	struct namelist *namelist;	     /* INOUT: list to which to add */
	struct vnode *own_vp;		     /* IN */
	int namedobj_len;		     /* IN */
	struct svcm_namedobj_name *namedobj; /* IN: cache subobject item */
	enum svcm_status status;	/* IN: desired status */
{
	struct namelist *elem = namelist_alloc();

	/*
	 * Setup new elem.
	 */
	elem->un.elem.own_vp = own_vp;
	elem->un.elem.name_len = namedobj_len;
	bcopy(namedobj, &elem->un.elem.name, namedobj_len);
	elem->un.elem.status = status;

	/*
	 * Insert at list head.
	 */
	elem->next = namelist->next;
	elem->prev = namelist;
	namelist->next = elem;
	elem->next->prev = elem;

	/* Traversals are no longer valid. */
	namelist_traversal_void(namelist);
}

/*
 * Flags for namelist_unlist:
 */
#define DO_UNLIST_WATCH 1

/*-
 *--------------------------------------------------------------------------
 *
 * namelist_unlist -- remove an element from the list
 *
 * DESCRIPTION:
 *	Do callbacks on elem because of conflicting request on namedobj.
 *
 * RETURN VALUES:
 *      An errno code.
 *
 *--------------------------------------------------------------------------
 */
static int
namelist_unlist(namelist, elem, name, obj_class, namedobj_len, namedobj, flags, cred)
	struct namelist *namelist, *elem;
	struct svcm_name *name;		/* IN: stack name */
	enum svcm_obj_classes obj_class;/* IN: cache subobject name */
	int namedobj_len;			/* IN: length of obj_name */
	struct svcm_namedobj_name *namedobj; /* IN: cache subobject item */
	int flags;			/* IN */
	struct ucred *cred;		/* IN */
{
	USES_NVOP_CACHE_CALLBACK;
	int error = 0;

	/*
	 * First callback the holder of the cache.
	 */
	if (elem->un.elem.status == SVCM_STATUS_WATCH) {
		error = NVOP_CACHE_CALLBACK(elem->un.elem.own_vp, name, obj_class,
				    namedobj_len, (void*)namedobj,
				    cred);
	} else {
		error = NVOP_CACHE_CALLBACK(elem->un.elem.own_vp, name, obj_class,
				    elem->un.elem.name_len,
				    (void*)&elem->un.elem.name,
				    cred);
	};
	if (error)
		log(LOG_PRI(LOG_FICUS, LOG_ERR), "namelist_unlist: callback failed with error %d.\n", error);

	/*
	 * Now remove the elem from the list
	 * and invalidate traversals.
	 */
	if (elem->un.elem.status != SVCM_STATUS_WATCH ||
	    		(flags & DO_UNLIST_WATCH)) {
		elem->next->prev = elem->prev;
		elem->prev->next = elem->next;
		namelist_free(elem);
		namelist_traversal_void(namelist);
	};
	return error;
}


/*-
 *--------------------------------------------------------------------------
 *
 * name_match -- compare two names
 *
 * DESCRIPTION:
 *	Compare two names, handling all the ugliness of optional prefix
 *	matching.
 *
 * RETURN VALUES:
 *      Boolean value "if a match".
 *
 *--------------------------------------------------------------------------
 */
static int
name_match(o1_len, o1, o2_len, o2)
	int o1_len, o2_len;
	struct svcm_namedobj_name *o1, *o2;
{
	/*
	 * The simple case.
	 */
	if (!o1->prefix_matchable && !o2->prefix_matchable) {
		/* Fast check of length and first character. */
		if (o1_len != o2_len || o1->name[0] != o2->name[0])
			return 0;
		return (bcmp((void*)o1->name, (void*)o2->name, namedobj_len_to_strlen(o1_len, o1)) == 0);
	};
	/*
	 * The next case:  one wildcard.
	 */
	if (o1->prefix_matchable + o2->prefix_matchable == 1) {
		/*
		 * Cannonicalize order:  make o1 the prefix.
		 */
		if (o2->prefix_matchable) {
			int tmp_len = o1_len;
			struct svcm_namedobj_name *tmp_o = o1;
			o1_len = o2_len;
			o1 = o2;
			o2_len = tmp_len;
			o2 = tmp_o;
		};
		if (o1_len > o2_len)
			return 0;   /* prefix must be at least possible to match */
		/* short-circut null check, else return if prefix matches */
		return ((o1_len == namedobj_strlen_to_len(0, NULL)) ||
			(bcmp((void*)o1, (void*)o2, o1_len) == 0));
	};
	/*
	 * The complex case:  two wildcards.
	 */
	/* Make o1 the shorter of the two. */
	if (o1_len > o2_len) {
		int tmp_len = o1_len;
		struct svcm_namedobj_name *tmp_o = o1;
		o1_len = o2_len;
		o1 = o2;
		o2_len = tmp_len;
		o2 = tmp_o;
	};
	/* Short circut the zero-length case. */
	if (namedobj_len_to_strlen(o1_len, o1) == 0)
		return 1;
	return bcmp((void*)o1->name, (void*)o2->name, namedobj_len_to_strlen(o1_len, o1)) == 0;
}





/**********************************************************************
 *
 * NAMEDOBJ REGISTER
 *
 */


static struct svcm_namedobj_name default_namedobj = { 1, "" };


/*-
 *--------------------------------------------------------------------------
 *
 * svcm_namedobj_register -- acquire/release rights to generic named object
 *
 * DESCRIPTION:
 *	Register a new resource request.
 *
 *	Watches are removed only by UNCACHED.
 *
 * SEE ALSO:
 *	svcm_unregister.
 *
 * RETURN VALUES:
 *      Returns an errno value.
 *
 *--------------------------------------------------------------------------
 */
int
svcm_namedobj_register(name, own_vp, obj_class, namedobj_len, namedobj, status, cred)
	struct svcm_name *name;		/* IN: stack name */
	struct vnode *own_vp;		/* IN: desired owning vp */
	enum svcm_obj_classes obj_class;/* IN: cache subobject name */
	int namedobj_len;			/* IN: length of obj_name */
	struct svcm_namedobj_name *namedobj; /* IN: cache subobject item */
	enum svcm_status status;	/* IN: desired status */
	struct ucred *cred;		/* IN */
{
	int error = 0;
	struct namelist *namelist, *elem;
	struct namelist_traversal_context context;

	/*
	 * Fast path the "common" case (at least with attributes)
	 * of no one caching the attributes.
	 * A registration doesn't matter if:
	 *	(1) it doesn't leave any permanent change
	 *  and (2) there's nothing in the namelist so it makes no
	 *		immediate change.
	 */
	namelist = (struct namelist*)name->classes[obj_class];
	if ((status == SVCM_STATUS_UNCACHED || status == SVCM_STATUS_NONCACHABLE) && 
	     (namelist == NULL || namelist->prev == namelist)) {
		SVCM_NAMEDOBJ_STATS_INC(obj_class, sns_register_fastpath_hit);
		return 0;
	} else {
		SVCM_NAMEDOBJ_STATS_INC(obj_class, sns_register_fastpath_miss);
	};

	/*
	 * Handle default namedobj.
	 */
	if (namedobj == NULL) {
		namedobj = &default_namedobj;
		namedobj_len = namedobj_strlen_to_len(0, namedobj);
	};

#ifdef SANITY_CHECKS
	if (name == NULL)
		panic("svcm_namedobj_register: null name.");
#endif /* SANITY_CHECKS */

	/*
	 * Possible initialization required.
	 */
	if (namelist == NULL) {
		if (error = SVCM_INIT(name, own_vp, obj_class, cred))
			return error;
		namelist = (struct namelist*)name->classes[obj_class];
	};

	/*
	 * Now fulfill the request.
	 */
	SVCM_NAMEDOBJ_STATS_INC(obj_class, sns_minimum_traversals);
 again:
	SVCM_NAMEDOBJ_STATS_INC(obj_class, sns_real_traversals);
	switch (status) {
	case SVCM_STATUS_UNCACHED:
		/*
		 * Uncached --> remove ourselves from the list.
		 * Removes WATCHES.
		 * No other callbacks.
		 */
		namelist_traverse_init(namelist, &context);
		while (elem = namelist_traverse(namelist, &context)) {
			SVCM_NAMEDOBJ_STATS_INC(obj_class, sns_traversal_hop);
			if (own_vp == elem->un.elem.own_vp &&
			        name_match(namedobj_len, namedobj,
					   elem->un.elem.name_len,
					   &elem->un.elem.name)) {
				(void)namelist_unlist(namelist, elem, name, obj_class, namedobj_len, namedobj, DO_UNLIST_WATCH, cred);
			};
		};
		if (!namelist_traversal_valid(namelist, &context))
			goto again;
		break;
	case SVCM_STATUS_NONCACHABLE:
		/*
		 * Noncachable --> Call back each list element.
		 */
		namelist_traverse_init(namelist, &context);
		while (elem = namelist_traverse(namelist, &context)) {
			SVCM_NAMEDOBJ_STATS_INC(obj_class, sns_traversal_hop);
			if (name_match(namedobj_len, namedobj,
					   elem->un.elem.name_len,
					   &elem->un.elem.name)) {
				(void)namelist_unlist(namelist, elem, name, obj_class, namedobj_len, namedobj, 0, cred);
			};
		};
		if (!namelist_traversal_valid(namelist, &context))
			goto again;
		break;
	case SVCM_STATUS_SHARED:
		/*
		 * Shared --> call back watchers and exclusives.
		 */
		namelist_traverse_init(namelist, &context);
		while (elem = namelist_traverse(namelist, &context)) {
			SVCM_NAMEDOBJ_STATS_INC(obj_class, sns_traversal_hop);
			if (name_match(namedobj_len, namedobj,
					   elem->un.elem.name_len,
					   &elem->un.elem.name))
				if (own_vp == elem->un.elem.own_vp) {
					if (elem->un.elem.status == SVCM_STATUS_SHARED)
						continue;
					else {
						(void)namelist_unlist(namelist, elem, name, obj_class, namedobj_len, namedobj, 0, cred);
					};
				} else {
					if (elem->un.elem.status == SVCM_STATUS_EXCLUSIVE ||
					    elem->un.elem.status == SVCM_STATUS_WATCH) {
						(void)namelist_unlist(namelist, elem, name, obj_class, namedobj_len, namedobj, 0, cred);
					} else ASSERT(elem->un.elem.status == SVCM_STATUS_SHARED);
				};
		};
		if (!namelist_traversal_valid(namelist, &context))
			goto again;
		namelist_list(namelist, own_vp, namedobj_len, namedobj, status);
		break;
	case SVCM_STATUS_EXCLUSIVE:
		/*
		 * Exclusive --> call back everyone.
		 */
		namelist_traverse_init(namelist, &context);
		while (elem = namelist_traverse(namelist, &context)) {
			SVCM_NAMEDOBJ_STATS_INC(obj_class, sns_traversal_hop);
			if (name_match(namedobj_len, namedobj,
					   elem->un.elem.name_len,
					   &elem->un.elem.name))
				if (own_vp == elem->un.elem.own_vp) {
					/* (Special-case ourselves.) */
					if (elem->un.elem.status == SVCM_STATUS_EXCLUSIVE)
						return 0;
					else {
						(void)namelist_unlist(namelist, elem, name, obj_class, namedobj_len, namedobj, 0, cred);
					};
				} else {
					(void)namelist_unlist(namelist, elem, name, obj_class, namedobj_len, namedobj, 0, cred);
				};
		};
		if (!namelist_traversal_valid(namelist, &context))
			goto again;
		namelist_list(namelist, own_vp, namedobj_len, namedobj, status);
		break;
	case SVCM_STATUS_WATCH:
		/*
		 * Watch --> just register interest.
		 * First make sure we haven't already registered.
		 */
		namelist_traverse_init(namelist, &context);
		while (elem = namelist_traverse(namelist, &context)) {
			SVCM_NAMEDOBJ_STATS_INC(obj_class, sns_traversal_hop);
			/* If already registered, skip. */
			if (own_vp == elem->un.elem.own_vp &&
			    	elem->un.elem.status == SVCM_STATUS_WATCH &&
				name_match(namedobj_len, namedobj,
					   elem->un.elem.name_len,
					   &elem->un.elem.name))
				return 0;
		};
		if (!namelist_traversal_valid(namelist, &context))
			goto again;   /* should never happen */
		namelist_list(namelist, own_vp, namedobj_len, namedobj, status);
		break;
	default:
		panic("svcm_namedobj_register: unknown status.");
	};
	return 0;
}




/**********************************************************************
 *
 *  DNLC INTERFACE
 *
 */

#ifdef svcm_dnlc_enter
#undef svcm_dnlc_enter
#endif /* svcm_dnlc_enter */
/*-
 *--------------------------------------------------------------------------
 *
 * svcm_dnlc_enter -- add a name to the name cache, cache coherently
 *
 * DESCRIPTION:
 *	Add a name to the name cache, cache coherently.
 *	This routine is exactly analgous to dnlc_enter but cache coherent.
 *
 *	A caller using svcm_dnlc_enter should also use svcm_dnlc_remove.
 *
 *	In the old design, the svcm's view of the dnlc is a
 *	(non-strict) superset of the actual dnlc contents.  The dnlc
 *	can at any time decide to purge cached entries (for example,
 *	because the cache is full).  Ideally the svcm would track the
 *	dnlc cache better.  Currently this problem is a NEEDSWORK in
 *	that the svcm view of the dnlc can grow excessively when the
 *	dnlc would not.
 *
 *	This design proved too expensive.  An ls -l of a large
 *	directory would cause all names to be cached.  Each cache
 *	enter would have to search the whole list of names.  For the
 *	find benchmark, this results in an average search length of 30
 *	names and a 10% performance hit.
 *
 *	The new approach optimizes cache name enters at the expense of
 *	cache removes.  Now instead of caching each name individually,
 *	we enter a wildcard name the first time we cache any name.
 *	Future name caches entries require no additional work.  Cache
 *	removes now invoke a callback and a purge at all cache layers,
 *	but removes are rare wrt name enters and so this should be a
 *	performance win.
 *
 * SEE ALSO:
 *	svcm_dnlc_remove, dnlc_enter.
 *
 * RETURN VALUES:
 *      Returns an errno value.
 *
 *-------------------------------------------------------------------------- */
int
svcm_dnlc_enter(name, dvp, nm, vp, cred)
	struct svcm_name *name;		/* IN: unlocked name */
	struct vnode *dvp;		/* IN: (dvp,nm) -> (vp) */
	char *nm;
	struct vnode *vp; 
	struct ucred *cred;		/* IN (can be NOCRED) */
{
	static struct svcm_namedobj_name wildcard_namedobj = { 1, "" };
	int error = 0;
	struct namelist *namelist;
	int needs_entry;

	ASSERT(name);
	/*
	 * Fast path the common case:  only one
	 * layer already has a name entered.
	 */
	namelist = (struct namelist*)name->classes[SVCM_CLASS_DNLC];
	needs_entry = !(namelist &&
	    	(namelist->prev == namelist->next && 
			namelist->prev != namelist) &&
	    	namelist->next->un.elem.own_vp == dvp &&
		namelist->next->un.elem.status == SVCM_STATUS_WATCH &&
		namelist->next->un.elem.name_len == namedobj_strlen_to_len(0, &wildcard_namedobj) &&
	   	namelist->next->un.elem.name.prefix_matchable);

	/*
	 * If we need to enter a name, do so.
	 */
	if (needs_entry) {
		ASSERT(dvp->v_type == VDIR);
		SVCM_NAMEDOBJ_STATS_INC(SVCM_CLASS_DNLC, sns_svcm_enter_fastpath_miss);
		SVCM_LOCK_CHECK_CHECKPOINT(name->lock);
		if (error = svcm_lock(name->lock)) {
			SVCM_LOCK_CHECK_VERIFY(name->lock);
			return error;
		};
		error = SVCM_REGISTER (name, dvp, SVCM_CLASS_DNLC,
			namedobj_strlen_to_len(0, &wildcard_namedobj),
			&wildcard_namedobj,
			SVCM_STATUS_WATCH, cred);
		(void)svcm_unlock(name->lock);
		SVCM_LOCK_CHECK_VERIFY(name->lock);
	} else {
		SVCM_NAMEDOBJ_STATS_INC(SVCM_CLASS_DNLC, sns_svcm_enter_fastpath_hit);
	};

	/*
	 * Finally cache the name.
	 * We don't need to be in the svcm_{,un}lock lock
	 * because we know our WATCH can't go away until dvp
	 * goes away and we old a reference to the dvp.
	 */
	dnlc_enter(dvp, nm, vp, cred);
	return error;
}

#ifdef svcm_dnlc_remove
#undef svcm_dnlc_remove
#endif /* svcm_dnlc_remove */
/*-
 *--------------------------------------------------------------------------
 *
 * svcm_dnlc_remove -- remove a name from any caches of all layers of a stack
 *
 * DESCRIPTION:
 *
 * SEE ALSO:
 *	svcm_dnlc_remove, dnlc_enter.
 *
 * RETURN VALUES:
 *      Returns an errno value.
 *
 *--------------------------------------------------------------------------
 */
int
svcm_dnlc_remove(name, dvp, nm, cred)
	struct svcm_name *name;		/* IN: unlocked name */
	struct vnode *dvp; 		/* IN */
	char *nm;			/* IN */
	struct ucred *cred;		/* IN: can be NOCRED */
{
	int error;
	struct svcm_namedobj_name namedobj;
	char *src, *dest;
	int len;
	SVCM_LOCK_CHECK_DECL;

	ASSERT(name != NULL);
	ASSERT(dvp->v_type == VDIR);

	src = nm;   dest = namedobj.name;   len = 0;
	while (*dest++ = *src++)
		if (++len > NC_NAMLEN+1)   /* +1 for the null byte */
			return 0;
	namedobj.prefix_matchable = 0;

	/*
	 * Lock the name cache,
	 * gain SHARED cache access to the name,
	 * and return.
	 */
	SVCM_LOCK_CHECK_CHECKPOINT(name->lock);
	if (error = svcm_lock(name->lock)) {
		SVCM_LOCK_CHECK_VERIFY(name->lock);
		return error;
	};
	/*
	 * Register ownership.  This will callback 
	 * anyone who has cached copies.
	 * (Even ourselves, if we do.)
	 *
	 * (Use len+1 to get null terminating byte.)
	 */
	error = SVCM_REGISTER (name, dvp, SVCM_CLASS_DNLC,
			       namedobj_strlen_to_len(len+1, &namedobj), &namedobj,
			       SVCM_STATUS_NONCACHABLE, cred);
	(void)svcm_unlock(name->lock);
	SVCM_LOCK_CHECK_VERIFY(name->lock);
	return error;
}



/**********************************************************************
 *
 *  ATTRIBUTE INTERFACE
 *
 */

DEFINE_ATTRIBUTE_NAME(svcm_vattr_fixed,1,"v");
DEFINE_ATTRIBUTE_NAME(svcm_vattr_fixed_var,2,"vv");
DEFINE_ATTRIBUTE_NAME(svcm_vattr_fixed_var_time,3,"vvt");
DEFINE_ATTRIBUTE_NAME(svcm_fattr_fixed,1,"f");
DEFINE_ATTRIBUTE_NAME(svcm_fattr_fixed_var,2,"fv");
DEFINE_ATTRIBUTE_NAME(svcm_fattr_fixed_var_vve,3,"fvv");





#endif /* SVCM_TWO */


