
/*-
 * svcm_namedobj.h
 *
 *	@(#)svcm_namedobj.h	1.6 (UCLA Ficus Project) 8/15/94
 *
 * Copyright (c) 1993 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef i405_svcm_namedobj_h
#define i405_svcm_namedobj_h

#include <sys/dnlc.h>   /* for NC_NAMLEN */

#define SVCM_NAMEDOBJ_NAMELEN 16
#if (SVCM_NAMEDOBJ_NAMELEN < NC_NAMLEN)
This makes an error.
/* We must have space for names in the dnlc plus the null. */
#endif


/*
 * A name is a byte sequenece.
 * If the prefix_matchable sequence is set
 * we consider matches on any prefix of the name to be a match.
 * This allows hierarchical naming of attributes
 * while allowing them to be easily purged.
 *
 * Note: strlen<->len omits the null byte, possibly causing
 * some false hits.
 */
#define namedobj_strlen_to_len(LEN, NO) ((LEN) + sizeof(u_int))
#define namedobj_len_to_strlen(LEN, NO) ((LEN) - sizeof(u_int))
struct svcm_namedobj_name {
	u_int prefix_matchable;
	char name[SVCM_NAMEDOBJ_NAMELEN];
};

#define SVCM_NAMEDOBJ_IS_WILDCARD(LEN,NOP) ((LEN)==sizeof(u_int) && (NOP)->prefix_matchable)


/*
 * Common names for attribute handling.
 */
#define EXTERN_DEFINE_ATTRIBUTE_NAME(NM) \
	extern int CONCAT2(NM,_namedobj_len); \
	extern struct svcm_namedobj_name CONCAT2(NM,_namedobj); \
	extern int CONCAT2(NM,_star_namedobj_len); \
	extern struct svcm_namedobj_name CONCAT2(NM,_star_namedobj);
#define DEFINE_ATTRIBUTE_NAME(NM,LEN,STR) \
	int CONCAT2(NM,_namedobj_len) = sizeof(int)+LEN; \
	struct svcm_namedobj_name CONCAT2(NM,_namedobj) = { 0, STR }; \
	int CONCAT2(NM,_star_namedobj_len) = sizeof(int)+LEN; \
	struct svcm_namedobj_name CONCAT2(NM,_star_namedobj) = { 1, STR };

EXTERN_DEFINE_ATTRIBUTE_NAME(svcm_vattr_fixed);
EXTERN_DEFINE_ATTRIBUTE_NAME(svcm_vattr_fixed_var);
EXTERN_DEFINE_ATTRIBUTE_NAME(svcm_vattr_fixed_var_time);
EXTERN_DEFINE_ATTRIBUTE_NAME(svcm_fattr_fixed);
EXTERN_DEFINE_ATTRIBUTE_NAME(svcm_fattr_fixed_var);
EXTERN_DEFINE_ATTRIBUTE_NAME(svcm_fattr_fixed_var_vve);



#include <i405/SVCM/svcm_namedobj__ah.h>


/*
 * One use of the namedobj cache is to keep dnlc caches coherent.
 * To use a cache coherent dnlc, call
 * 	svcm_dnlc_{lookup,enter,remove,purge}
 * instead of
 *	dnlc_{lookup,enter,remove,purge}.
 * There is currently no equivlent to dnlc_purge_vp or dnlc_iter.
 */
#define svcm_dnlc_lookup(NAME,DVP,NM,CRED)	dnlc_lookup(DVP,NM,CRED)
#define svcm_dnlc_purge()		dnlc_purge()
/* svcm_dnlc_{enter,remove} are function calls. */
#if 0 /* USE_SVCM_DNLC */
#define svcm_dnlc_lookup(NAME,DVP,NM,CRED)	dnlc_lookup(DVP,NM,CRED)
#define svcm_dnlc_purge()		dnlc_purge()
#define svcm_dnlc_enter(NAME, DVP, NM, VP, CRED) dnlc_enter(DVP, NP, VP, CRED)
#define svcm_dnlc_remove(NAME, DVP, NM, CRED) dnlc_remove(DVP, NM)
#endif /* USE_SVCM_DNLC */


#endif /* i405_svcm_namedobj_h */
