
/*-
 * svcm_subr.c
 *
 *	@(#)svcm_subr.c	1.4 (UCLA Ficus Project) 5/8/95
 *
 * Copyright (c) 1994 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifdef SVCM_TWO

#include <sys/param.h>
#include <sys/time.h>
#include <sys/buf.h>
#include <sys/vnode.h>
#include <i405/i405.h>
#include <i405/SVCM/svcm.h>
#include <sys/user.h>
#include <sys/resource.h>
#include <sys/uio.h>
#include <sys/vmmeter.h>
#include <sys/vmsystm.h>
#include <sys/mman.h>
#include <sys/vfs.h>
#include <sys/debug.h>
#include <sys/trace.h>
#include <sys/ucred.h>

#include <vm/hat.h>
#include <vm/as.h>
#include <vm/seg.h>
#include <vm/seg_vn.h>
#include <vm/rm.h>
#include <vm/pvn.h>
#include <vm/page.h>
#include <vm/seg_map.h>
#include <vm/swap.h>   /* for IS_SWAPVP */

#include <sys/debug.h>


#if 1
#define DODEBUG(A) A
#else
#define DODEBUG(A)
#endif


/*
 *--------------------------------------------------------------------------
 *
 * i405_vn_cachenamevp -- generic vop_cachenamevp code for backing store'ers
 *
 * DESCRIPTION:
 *	This routines simply returns the local vnode as the cache name vnode.
 *	This is suitable for people who actually implement backing storage,
 *	such as the UFS and NFS.
 *
 * RETURN VALUES:
 *      Zero (no error).
 *
 *--------------------------------------------------------------------------
 */
int
i405_vn_cachenamevp(ap)
	struct nvop_cachenamevp_args *ap;
{
	/* We *are* the backing store. */
	*ap->a_vpp = ap->a_vp;
	return 0;
}


/*
 *--------------------------------------------------------------------------
 *
 * i405_vn_cache_callback -- generic code to implement vop_cache_callback
 *
 * DESCRIPTION:
 *	Cache callback is used keep different caches
 *	(page, attribute, name) coherent.
 *
 * RETURN VALUES:
 *      Returns an errno value.
 *
 *--------------------------------------------------------------------------
 */
int
i405_vn_cache_callback(ap)
	struct nvop_cache_callback_args *ap;
{
	USES_NVOP_STACKPUTPAGE;
	struct svcm_byterange_name *rangep;
	struct svcm_namedobj_name *namedobjp;

	switch (ap->a_obj_class) {
	case SVCM_CLASS_DATA_PAGES:
		rangep = (struct svcm_byterange_name *)
				struct_nvop_cache_callback_args_a_obj(ap);
#ifdef SVCM_PAGEFLIP
		/*
		 * B_INVALID is required to make page_find work.
		 */
		return NVOP_STACKPUTPAGE(ap->a_vp, ap->a_name,
					 ap->a_vp,
					 rangep->offset,
					 rangep->length,
					 B_INVAL|B_PAGEFLIP,
					 ap->a_cred);
#else /* ! SVCM_PAGEFLIP */
		return NVOP_STACKPUTPAGE(ap->a_vp, ap->a_name,
					 ap->a_vp,
					 rangep->offset,
					 rangep->length,
					 B_INVAL,
					 ap->a_cred);
#endif /* SVCM_PAGEFLIP */
	case SVCM_CLASS_DNLC:
		namedobjp = (struct svcm_namedobj_name *)
				struct_nvop_cache_callback_args_a_obj(ap);
		/*
		 * Discard wildcard callbacks.
		 * They happen only on vp inactive
		 * when there can be no DNLC names.
		 */
		if (!SVCM_NAMEDOBJ_IS_WILDCARD(struct_nvop_cache_callback_args_a_obj_elems(ap), namedobjp)) {
			dnlc_remove(ap->a_vp, namedobjp->name);
		};
		return 0;
	default:
		panic("i405_vn_cache_callback: unkown obj class");
	};
	/* NOTREACHED */
}


int
i405_vn_fsync(ap)
	struct nvop_fsync_args *ap;
{
	USES_NVOP_PUTPAGE;
	return NVOP_PUTPAGE(ap->a_vp, 0, 0, 0, ap->a_cred);
}



#endif SVCM_TWO
