#!/usr/local/bin/perl

#
# i405_tests.perl
#	-- a master perl script to run some tests
#

package i405_tests;

$testdir = "/global/us/edu/ucla/cs/ficus/sys/i405/TEST";


#
# The entry point.
#
sub main'i405_tests {
    local ($test);
    foreach $test (split(/ /, "getattr_test readdir_test newfid_test ".
			      "newfidtovp_test")) {
	eval ("print \"\t$test\n\"; &$test") ||
	    die ("eval: error $@");
	print "\t\t$test passes.\n";
    };
    return 1;   # things seemed to work.
}


sub cleanup {
	&main'cleanup(@_);
}

sub system {
	&main'system(@_);
}


#
# 1. getattr
#
# This tests basic things.  Returning attributes,
# passing in credentials.
#
sub getattr_test {
    ($error, @output) = &system("$testdir/getattr .");
    if ($error) { &cleanup("Cannot run $testdir/getattr.\n"); }
    &cleanup("va_type returned is not VDIR.\n")
	if (-1 == grep(/^va_type\s*=\s*(2|VDIR)$/, @output));
    return 1;
}


#
# 2. readdir
#
# This tests uios moving data around.
#
sub readdir_test {
    local ($i);

    &readdir_subtest(".");
    mkdir ("readdir_test", 0777) || &cleanup("Cannot mkdir readdir_test.");
    if (0) {
	chdir ("readdir_test") || &cleanup("Cannot chdir readdir_test.");
	for ($i = 0; $i < 4096; $i++) {
	    &touch("./$i") || &cleanup("Cannot touch $i.");
	};
	chdir ("..") || &cleanup("Cannot chdir .. .");
	&readdir_subtest("readdir_test");
    };
    return 1;
}

sub readdir_subtest {
    local ($dir) = @_;
    opendir (DIR, $dir) || &cleanup("Cannot open directory $dir.\n");
    @dirents1 = readdir(DIR);
    closedir(DIR);
    ($error, @dirents2) = &system("$testdir/readdir .");
    if ($error) { &cleanup("Cannot run $testdir/readdir."); };
    # Take the second word, throwing out lines without a second word.
    @dirents2 = grep($_ !~ /^$/, grep($_ =~ s/^\S+\s+(\S+)\n$/\1/, @dirents2));

    &cleanup("readdir_subtest $dir fails with different numbers of entries.\n")
	if ($#dirents1 != $#dirents2);

    local ($i);
    foreach $i (0..$#dirents1) {
	&cleanup("readdir_subtest $dir, entry $dirents1[$i] ne $dirents2[$i].\n")
	    if ($dirents1[$i] ne $dirents2[$i]);
    };
}


#
# newfid_test
#
# This test demonstrates returned variable length arrays.
#
sub newfid_test {
    ($error, @output) = &system("$testdir/newfid .");
    if ($error) { &cleanup("Cannot run $testdir/newfid.\n"); };
    &cleanup("newfid does not appear to be of the correct format.\n")
	if (-1 == grep(/^\d+:[0-9a-f,]+$/, @output));
    return 1;
}


#
# newfidtovp_test
#
# This test demonstrates the return of vnodes.
#
sub newfidtovp_test {
    local ($fn, $data) = ("newfidtovp_test", "The quick brown fox.\n");
    open (FN, "> ./$fn") || die ("Cannot open file $fn.");
    print FN $data;
    close (FN);

    local (@output);
    ($error, @output) = &system("$testdir/newfid ./$fn");
    if ($error) { &cleanup("Cannot run $testdir/newfid.\n"); };
    &cleanup("newfid does not appear to be of the correct format.\n")
	if (-1 == grep(/^\d+:[0-9a-f,]+$/, @output));
    &cleanup("too many lines of newfid output.")
	if ($#output != 0);

    local ($newfid) = $output[0];
    chop $newfid;


    ($error, @output) = &system("$testdir/newfidtovp . $newfid");
    if ($error) { &cleanup("Cannot run $testdir/newfidtovp.\n"); };
    # Output will be somve fattr stuff followed by the file contents.
    # Just make sure the last line is the right file data.
    die ("File data returned from newfidtovp does not appear to match.")
	if (-1 == grep(/^${data}$/, @output));

    unlink ("./$fn") ||
	die ("Cannot remove $fn.");

    return 1;
}


1;
