
/*
 * newfid_to_vp.c
 *
 * KERNEL, OUTOFKERNEL, STANDALONE_OOK are defined in the Makefile.
 */

#include <stdio.h>
#undef FILE

#include <sys/types.h>
#include <sys/time.h>
#include <sys/vnode.h>
#include <i405/i405.h>
#include <sys/param.h>
#include <sys/uio.h>

#include <i405/ook_compat.h>
#include <newofs/newo_interface.h>

#include <sys/fcntl.h>


void
decode_newfid(s, newfidp)
	char *s;
	struct newfid *newfidp;
{
	extern char *strdup(), *strtok();
	char *ns = strdup(s);
	char *p;
	int n, i;
	static char sep[] = ",: \t\n";

	p = strtok(ns, sep);

	n = atoi(p);
	if (n > newfidp->newfid_data_space) {
		printf ("Not enough space for fid.\n");
		exit (1);
	};
	newfidp->newfid_data_elems = n;

	for (i = 0; i < n; i++) {
		p = strtok(NULL, sep);
		newfidp->newfid_data[i] = strtol(p, NULL, 16);
	};

	/*
	 * Echo print.
	 */
	printf ("%d:", newfidp->newfid_data_elems);
	for (i =0 ; i < newfidp->newfid_data_elems; i++)
		printf ("%.2x,", (unsigned char)(newfidp->newfid_data[i]));
	printf ("\n");

}


int
main(argc, argv)
	int argc;
	char **argv;
{
	struct vnode *vp, *nvp;
	USES_NVOP_NEWFID;
	USES_NVOP_NEWFID_TO_VP;
	USES_NVOP_GETATTR;
	int error,i;
	struct newfid newfid;
#define DATASPACE 128
	static u_char newfiddata[DATASPACE];
#define MYBUFSIZE 1024
	char buf[MYBUFSIZE];
	int offset, resid;
	struct vattr va;

	if (argc != 3) {
		printf ("usage: %s volumeroot newfid\n", argv[0]);
		exit (1);
	};

	if (error = ook_vn_open(argv[1], &vp)) {
		perror ("vn_open");
		exit (1);
	};

	newfid.newfid_data = newfiddata;
	newfid.newfid_data_space = DATASPACE;
	newfid.newfid_data_elems = 0;

	decode_newfid(argv[2], &newfid);

	printf ("calling nvop_newfid_to_vp on %d,(%d,%d,...)...\n",
		vp, newfid.newfid_data_elems, newfid.newfid_data_space);
	if (error = NVOP_NEWFID_TO_VP (vp, &newfid, &nvp, NULL)) {
		perror ("newfid_to_vp");
		exit (1);
	};

	printf ("newfid_to_vp returns %d, nvp=%d.\n", error, nvp);

	if (error = NVOP_GETATTR (nvp, &va, NULL)) {
		perror ("getvattr");
		exit (1);
	};
	print_vattr (&va);

	offset = resid = 0;
	while (0==(error=ook_vn_rdwr(UIO_READ, nvp, buf, MYBUFSIZE, offset, UIO_OURSPACE, 0, &resid))) {
		fwrite (buf, 1, MYBUFSIZE-resid, stdout);
		if (resid != 0)
			break;
		offset += MYBUFSIZE;
	};

	exit (0);
}


print_vattr(va)
struct vattr	va;
{
	printf("va_type = %d\n",va.va_type);
	printf("va_mode = %o\n",va.va_mode);
	printf("va_uid = %d\n",va.va_uid);
	printf("va_gid = %d\n",va.va_gid);
	printf("va_fsid = %d\n",va.va_fsid);
	printf("va_nodeid = %d\n",va.va_nodeid);
	printf("va_nlink = %d\n",va.va_nlink);
	printf("va_size = %d\n",va.va_size);
	printf("va_blocksize = %d\n",va.va_blocksize);
	printf("va_blocks = %d\n",va.va_blocks);
}

