
/*-
 * putpage.c
 *
 *	%W% (UCLA Ficus Project) %G%
 *
 * Copyright (c) 1994 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/* to get B_PAGEFLIP we need these defines */
#define SVCM_TWO
#define SVCM_PAGEFLIP

#include <unistd.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <stdio.h>
#include <sys/param.h>
#include <i405/i405_syscalls.h>
#include <errno.h>
#include <ficus/fix_sunos_headers.h>

#undef FILE  /* for vnode.h */
#include <ficus/ficus.h>
#include <ficus/ficus_interface.h>
#include <i405/i405_syscalls.h>

#include <sys/buf.h>   /* for B_INVAL and friends */

extern char *optarg;
extern int optind, opterr;


void
usage()
{
	printf("usage: putpage -i|-f file [file...]\n");
	exit (1);
}



void
putpage_file(file, flags)
	char *file;
	int flags;
{
	struct vnode *vp;
	int error;
	USES_NVOP_PUTPAGE;

	if (error = ook_lookupname(file, NO_FOLLOW, NULL, &vp)) {
		perror("ook_lookupname");
		exit(1);
	};
	error = NVOP_PUTPAGE(vp, 0, 0, flags, NULL);
	if (error)
		printf("nvop_putpage: error %d\n", error);
	ook_vn_close (vp);
}



main(argc, argv)
	int argc;
	char **argv;
{
	int flags = 0;
	int c;
	int i;

	while ((c = getopt (argc, argv, "fi?")) != -1)
		switch (c) {
		case 'f':
			flags = B_INVAL|B_PAGEFLIP;
			break;
		case 'i':
			flags = B_INVAL;
			break;
		default:
			usage();
		};
	if (flags == 0)
		usage();

	for (i = optind; i < argc; i++) {
		putpage_file(argv[i], flags);
	};
}
