
/*
 * readdir.c
 *
 *	@(#)readdir.c	1.2 (UCLA Ficus Project) 6/30/93
 *
 * Copyright (c) 1993 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/types.h>
#include <sys/time.h>
#include <sys/uio.h>
#include <sys/vnode.h>
#include <i405/i405.h>
#include <sys/param.h>

#include <i405/ook_compat.h>

#include <sys/fcntl.h>

#include <sys/dirent.h>


int
main(argc, argv)
	int argc;
	char **argv;
{
	struct vnode *vp;
	USES_NVOP_GETATTR;
	USES_NVOP_READDIR;
	int offset;
	int error,i;
	struct vattr  va;
	struct uio auio;
	struct iovec aiov;
#define BUFSIZE (8*1024)
	char buf[BUFSIZE];
	int n;
	struct dirent *de;

	if (argc != 2) {
		printf ("usage: %s FileName \n", argv[0]);
		exit (1);
	};


	if (error = ook_vn_open(argv[1], &vp)) {
		perror ("vn_open");
		exit (1);
	};

	if (error = NVOP_GETATTR (vp, &va, NULL)) {
		perror ("getvattr");
		exit (1);
	};
	if (va.va_type != VDIR) {
		printf ("not a directory.\n");
		exit (1);
	};

	aiov.iov_base = buf;
	aiov.iov_len = BUFSIZE;

	auio.uio_iov = &aiov;
	auio.uio_iovcnt = 1;
	auio.uio_offset = 0;
	auio.uio_segflg = UIO_OURSPACE;
	auio.uio_resid = BUFSIZE;

	if (error = NVOP_READDIR (vp, &auio, NULL)) {
		perror ("readdir");
		exit (1);
	};

	printf ("resid=%d\n", auio.uio_resid);

	n = BUFSIZE - auio.uio_resid;

	printf ("n=%d\n", n);
	de = (struct dirent*)buf;
	while (((char*)de) - buf < n) {
		printf ("%d %s\n", de->d_fileno, de->d_name);
		de = (struct dirent*)((char*)de + de->d_reclen);
		if (de->d_reclen == 0)
			break;
	};
	printf ("done\n");

	exit (0);
}


