
/*
 * repeat_getfattrs.c
 *
 *	@(#)repeat_getfattrs.c	1.2 (UCLA Ficus Project) 10/25/93
 *
 * Copyright (c) 1992 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*
 * KERNEL, OUTOFKERNEL, STANDALONE_OOK are defined in the Makefile.
 */

#include <sys/types.h>
#include <sys/time.h>
#include <sys/vnode.h>
#include <i405/i405.h>
#include <sys/param.h>
/*
 * Stdio.h must be included after vnode.h because somebody used the
 * symbol FILE in "enum rm".  Note that this means that "enum rm"
 * isn't usable after stdio.h gets included.
 */
#include <stdio.h>

#include <ficus/ficus.h>
#include <ficus/ficus_interface.h>
#include <i405/i405_syscalls.h>

#include <sys/fcntl.h>




void
print_vattr(va)
struct vattr	va;
{
	printf("va_type = %d\n",va.va_type);
	printf("va_mode = %o\n",va.va_mode);
	printf("va_uid = %d\n",va.va_uid);
	printf("va_gid = %d\n",va.va_gid);
	printf("va_fsid = 0x %x\n",va.va_fsid);
	printf("va_nodeid = %d\n",va.va_nodeid);
	printf("va_nlink = %d\n",va.va_nlink);
	printf("va_size = %d\n",va.va_size);
	printf("va_blocksize = %d\n",va.va_blocksize);
	printf("va_blocks = %d\n",va.va_blocks);
}


void
ffsh_to_str (ffshp, s)
	struct ffshandle *ffshp;
	char *s;
{
	sprintf (s, ":%x,%x,%x:%x:%x,%x:%x", ffshp->ffsh_volid.net,
		ffshp->ffsh_volid.host, ffshp->ffsh_volid.uid,
		 ffshp->ffsh_dirid,
		 ffshp->ffsh_fileid.a,
		 ffshp->ffsh_fileid.b,
		 ffshp->ffsh_replid);
}


void
print_ffshandle(ffsh)
	struct ffshandle ffsh;
{
	char tmps[256];
	ffsh_to_str(&ffsh, tmps);
	printf ("ffsh = %s\n", tmps);
	printf("volid = 0x %x,%x,%x\n",ffsh.ffsh_volid.net,
				      ffsh.ffsh_volid.host,
				      ffsh.ffsh_volid.uid);
	printf("dirid = 0x %x\n",ffsh.ffsh_dirid);
	printf("fileid = 0x %x,%x\n",ffsh.ffsh_fileid.a, ffsh.ffsh_fileid.b);
	printf("replid = 0x %x\n",ffsh.ffsh_replid);
}

void
print_replist(replist)
	replicalist_t replist;
{
	printf("replid = 0x %x, ",replist.replid);
	printf("vvelem = %d, ",replist.vvelem);
	printf("saved_vvelem = %d, ",replist.saved_vvelem);
	printf("glob_vvelem = 0x %x, ",replist.glob_vvelem);
	printf("status = %d\n",replist.status);
}



int
main(argc, argv)
	int argc;
	char **argv;
{
	struct vnode *vp;
	USES_FVOP_GETFATTRS;
	int i;
	int count;
	char *filename;
	int error;
	struct fattr  nfa;
	replicalist_t replist[REPLIST_SPACE];
	int use_syscall = 0;

	if (argc != 3 && argc != 4) {
		fprintf(stderr, "usage: %s [-s] FileName count\n", argv[0]);
		exit (1);
	};

	if (strcmp(argv[1], "-s") == 0) {
		use_syscall = 1;
		filename = argv[2];
		count = atoi(argv[3]);
	} else {
		filename = argv[1];
		count = atoi(argv[2]);
	};

	if (error = ook_lookupname (filename, NO_FOLLOW, NULL, &vp)) {
		perror ("vn_open");
		exit (1);
	};

	init_struct_fattr_fa_replist(&nfa, 0, REPLIST_SPACE, replist);

	for (i = 0; i < count; i++) {
		if (use_syscall) {
			error = syscall(SYS_test_getfattrs, vp, &nfa, NULL);
		} else {
			error = FVOP_GETFATTRS(vp, &nfa, NULL);
		};
		if (error) {
			fprintf(stderr,"error = %d\n", error);
			exit (1);
		};
	};

	ook_vn_close(vp);

#if 1
	print_ffshandle(nfa.fa_ffsh);
	print_vattr(nfa.fa_va);
	printf("tlink = %d\n",nfa.fa_tlink);
	printf("conflict = %d\n",nfa.fa_conflict);
	printf("elems = %d\n",struct_fattr_fa_replist_elems(&nfa));
	printf("space = %d\n",struct_fattr_fa_replist_space(&nfa));
        printf("gp = %d\n",nfa.fa_gpflag);
	for (i=0;i<struct_fattr_fa_replist_elems(&nfa);i++)
		 print_replist(struct_fattr_fa_replist(&nfa)[i]);
#endif

	exit (0);
}
