
/*
 * twoholds.c
 *
 * Open a file, open it again, then release it twice.
 * (Verify that multiple accesses work.)
 *
 *	@(#)twoholds.c	1.3 (UCLA Ficus Project) 10/25/93
 *
 * Copyright (c) 1993 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <stdio.h>
#undef FILE

#include <sys/types.h>
#include <sys/time.h>
#include <sys/vnode.h>
#include <i405/i405.h>
#include <sys/param.h>

#include <sys/fcntl.h>


char *files[] = {
	"/",
	"/sbin",
	"/usr",
	"/etc",
};



void
bail(s, e)
	char *s;
	int e;
{
	fprintf(stderr, "%s: errno = %d\n", s, e);
	exit(1);
}




int
main(argc, argv)
	int argc;
	char **argv;
{
	struct vnode *vp1, *vp2, *vp3, *vp4;
	int error;

	if (argc != 1) {
		printf("usage: %s\n", argv[0]);
		exit(1);
	};

	if (error = ook_lookupname(files[0], NO_FOLLOW, NULL, &vp1)) {
		bail("twoholds: FAILURE ook_lookname: first open fails", error);
	};

	if (error = ook_lookupname(files[0], NO_FOLLOW, NULL, &vp2)) {
		bail("twoholds: FAILURE ook_lookname: second open fails", error);
	};

	if (vp1 != vp2) {
		char tmps[80];

		sprintf(tmps, "twoholds: FAILURE %d == vp1 != vp2 == %d", (int)vp1, (int)vp2);
		bail(tmps, 0);
	} else {
		printf("\tGood: vp1 (file[0]) == vp2 (file[0]).\n");
	};

	if (error = ook_vn_rele(vp1)) {
		bail("twoholds: FAILURE ook_vn_rele: first release fails", error);
	} else {
		printf("\tGood: vp1 rele'd successfully.\n");
	};

	/*
	 * Open a second file, it should get a new descriptor (indicating
	 * that we have not rele'ed the fd for vp1).
	 */
	if (error = ook_lookupname(files[1], NO_FOLLOW, NULL, &vp3)) {
		bail("twoholds: FAILURE ook_lookname: third lookup fails", error);
	};

	if (vp2 == vp3) {
		char tmps[80];

		sprintf(tmps, "twoholds: FAILURE %d == old fd == new fd == %d, they should differ", (int)vp2, (int)vp3);
		bail(tmps, 0);
	} else {
		printf("\tGood: vp3 (file[1]) != vp2 (file[0]).  vp2 not rele'd.\n");
	};

	if (error = ook_vn_rele(vp2)) {
		bail("twoholds: ook_vn_rele: second release fails", error);
	} else {
		printf("\tGood: vp2 rele'd successfully.\n");
	};

	/*
	 * Now we should reuse the old fd.
	 */
	if (error = ook_lookupname(files[2], NO_FOLLOW, NULL, &vp4)) {
		bail("twoholds: FAILURE ook_lookname: fourth lookup fails", error);
	};

	if (vp1 != vp4) {
		char tmps[80];

		sprintf(tmps, "twoholds: FAILURE %d == free old fd != new fd == %d, they should differ", (int)vp1, (int)vp3);
		bail(tmps, 0);
	} else {
		printf("\tGood: vp4 reused vp1 correctly.\n");
	};

	exit(0);
}
