
/*
 * twoholds.c
 *
 * Open a file, open it again, then release it twice.
 * (Verify that multiple accesses work.)
 *
 *	@(#)twoholds2.c	1.1 (UCLA Ficus Project) 2/18/94
 *
 * Copyright (c) 1993 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <stdio.h>
#undef FILE

#include <sys/types.h>
#include <sys/time.h>
#include <sys/vnode.h>
#include <i405/i405.h>
#include <sys/param.h>

#include <sys/fcntl.h>


void
print_vattr(va)
	struct vattr va;
{
	printf("va_type = %d\n",va.va_type);
	printf("va_mode = %o\n",va.va_mode);
	printf("va_uid = %d\n",va.va_uid);
	printf("va_gid = %d\n",va.va_gid);
	printf("va_fsid = %d\n",va.va_fsid);
	printf("va_nodeid = %d\n",va.va_nodeid);
	printf("va_nlink = %d\n",va.va_nlink);
	printf("va_size = %d\n",va.va_size);
	printf("va_blocksize = %d\n",va.va_blocksize);
	printf("va_blocks = %d\n",va.va_blocks);
}

void
bail(s, e)
	char *s;
	int e;
{
	char tmps[1024];
	fprintf(stderr, "%s: errno = %d\n", s, e);
	exit (1);
}

int
do_it(fname, level)
	char *fname;
{
	struct vnode *vp1;
	int error;

	if (level >= 3)
		return 0;
	
	if (error = ook_lookupname(fname, NO_FOLLOW, NULL, &vp1)) {
		bail ("twoholds: ook_lookname: first open fails", error);
	}
	printf("%d\n", vp1);
	error = do_it(fname, level+1);
	if (error)
		bail("twoholds: error from recursive call", error);

	if (error = ook_vn_rele(vp1)) {
		bail("twoholds: vn_rele failed", error);
	}
	return 0;
}	


int
main(argc, argv)
	int argc;
	char **argv;
{
	struct vnode *vp1, *vp2;
	int error, i;
	struct vattr va;
	char *file;

	if (argc != 2) {
		printf ("usage: %s filename \n", argv[0]);
		exit (1);
	};

	do_it(argv[1], 0);
}


