
/*
 * A simpler-to-use interface than that of the log/phys layers.
 */


#include <stdio.h>

#include <sys/types.h>
#include <sys/time.h>
#include <sys/uio.h>
#include <sys/errno.h>
#undef FILE
#include <sys/vnode.h>
#include <newvint/newvint.h>
#include <newvint/newvint_uuids.h>
#include <newvint/newvint_syscalls.h>
#include <newofs/newo_interface.h>

#include <sys/fcntl.h>


int user_new_vnodeop_debug=0;
/*
 * User level interface to the new vnode operations.
 * NEEDSWORK: Bad style, uses magic syscall numbers.
 */
int
user_new_vnodeop(vp, off, ap)
	struct vnode *vp;
	int off;
	struct nvop_generic_args *ap;
{
	struct nvop_generic_args **app = &ap;  /* YAH--yet another hack */
	int error;
	extern int errno;

	if (user_new_vnodeop_debug)
		printf ("calling vop, vp=%x, off=%d(%s), app=%x", vp, off,
			ap->a_desc->vdesc_name,
			app);
	error = syscall (SYS_user_new_vnodeop, vp, off, app);
	if (user_new_vnodeop_debug)
		printf (".\n");
	return (error == -1) ? errno : 0;
}





/*
 * Open/create a vnode.
 * This uses tricks of the new ook vn interface.
 */
int
ook_vn_open(pnamep, vpp)
	char *pnamep;
	struct vnode **vpp;
{
	extern int errno;
	int error;
	int fd;

	/*
	 * Wish to open a file.
	 * Just call the kernel.
	 */
	if (-1==(fd = open(pnamep, O_RDONLY)))
		return errno;

	/*
	 * fd of zero is confused with a null vnode pointer.
	 * Just hope this error doesn't happen.
	 */
	if (fd==0) {
		printf ("Internal problem in vn_open.\n");
		exit (1);
	};

	*vpp = (struct vnode*)fd;
	return 0;
}


int
ook_vn_close(vp)
	struct vnode *vp;
{
	close ((int) vp);
	return 0;
}


/*
 * read or write a vnode
 */
int
ook_vn_rdwr(rw, vp, base, len, offset, seg, ioflag, aresid)
	enum uio_rw rw;
	struct vnode *vp;
	caddr_t base;
	int len;
	int offset;
	int seg;
	int ioflag;
	int *aresid;
{
	USES_NVOP_RDWR;
	struct uio auio;
	struct iovec aiov;
	int error;

#if 0
	if ((rw == UIO_WRITE) && isrofile(vp)) {
		return (EROFS);
	}
#endif

	aiov.iov_base = base;
	aiov.iov_len = len;
	auio.uio_iov = &aiov;
	auio.uio_iovcnt = 1;
	auio.uio_offset = offset;
	auio.uio_segflg = seg;
	auio.uio_resid = len;
	error = NVOP_RDWR(vp, &auio, rw, ioflag, NULL);  /* kernel will fill in u.u_cred */
	if (aresid)
		*aresid = auio.uio_resid;
	else
		if (auio.uio_resid)
			error = EIO;
	return (error);
}
