
/*
 * vnode_to_fd.c
 *
 *	@(#)vnode_to_fd.c	1.5 (UCLA Ficus Project) 3/6/95
 *
 * Copyright (c) 1993 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/types.h>
#include <sys/time.h>
#include <sys/vnode.h>
#include <i405/i405.h>
#include <i405/i405_syscalls.h>
#include <sys/param.h>

#include <sys/fcntl.h>
#include <sys/stat.h>



void
print_vattr(va)
	struct vattr va;
{
	printf("va_type = %d\n", va.va_type);
	printf("va_mode = %o\n", va.va_mode);
	printf("va_uid = %d\n", va.va_uid);
	printf("va_gid = %d\n", va.va_gid);
	printf("va_fsid = %d\n", va.va_fsid);
	printf("va_nodeid = %d\n", va.va_nodeid);
	printf("va_nlink = %d\n", va.va_nlink);
	printf("va_size = %d\n", va.va_size);
	printf("va_blocksize = %d\n", va.va_blocksize);
	printf("va_blocks = %d\n", va.va_blocks);
}


void
print_stat(buf)
	struct stat *buf;
{
	/* printf ("%s\n", pn); */
	printf ("st_dev:       %d\n", buf->st_dev);
	printf ("st_ino:       %d\n", buf->st_ino);
	printf ("st_mode:      0%o\n", buf->st_mode);
	printf ("st_nlink:     %d\n", buf->st_nlink);
	printf ("st_uid:       %d\n", buf->st_uid);
	printf ("st_gid:       %d\n", buf->st_gid);
	printf ("st_rdev:      %d\n", buf->st_rdev);
	printf ("st_size:      %d\n", buf->st_size);
	printf ("st_atime:     %d/%s", buf->st_atime, ctime(&buf->st_atime));
	printf ("st_spare1:    %d\n", buf->st_spare1);
	printf ("st_mtime:     %d/%s", buf->st_mtime, ctime(&buf->st_mtime));
	printf ("st_spare2:    %d\n", buf->st_spare2);
	printf ("st_ctime:     %d/%s", buf->st_ctime, ctime(&buf->st_ctime));
	printf ("st_spare3:    %d\n", buf->st_spare3);
	printf ("st_blksize:   %d\n", buf->st_blksize);
	printf ("st_blocks:    %d\n", buf->st_blocks);
	printf ("st_spare4[0]: %d\n", buf->st_spare4[0]);
	printf ("st_spare4[1]: %d\n", buf->st_spare4[1]);
	printf ("\n");
}


int
main(argc, argv)
	int argc;
	char **argv;
{
	struct vnode *vp;

	USES_NVOP_GETATTR;
	int error;
	struct vattr va;
	int fd;
	struct stat sb;

	if (argc != 2) {
		printf("usage: %s filename \n", argv[0]);
		exit(1);
	};

	if (error = ook_lookupname(argv[1], NO_FOLLOW, NULL, &vp)) {
		perror("ook_lookupname");
		exit(1);
	};

	if (error = NVOP_GETATTR(vp, &va, NULL)) {
		perror("getvattr");
		exit(1);
	};
	print_vattr(&va);

	/* NEEDSWORK: should be a subroutine in i405 library. */
	if (error = syscall(SYS_vnode_to_fd, vp, O_RDONLY, &fd)) {
		perror("vnode_to_fd");
		exit(1);
	};

	if (error = ook_vn_rele(vp)) {
		perror("ook_vn_rele");
		exit(1);
	};

	if (error = fstat(fd, &sb)) {
		perror("fstat");
		exit(1);
	};
	print_stat(&sb);

	exit(0);
}
