
/*
 * Copyright (c) 1991,1992 The Regents of the University of California.
 * All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * John Heidemann of the Ficus project at UCLA.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*
 * i405.h  -- the new vnode interface
 *
 *	@(#)i405.h	2.30 (UCLA Ficus Project) 10/10/94
 *
 */

#ifndef _i405_i405_h
#define _i405_i405_h 

#define I405

#ifdef sun
#include <i405/i405_sunos41.h>
/* Standard sun-os interface is brought in at the very end. */
#endif

/* Function prototypes are cool. */
#if defined(KERNEL) && !defined(STANDALONE_OOK)
#include <ficus/kernel_prototypes.h>
#endif

/* two more files are included at the end */

/*
 * flags for vdesc_flags:
 */
#define VDESC_MAX_VPS		16
/*
 * #define VDESC_NOMAP_VP1	0x0001 NOT USED---first vp must always be mapped */
#define VDESC_NOMAP_VP2		0x0002
/* low order 16-bits are reserved for additional vp arguments. */
#define VDESC_NOMAP_VPP		0x0100
#define VDESC_INVPP		0x0200	/* hack to handle SunOS's open which lacks a vp first arg */
#define VDESC_OLDVOP		0x0400  /* if this op doesn't pass args with new structure */

/*
 * VDESC_NO_OFFSET: Value in vdesc_vp[12p]_offset if there
 * is no such field.
 */
#define VDESC_NO_OFFSET -1

/*
 * This structure describes the vnode operation taking place.
 *
 */
struct vnodeop_desc {
	int vdesc_offset;   /* offset in vector--first in struct for speed */
	char *vdesc_name;   /* a user-readable string name of the offset */
	int vdesc_flags;    /* flags from above (VDESC_FOOFLAG) */

	/*
	 * These ops are used by bypass routines
	 * to map and locate arguments.
	 */
	int *vdesc_vp_offsets;   /* list terminated by VDESC_NO_OFFSET */
	int vdesc_vpp_offset;
	int vdesc_cred_offset;   /* creds are sometimes useful, so we locate them */
	int vdesc_proc_offset;   /* like creds, procs (in BSD) can be helpful */

	/*
	 * The following data is for transport layers aid.
	 * vdesc_datatype describes the arguments in detail.
	 * vdesc_uuid_p is a uuid that uniquely names the operation.
	 * (Although technically the uuid should be transport-layer
	 * specific, its kept here because it's common to several
	 * transport layers.)
	 */
	struct xport_datatype *vdesc_datatype;
	struct nca_uuid *vdesc_uuid_p;
	struct vnodeop_desc *vdesc_uuidhash;  /* see i405_uuid_to_desc() */
};




/*
 * A list of all the descs.
 */
extern struct vnodeop_desc *vnodeop_descs[];


/*
 * This macro is very helpful in defining those offsets in the vdesc struct.
 *
 * This is stolen from X11R4.  I ingored all the fancy stuff for
 * Crays, so if you decide to port this to such a serious machine,
 * you might want to consult Intrisics.h.
 */
#define VOPARG_OFFSET(p_type,field) \
        ((int) (((char *) (&(((p_type)NULL)->field))) - ((char *) NULL)))
#define VOPARG_OFFSETOF(s_type,field) VOPARG_OFFSET(s_type*,field)
#define VOPARG_OFFSETTO(S_TYPE,S_OFFSET,STRUCT_P) \
	((S_TYPE)(((char*)(STRUCT_P))+(S_OFFSET)))
/*
 * Alternatives for arrays.
 */
#define VOPARG_AOFFSET(p_type,field) \
        ((int) (((char *) (&(((p_type)NULL)->field[0]))) - ((char *) NULL)))
#define VOPARG_AOFFSETOF(s_type,field) VOPARG_AOFFSET(s_type*,field)

/*
 * A few more gross macros used for computing structure padding.
 */
#define VOPARG_INTERNALPAD(s_type,first_field,next_field,first_field_type) \
	(((char*)&(((s_type *)0)->next_field) \
	 - (char*)&(((s_type *)0)->first_field)) \
	 - sizeof(first_field_type))

#define VOPARG_ENDINGPAD(s_type,last_field,last_field_type) \
	(sizeof(s_type) \
	 - VOPARG_OFFSETOF(s_type,last_field) \
	 - sizeof(last_field_type))


/*
 * This structure is used to configure the new vnodeops vector.
 */
struct vnodeopv_entry_desc {
	struct vnodeop_desc *opve_op;   /* which operation this is */
	int (*opve_impl)();	/* code implementing this operation */
};
struct vnodeopv_desc {
	int (***opv_desc_vector_p)();
			/* ptr to the ptr to the vector where op should go */
	struct vnodeopv_entry_desc *opv_desc_ops;   /* null terminated list */
};





/*
 * Standards, standards, standards...
 */
#ifdef __STDC__
#define CONCAT2(A,B) A##B
#else
#define CONCAT2(A,B) A/**/B
#endif

/*
 * This macro is provided to declare variable length arrays.
 * It turns VARRAY(TYPE,NAME) into NAME{,_elems,_space}
 * NAME_elems is the number of used elements,
 * NAME_space is the number of elements in the pre-reserved buffer.
 * NAME is a pointer to the buffer.
 *
 * A vnodeop should never allocate space for NAME.  That
 * is the client's responsibility.
 */
#define VOUTARRAY(TYPE,NAME) \
	unsigned int CONCAT2(NAME,_elems), CONCAT2(NAME,_space); \
	TYPE *NAME
#define VINARRAY(TYPE,NAME) \
	unsigned int CONCAT2(NAME,_elems); TYPE *NAME


/*
 * A generic structure.
 * This is in several ways superceeded by the vdesc structure
 * and the VOPARG_OFFSET* macros.
 */
struct nvop_generic_args {
	struct vnodeop_desc *a_desc;
	/*
	 * Other random stuff follows---
	 * See a bypass routine on how to extract useful information
	 * (like vnodes).
	 */
};





#ifdef OUTOFKERNEL
/*
 * This call goes into the kernel with the user_vnodeop call.
 */
extern int user_new_vnodeop();
#define VKCALL(VP,UUIDP,AP) (user_vnodeop((VP),(UUIDP),(AP)))
#endif


/*
 * This call works for vnodes in the same address space.
 * NEEDSWORK: This call is a mess because we don't currently
 * change the definition of v_op in the vnode structure.
 */
#ifndef CHANGED_V_OP_DEFINITION
#define VACALL(VP,OFF,AP) (( *( ( (int(**)()) (VP)->v_op ) [OFF] ))(AP))
/*                   [ind.f.call][ptr.f.int] [opsvec....][offset] [args] */
#else /* CHANGED_V_OP_DEFINITION */
/* Call assuming v_op definiton has been changed (so we don't do the cast. */
#define VACALL(VP,OFF,AP) (( *( (VP)->v_op [OFF] ))(AP))
#endif /* CHANGED_V_OP_DEFINITION */

/*
 * A common bug when programming with vnode operations is
 * to invoke operations on a null pointer.  Unfortuntately
 * this results in a bad backtrace in the kernel.
 * To aid in debugging this sort of problem, 
 * define SAFE_VNODEOPS.
 *
 * This define should not be used in production code
 * because it adds significant cost to every vnode operation.
 *
 * (The idea for doing this was courtsey of <ratner@cs.ucla.edu>.)
 */
#ifdef SAFE_VNODEOPS
#undef VACALL
#define VACALL(VP,OFF,AP) ((VP) ? ( *( ( (int(**)()) (VP)->v_op ) [OFF] ))(AP) \
			   : safe_vnodeop_failure())
#endif /* SAFE_VNODEOPS */


/*
 * This is the call used in the vop invocation macros.
 * There are three cases:
 *    1) in kernel: VCALL == VACALL
 *    2) out of kernel with kernel environment simulation: VCALL == VACALL
 *    3) out of kernel without simulation ("stand-alone"): VCALL == VKCALL
 *
 * When you use VKCALL you must pass the UUID (because you're going
 * between address spaces) rather than the offset.
 */
#ifdef STANDALONE_OOK
#define VCALL(VP,OFF,AP) (VKCALL(VP,OFF,AP))
#define VDESC(OP) (NULL)
#define VOFFSET(OP) (&CONCAT2(OP,_uuid))
#else
#define VCALL(VP,OFF,AP) (VACALL(VP,OFF,AP))
#define VDESC(OP) (& CONCAT2(OP,_desc))
#define VOFFSET(OP) (VDESC(OP)->vdesc_offset)
#endif


#ifdef KERNEL
extern struct vnodeop_desc *i405_op_descs[];
extern int num_operations;
#endif /* KERNEL */


#include <i405/i405_transport.h>   /* must preceed i405_advanced_support.h */
#include <i405/sunos41_interface.h>
#include <i405/i405_basic_support.h>
#include <i405/i405_advanced_support.h>
#include <i405/i405_init.h>
#include <i405/i405_subr.h>
#include <i405/i405_syscalls.h>

#ifdef SVCM_TWO
#include <i405/SVCM/svcm.h>
#endif /* SVCM_TWO */

#ifdef I_FWL
#include <i405/FWL/fwl.h>
#endif /* I_FWL */

#endif /* _i405_i405_h */
