
/*
 * i405_alloc_xport.c
 *
 *	@(#)i405_alloc_xport.c	1.5 (UCLA Ficus Project) 8/25/93
 *
 * Copyright (c) 1993 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*
 * i405_alloc_xport.c is a the xport code which deals
 * with determining if structures require pre-allocation.
 */

#include <sys/types.h>
#include <sys/errno.h>
#include <rpc/types.h>
#include <rpc/xdr.h>
#include <sys/param.h>
#include <sys/ucred.h>
#include <sys/vnode.h>
#include <sys/vfs.h>
#include <i405/i405.h>
#include <nfs/nfs.h>
#include <nfs/export.h>
#include <nfs/rnode.h>   /* for vtofh */
#include <sys/user.h>
#ifdef OUTOFKERNEL
#include <i405/ook_compat.h>
#endif


#ifdef NOTDEF
#define DODEBUG(A) A
#else
#define DODEBUG(A)
#endif


int
i405_alloc_no(there, here, what_d, how, why)
	void *there;   /* IN */
	char *here;  /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	return 0;
}

int
i405_alloc_yes(there, here, what_d, how, why)
	void *there;   /* IN */
	char *here;  /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	return -1;
}


/*
 *--------------------------------------------------------------------------
 *
 * i405_alloc_structure -- fix allocation flags for structures
 *
 * DESCRIPTION:
 *	Walk through a structure's childen, setting ALLOC/FREE
 *	on the structure if any of its children require it.
 *
 *	This routine will result in an infinite loop for recursive
 *	data structures.  While not a feature, this should not be a
 *	problem.
 *
 * 	This same routine also handles unions.
 *	Note that unions are not actually allowed to be
 *	called with the alloc flag.  If we're 
 *	interpreting a union and we're going to walk it with
 *	why == WHY_ALLOC, we should panic.
 *
 * RETURN VALUES:
 *      Zero if no allocation is needed, -1 otherwise.
 *
 *--------------------------------------------------------------------------
 */
int debug_i405_alloc_structure = 0;
int
i405_alloc_structure(there, here, what_d, how, why)
	caddr_t there;   /* IN */
	caddr_t here;  /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	int i, error = 0;
	int num_elems = what_d->datatype_int;
	struct xport_datatype_instance *elems =
		(struct xport_datatype_instance *)what_d->datatype_ptr;
	int needs_alloc = 0;

	if (why != WHY_NEEDALLOC)
		panic("i405_alloc_structure");

	if (debug_i405_alloc_structure)
		printf ("i405_alloc_structure: what_d->name %s, why=%d\n", what_d->name, why);

	for (i = 0; i < num_elems; i++) {
		if (debug_i405_alloc_structure)
			printf ("i405_alloc_structure:   doing field %d, flags=%x\n", 
				i, elems[i].instance_flags);
		error = I405_XPORT_INTERPRET(
					     NULL,
					     NULL,
					     elems[i].datatype,
					     how,
					     why);
		if (error) {
			needs_alloc = -1;
			elems[i].instance_flags &=  (~WHY_UNKNOWN);
			elems[i].instance_flags |= WHY_ALLOC|WHY_FREE;
		} else {
			elems[i].instance_flags &=  (~WHY_UNKNOWN);
		};
	};

	if (needs_alloc) {
		DODEBUG(printf("i405_alloc_structure: entry %d (%s) needs alloc/free.\n", i, what_d->name));
	} else {
		DODEBUG(printf("i405_alloc_structure:  entry %d (%s) avoids alloc/free.\n", i, what_d->name));
	};

	return needs_alloc;
}



