
/*
 * i405_misccalls.c
 * A few random new system calls for development services.
 *
 *	@(#)i405_misccalls.c	1.2 (UCLA Ficus Project) 3/28/94
 *
 * Copyright (c) 1991,1992 The Regents of the University of California.
 * All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * John Heidemann of the Ficus project at UCLA.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */



#include <sys/param.h>
#include <sys/user.h>
#include <sys/uio.h>
#include <sys/file.h>
#include <sys/vfs.h>
#include <sys/vfs_stat.h>
#include <sys/buf.h>
#include <sys/vm.h>
#include <sys/socket.h>
#include <sys/systm.h>
#include <sys/time.h>
#include <sys/kernel.h>
#undef NFSSERVER
#include <sys/mount.h>
#include <sys/pathname.h>
#include <sys/vnode.h>
#ifdef UFS
#include <ufs/inode.h>
#endif
#include <sys/bootconf.h>
#include <sys/syslog.h>




/*
 * get filesystem statistics
 */
int
vfsstats(uap)
        struct a {
                char *path;
                struct vfsstats *buf;
		int reset;
        } *uap;
{
#ifdef VFSSTATS
        struct vnode *vp;
	struct vfs *vfsp;

        u.u_error = lookupname(uap->path, UIO_USERSPACE, FOLLOW_LINK,
            (struct vnode **)0, &vp);
        if (u.u_error)
                return;

	vfsp = vp->v_vfsp;

	if (vfsp->vfs_stats == NULL) {
		u.u_error = ENOMEM;
	};

	u.u_error = copyout((caddr_t)(vfsp->vfs_stats), uap->buf, sizeof(struct vfsstats));

	if (uap->reset)
		bzero(vfsp->vfs_stats, sizeof(struct vfsstats));

        VN_RELE(vp);
#else
	u.u_error = ESRCH;
#endif
}

 

/*
 * fs_type  a syscall to identify the file system type of a vnode.
 */

vnode_type (uap)
        struct a {
                char *path;
        } *uap;
{
        struct vnode *vp;
	struct vfs *vfsp;
	struct vfssw *vs;
	char *vfstype;

        u.u_error = lookupname(uap->path, UIO_USERSPACE, FOLLOW_LINK,
            (struct vnode **)0, &vp);
        if (u.u_error)
                return;

	vfsp = vp->v_vfsp;

	for (vs = vfssw; vs < vfsNVFS; vs++) {
		if (vs->vsw_name != 0 &&
		    vs->vsw_ops == vfsp->vfs_op)
			break;
	}
	if (vs == vfsNVFS)
		vfstype = "unknown";
	else vfstype = vs->vsw_name;

	log (LOG_ERR, "vnode at %x, refcount %d\n", vp, vp->v_count);
	log (LOG_ERR, "vfs at %x, type %s\n", vfsp, vfstype);

#ifdef SVCM_TWO
	pvn_dump_pages_with_text(vp, 8);
#endif /* SVCM_TWO */

        VN_RELE(vp);
}


