
/*-
 * i405_op_conf.c
 *
 *	$Id: i405_op_conf.c,v 1.94 1997/04/07 18:38:16 johnh Exp $ (UCLA Ficus Project) $Date: 1997/04/07 18:38:16 $
 *
 * Copyright (c) 1992 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 *
 * This is the user-changable configuration file for the new interface.
 * It parallels vfs_conf.c, but for the vnode interface rather than the
 * vfs inteface.
 *
 * BEWARE:  When adding a new operation, add it to both the external
 * declaration *and* the array of operations.
 *
 */

#include <sys/param.h>
#include <sys/time.h>
#include <sys/vnode.h>
#include <i405/i405.h>

#ifdef FICUS
#include <ficus/ficus.h>
#include <ficus/ficus_interface.h>
#ifdef VIEWFS
#include <ficus/viewfs/view_interface.h>
#else /*VIEWFS*/
#include <ficus/selectfs/select_interface.h>
#endif /*VIEWFS*/
#endif /* FICUS */

#ifdef SVCM_TWO
#include <i405/SVCM/svcm_interface.h>
#endif /* SVCM_TWO */

#ifdef CONSISTENTFS
#include <consistentfs/consistent_interface.h>
#endif /* CONSISTENTFS */

#ifdef I_FWL
#include <i405/FWL/fwl_interface.h>
#include <i405/FWL/specref_interface.h>
#include <i405/FWL/ro_interface.h>
#include <i405/FWL/fdnlc_interface.h>
#endif /* I_FWL */

extern struct vnodeop_desc
/* Ficus ops */
	nvop_newopen_desc,
	nvop_newclose_desc,
	nvop_newioctl_desc,
	nvop_newselect_desc,
	nvop_newaccess_desc,
	nvop_newremove_desc,
	nvop_newfsync_desc,
	nvop_newinactive_desc,
	nvop_newlockctl_desc,
	nvop_newgetpage_desc,
	nvop_newputpage_desc,
	nvop_newmap_desc,
	nvop_newdump_desc,
	nvop_newcmp_desc,
	nvop_newrealvp_desc,
	nvop_newcntl_desc,
	nvop_newfid_desc,
	nvop_newfid_to_vp_desc,
	nvop_fid_to_newfid_desc,
	nvop_get_proxy_desc,
	nvop_get_actual_desc,
#ifdef ENCRFS
	nvop_bogus_desc,
#endif
#ifdef EAFS
	eavop_getexattrs_desc,
	eavop_setexattrs_desc,
	eavop_getexattrs_info_desc,
	eavop_setexattrs_info_desc,
	eavop_delexattrs_desc,
	eavop_exattr_reg_strtoid_desc,
	eavop_exattr_strtoid_desc,
	eavop_exattr_idtostr_desc,
#endif
#ifdef IACCESS
	cvop_icreate_desc,
	cvop_ifree_desc,
	cvop_rdwr_attr_desc,
#endif
#ifdef UFSATTR
        cvop_aget_desc,
#endif
#ifdef JOHNH
	fvop_i405_union_test_desc,
#endif /*JOHNH*/
#ifdef IFS
	ivop_alloc_desc,
	ivop_free_desc,
	ivop_rdwrattr_desc,
	ivop_dirpref_desc,
#endif IFS
#if defined(FFG) || defined(FFD)
	ffdvop_get_server_desc,
#endif
#if defined(FFG) || defined(FFD) || defined(FFS)
	ffgvop_drop_desc,
	ffgvop_get_object_desc,
	ffgvop_inbuffer_desc,
	ffgvop_method_desc,
	ffgvop_outbuffer_desc,
#endif
/* and the default */
	/*
	 * DON'T ADD NEW OPS THIS WAY!
	 * intgen (as of 1-Oct-94) 
	 * defines xvop_foo_desc in your interface file's .h file.
	 * include the .h; don't make an external definiton here!
	 *    -JSH 1-Oct-94
	 */
	nvop_default_desc;



/*
 * This lists all vnodeoperations supported.  It's used in initialization
 * at compile time.  Position in this null-terminated list
 * determines vnode operation offset (possibly with compatibilty allowed).
 */

struct vnodeop_desc *i405_op_descs[] = {
#ifdef I_FWL
#ifdef BACKVINT
	/*
	 * For backwards compatibility,
	 * these operations must come first.
	 */
	&old_nvop_open_desc,
	&old_nvop_close_desc,
	&old_nvop_rdwr_desc,
	&old_nvop_ioctl_desc,
	&old_nvop_select_desc,
	&old_nvop_getattr_desc,
	&old_nvop_setattr_desc,
	&old_nvop_access_desc,
	&old_nvop_lookup_desc,
	&old_nvop_create_desc,
	&old_nvop_remove_desc,
	&old_nvop_link_desc,
	&old_nvop_rename_desc,
	&old_nvop_mkdir_desc,
	&old_nvop_rmdir_desc,
	&old_nvop_readdir_desc,
	&old_nvop_symlink_desc,
	&old_nvop_readlink_desc,
	&old_nvop_fsync_desc,
	&old_nvop_inactive_desc,
	&old_nvop_lockctl_desc,
	&old_nvop_fid_desc,
	&old_nvop_getpage_desc,
	&old_nvop_putpage_desc,
	&old_nvop_map_desc,
	&old_nvop_dump_desc,
	&old_nvop_cmp_desc,
	&old_nvop_realvp_desc,
	&old_nvop_cntl_desc,
#endif /* BACKVINT */
	&nvop_default_desc,
#else /* !I_FWL */
	&nvop_default_desc,   /* must be first */
#endif /* I_FWL */
	&nvop_open_desc,
	&nvop_close_desc,
	&nvop_rdwr_desc,
	&nvop_ioctl_desc,
	&nvop_select_desc,
	&nvop_getattr_desc,
	&nvop_setattr_desc,
	&nvop_access_desc,
	&nvop_lookup_desc,
	&nvop_create_desc,
	&nvop_remove_desc,
	&nvop_link_desc,
	&nvop_rename_desc,
	&nvop_mkdir_desc,
	&nvop_rmdir_desc,
	&nvop_readdir_desc,
	&nvop_symlink_desc,
	&nvop_readlink_desc,
	&nvop_fsync_desc,
	&nvop_inactive_desc,
	&nvop_lockctl_desc,
	&nvop_fid_desc,
	&nvop_getpage_desc,
	&nvop_putpage_desc,
	&nvop_map_desc,
	&nvop_dump_desc,
	&nvop_cmp_desc,
	&nvop_realvp_desc,
	&nvop_cntl_desc,
/* Ficus vnodeops */
#ifdef FICUS
	&fvop_debug_desc,
	&fvop_orphan_desc,
	&fvop_getfattrs_desc,
	&fvop_setfattrs_desc,
	&fvop_newcreate_desc,
	&fvop_newmkdir_desc,
	&fvop_newrename_desc,
        &fvop_newremove_desc,
	&fvop_replidalloc_desc,
	&fvop_addentry_desc,
	&fvop_openshadow_desc,
	&fvop_commitshadow_desc,
	&fvop_removeshadow_desc,
	&fvop_ffshlook_desc,
	&fvop_ffshremove_desc,
	&fvop_openqueue_desc,
	&fvop_openspecial_desc,
	&fvop_propdirentry_desc,
	&fvop_newpropdirentry_desc,
	&fvop_upddirentry_desc,
	&fvop_remdirentry_desc,
	&fvop_newremdirentry_desc,
	&fvop_propfile_desc,
	&fvop_proprename_desc,
	&fvop_propsymlink_desc,
	&fvop_control_desc,
	&fvop_dropvolcache_desc,
#ifdef INTGEN_TWO
#else
	&fvop_readunnamed_desc,
#endif
	&fvop_makeunnamedentry_desc, 
	&fvop_cntl_desc,
	&fvop_clearupdater_desc,
	&fvop_getupdater_desc,
#endif FICUS
	&nvop_newopen_desc,
	&nvop_newclose_desc,
	&nvop_newioctl_desc,
	&nvop_newselect_desc,
	&nvop_newaccess_desc,
	&nvop_newremove_desc,
	&nvop_newfsync_desc,
	&nvop_newinactive_desc,
	&nvop_newlockctl_desc,
	&nvop_newgetpage_desc,
	&nvop_newputpage_desc,
	&nvop_newmap_desc,
	&nvop_newdump_desc,
	&nvop_newcmp_desc,
	&nvop_newrealvp_desc,
	&nvop_newcntl_desc,
	&nvop_newfid_desc,
	&nvop_newfid_to_vp_desc,
	&nvop_fid_to_newfid_desc,
	&nvop_get_proxy_desc,
	&nvop_get_actual_desc,
#ifdef SVCM_TWO
	&nvop_stackgetpage_desc,
	&nvop_stackputpage_desc,
	&nvop_stackrdwr_desc,
	&nvop_cachenamevp_desc,
	&nvop_cache_callback_desc,
#endif /* SVCM_TWO */
#ifdef CONSISTENTFS
	&nvop_get_token_desc,
#endif
#ifdef ENCRFS
	&nvop_bogus_desc,
#endif
#ifdef EAFS
	&eavop_getexattrs_desc,
	&eavop_setexattrs_desc,
	&eavop_getexattrs_info_desc,
	&eavop_setexattrs_info_desc,
	&eavop_delexattrs_desc,
	&eavop_exattr_reg_strtoid_desc,
	&eavop_exattr_strtoid_desc,
	&eavop_exattr_idtostr_desc,
#endif
#ifdef IACCESS
	&cvop_icreate_desc,
	&cvop_ifree_desc,
	&cvop_rdwr_attr_desc,
#endif
#ifdef UFSATTR
	&cvop_aget_desc,
#endif
#ifdef FICUS
#ifdef VIEWFS
	&viewop_update_volumeinfo_desc,
#else /* ! VIEWFS */
	&selectvop_getfattrs_desc,
#endif /* VIEWFS */
	&fvop_update_link_desc,
	&fvop_subgetfattrs_desc,
	&fvop_graft_desc,	/* these last three used to be MSELECT */
	&fvop_ungraft_desc,
	&fvop_replica_lookup_desc,
	&fvop_switch_graft_desc,
#endif /* FICUS */
#ifdef JOHNH
	&fvop_i405_union_test_desc,
#endif /*JOHNH*/
#ifdef IFS
	&ivop_alloc_desc,
	&ivop_free_desc,
	&ivop_rdwrattr_desc,
	&ivop_dirpref_desc,
#endif IFS
#if defined(FFG) || defined(FFD)
	&ffdvop_get_server_desc,
#endif
#if defined(FFG) || defined(FFD) || defined(FFS)
	&ffgvop_drop_desc,
	&ffgvop_get_object_desc,
	&ffgvop_inbuffer_desc,
	&ffgvop_method_desc,
	&ffgvop_outbuffer_desc,
#endif
#ifdef I_FWL
	&nvop_get_fsindex_desc,
	&nvop_get_svcm_name_desc,
	&specref_doubled_nvop_lookup_desc,
	&specref_doubled_nvop_create_desc,
	&ro_doubled_nvop_rdwr_desc,
	&fdnlc_doubled_nvop_lookup_desc,
	&fdnlc_doubled_nvop_rename_desc,
	&fdnlc_doubled_nvop_remove_desc,
	&fdnlc_doubled_nvop_rmdir_desc,
#endif /* I_FWL */
	NULL
};



int num_operations;
#ifdef BACKVINT
int num_back_operations;
#endif

