
/*
 * i405_opv_conf.c
 *
 *	@(#)i405_opv_conf.c	1.33 (UCLA Ficus Project) 9/19/95
 *
 * Copyright (c) 1992 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * This is the user-changable configuration file for the new interface.
 * It parallels vfs_conf.c, but for the vnode interface rather than the
 * vfs inteface.
 *
 * BEWARE:  When adding a new operation, add it to both the external
 * declaration _and_ the array of operations.
 *
 */

#include <sys/param.h>
#include <sys/time.h>
#include <sys/vnode.h>
#include <i405/i405.h>



extern struct vnodeopv_desc i405_dead_vnodeop_opv_desc;

#if defined(NFSCLIENT) && defined(NFS_CONVERTED)
extern struct vnodeopv_desc nfs_vnodeop_opv_desc;
#endif

#if defined(UFS) && defined(UFS_CONVERTED)
extern struct vnodeopv_desc ufs_vnodeop_opv_desc;
#endif


#ifdef FICUS
extern struct vnodeopv_desc flfs_vnodeop_opv_desc;
extern struct vnodeopv_desc flfs_raw_vnodeop_opv_desc;
extern struct vnodeopv_desc flfs_control_vnodeop_opv_desc;
extern struct vnodeopv_desc flfs_unnamed_vnodeop_opv_desc;

extern struct vnodeopv_desc fpfs_vnodeop_opv_desc;
extern struct vnodeopv_desc fpfs_queue_vnodeop_opv_desc;
extern struct vnodeopv_desc fpfs_unnamed_vnodeop_opv_desc;
extern struct vnodeopv_desc fpfs_norepl_vnodeop_opv_desc;

extern struct vnodeopv_desc ficus_nfs_vnodeop_opv_desc;
#endif

#ifdef NULLFS
extern struct vnodeopv_desc null_vnodeop_opv_desc;
#endif

#ifdef MINIMALFS
#include <minimalfs/minimalfs_vnodeops.h>
#endif /* MINIMALFS */

#ifdef SELECTFS
extern struct vnodeopv_desc select_vnodeop_opv_desc;
#endif /* SELECTFS */

#ifdef VIEWFS
extern struct vnodeopv_desc view_vnodeop_opv_desc;
#endif /* VIEWFS */

#ifdef NEWTOOLDFS
extern struct vnodeopv_desc newtoold_vnodeop_opv_desc;
#endif

#ifdef NEWOFS
extern struct vnodeopv_desc newo_vnodeop_opv_desc;
#endif

#ifdef EAFS
extern struct vnodeopv_desc ea_vnodeop_opv_desc;
extern struct vnodeopv_desc testea_vnodeop_opv_desc;
#endif

#ifdef SHRINKFIDFS
extern struct vnodeopv_desc shrinkfid_vnodeop_opv_desc;
#endif

#ifdef CONSISTENTFS
extern struct vnodeopv_desc consistent_vnodeop_opv_desc;
#endif

#ifdef UMAPFS
extern struct vnodeopv_desc umap_vnodeop_opv_desc;
#endif

#ifdef IFS
extern struct vnodeopv_desc ifs_vnodeop_opv_desc;
#endif

#ifdef U2FS
extern struct vnodeopv_desc u2fs_vnodeop_opv_desc;
#endif

#ifdef FFG
extern struct vnodeopv_desc ffd_vnodeop_opv_desc;
extern struct vnodeopv_desc ffd_svr_vop_opv_desc;
#endif

/*
 *
 * i405_operation_vector_defns defines the operations
 * supported by each vnode type.  It is consulted at system
 * boot to build operation vectors.
 * It's also null terminated.
 *
 * Out-of-kernel, someone else (more knowlegable about what file
 * systems live in this address space) must specify this table.
 */

struct vnodeopv_desc *i405_opv_descs[] = {
	&i405_dead_vnodeop_opv_desc,

#if defined(UFS) && defined(UFS_CONVERTED)
	&ufs_vnodeop_opv_desc,
#endif

#if defined(NFSCLIENT) && defined(NFS_CONVERTED)
	&nfs_vnodeop_opv_desc,
#endif

#ifdef FICUS
	&flfs_vnodeop_opv_desc,
	&flfs_raw_vnodeop_opv_desc,
	&flfs_control_vnodeop_opv_desc,
	&flfs_unnamed_vnodeop_opv_desc,

	&fpfs_vnodeop_opv_desc,
	&fpfs_queue_vnodeop_opv_desc,
	&fpfs_unnamed_vnodeop_opv_desc,
	&fpfs_norepl_vnodeop_opv_desc,

	&ficus_nfs_vnodeop_opv_desc,
#endif

#ifdef NULLFS
	&null_vnodeop_opv_desc,
#endif

#ifdef MINIMALFS
	&minimalfs_vnodeop_opv_desc,
#endif

#ifdef SELECTFS
	&select_vnodeop_opv_desc,
#endif /*SELECTFS*/

#ifdef VIEWFS
	&view_vnodeop_opv_desc,
#endif /* VIEWFS */

#ifdef NEWTOOLDFS
	&newtoold_vnodeop_opv_desc,
#endif NEWTOOLDFS

#ifdef NEWOFS
	&newo_vnodeop_opv_desc,
#endif

#ifdef EAFS
	&ea_vnodeop_opv_desc,
	&testea_vnodeop_opv_desc,
#endif

#ifdef SHRINKFIDFS
	&shrinkfid_vnodeop_opv_desc,
#endif

#ifdef CONSISTENTFS
	&consistent_vnodeop_opv_desc,
#endif

#ifdef UMAPFS
	&umap_vnodeop_opv_desc,
#endif

#ifdef IFS
	&ifs_vnodeop_opv_desc;
#endif

#ifdef U2FS
	&u2fs_vnodeop_opv_desc;
#endif

#ifdef FFG
	&ffd_vnodeop_opv_desc,
	&ffd_svr_vop_opv_desc,
#endif

	NULL
};

