
/*
 * i405_test.c
 *
 *	@(#)i405_test.c	1.2 (UCLA Ficus Project) 3/3/94
 *
 *
 * Copyright (C) 1992 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * User system calls to the new vnode interface.
 *
 * The uap's below aren't the ones that come directly from the
 * system call.  They're just pointers into user space
 * of the user's vnode args struct.
 *
 * Vnodes are given to the user level as file descriptiors.
 * This means that when a user program crashes, its vnodes
 * will be released.
 *
 * To support the null layer out-of-kernel (i.e., under nfs),
 * we need to be able to to get the vfs structure for
 * a vnode, and to do a vget on that vfs.  As a temporary
 * patch, these are new system calls.  The vfs is given
 * to the user as a pointer, but list of current mounted
 * vfs's is always checked to make sure the pointer is valid.
 *
 */
 
 
#include <sys/param.h>
#ifdef OUTOFKERNEL
#include "myuser.h"
#else
#include <sys/user.h>
#endif
#include <sys/file.h>
#include <sys/uio.h>
#include <sys/vfs.h>
#include <sys/vnode.h>
#include <i405/i405.h>
#include <rpc/types.h>
#include <rpc/xdr.h>
#include <sys/syslog.h>

#include <ficus/cdefs.h>
#ifdef BACKVINT
#include <i405/i405_backvint.h>
#endif
#ifdef INTGEN_TWO
#include <i405/i405_subr.h>
#include <i405/i405_utok_fdvp.h>
#endif

#include <ficus/ficus.h>
#include <ficus/ficus_interface.h>

#if 0
#define DODEBUG(A) A
#else
#define DODEBUG(A)
#endif



#ifdef JOHNH
int 
flfs_fvop_i405_union_test(ap)
	struct fvop_i405_union_test_args *ap;
{
	int foo;
	foo++;   /* this is just a place to hang a symbolic debugger */
	return 0;
}


/*
 *--------------------------------------------------------------------------
 *
 * test_getfattrs -- a by-hand utok getfattrs call
 *
 * DESCRIPTION:
 *	Implement getfattrs by hand to compare against INTGEN_{ONE,TWO}.
 *
 * RETURN VALUES:
 *      Returns error code in u.u_error (Sun's convention).
 *
 *--------------------------------------------------------------------------
 */
int /* really is void */
test_getfattrs(uap)
        struct a {
                int uvp;		/* IN: user-level vnode */
		struct fattr *ufap;	/* OUT: returned fattrs */
		struct ucred *cred;	/* IN: user-credentials */
        } *uap;
{
	struct vnode *vp = NULL;
	int error = 0;
	USES_FVOP_GETFATTRS;
	struct fattr ufa;
	struct fattr *fap = NULL;
	extern struct fattr *flfs_fattr_alloc();
	replicalist_t *replist;
	int replist_elems, replist_space;
	int copy_count;
	struct ucred *tmpcr = NULL;

	/*
	 * First get the vnode.
	 */
	if (error = fdtovp(uap->uvp, &vp))
		goto out;

	/*
	 * Cred garbage.
	 */
	if (uap->cred != NULL) {
		panic("test_getfattrs: non-NULL credentials yet supported.");
	} else {
		/*
		 * Emulate what happens for real.
		 */
		tmpcr = u.u_cred;
		u.u_cred = crgetcred();
	};

	fap = flfs_fattr_alloc();

	/*
	 * NEEDSWORK: Handle credentials.
	 * We don't; it's too much bother for demo code.
	 */

#ifdef BACKVINT
	/*
	 * Bail out if we're not on a i405-enabled file system.
	 * This grossness is required because we don't convert
	 * all file system types.
	 *
	 * To demonstrate the bug this avoids, try getfattrs on different
	 * file systems.  On UFS (which is converted) it will return
	 * EOPNOTSUP, but on /tmp it will crash the machine
	 * because tmpfs isn't converted.
	 */
	if (vp->v_op->vn_close != oldtonew_vnodeops.vn_close) {
		log (LOG_DEBUG, "test_getfattrs: new operation attempted on non-converted file system type.\n");
		error = EOPNOTSUPP;
		goto out;   /* don't do if/else because of #ifdef */
	};
#endif

	if (error = FVOP_GETFATTRS(vp, fap, u.u_cred))
		goto out;

	/*
	 * Send the results.
	 * Read the user's structure to get the pointer,
	 * patch up our structure with their pointers,
	 * clobber their structure with ours,
	 * put our pointers back,
	 * send out the array.
	 */
	if (error = copyin(uap->ufap, &ufa, sizeof(struct fattr)))
		goto out;
	/* save our stuff */
	replist = struct_fattr_fa_replist(fap);
	replist_elems = struct_fattr_fa_replist_elems(fap);
	replist_space = struct_fattr_fa_replist_space(fap);
	/* fix what we'll write */
	struct_fattr_fa_replist(fap) = struct_fattr_fa_replist(&ufa);
	struct_fattr_fa_replist_space(fap)=struct_fattr_fa_replist_space(&ufa);
	/* write it */
	error = copyout(fap, uap->ufap, sizeof(struct fattr));
	/* now put our pointers back */
	struct_fattr_fa_replist(fap) = replist;
	struct_fattr_fa_replist_space(fap) = replist_space;
	if (error)
		goto out;
	/* now the array */
	copy_count = struct_fattr_fa_replist_elems(fap);
	if (struct_fattr_fa_replist_space(fap) < copy_count)
		copy_count = struct_fattr_fa_replist_space(fap);
	if (error = copyout(struct_fattr_fa_replist(fap),
			    struct_fattr_fa_replist(&ufa),
			    copy_count * sizeof(replicalist_t)))
		goto out;
	/* done at last */
	
 out:
	if (tmpcr) {
		crfree(u.u_cred);
		u.u_cred = tmpcr;
		tmpcr = NULL;
	};
	if (fap) {
		flfs_fattr_free(fap);
		fap = NULL;
	};

	return u.u_error = error;
}

#endif /*JOHNH*/

