
/*
 * i405_transport.h
 *
 *	@(#)i405_transport.h	1.18 (UCLA Ficus Project) 9/24/93
 */


#ifndef __i405_transport_h
#define __i405_transport_h

#include <sys/uio.h>
/*
 * We define our own uio structure so
 * we can store copies of iov and iovcnt.
 * See the comment in i405_utok_struct_uio for details.
 */
struct i405_uio {
	struct uio iuio_uio;
	struct iovec *iuio_iov;
	int iuio_iovcnt;
};



/*
 * NEEDSWORK: Need more sophisticated string handle code.
 */
#define I405_MAXSTRLEN (256)   /* all strings are this long (sigh) */


/*
 * Array stuff:
 *
 * Four types of arrays are supported:
 * ia:  IMPLICIT ARRAY (et, ln) nm
 * iia: IMPLICIT ARRAY (et, ln) *nm
 * fa:  FIXED ARRAY (et) nm
 * va:  VARIABLE ARRAY (et) nm
 * (et = elem type, ln = length, nm = name, sn = structure name,
 * in = instance name)
 *
 * Intgen2 translates each into C code:
 * ia:	#define sn_nm(in) (in)->nm_a
 *	#define sn_nm_elems(in) ln
 *	et nm_a[sn_nm_elems];
 * iia:	#define sn_nm(in) (in)->nm_a
 *	#define sn_nm_elems(in) ln
 *	et *nm_a;
 * fa:	#define sn_nm(in) (in)->nm_a
 *	#define sn_nm_elems(in) (in)->nm_elems
 *	u_int nm_elems;
 *	et *nm_a;
 * va:	#define sn_nm(in) (in)->nm_a
 *	#define sn_nm_elems(in) (in)->nm_elems
 *	#define sn_nm_space(in) (in)->nm_space
 *	u_int nm_elems, nm_space;
 *	et *nm_a;
 *
 * In addition, if a complex array (fa or va) is embedded in a
 * vnode operation directly, a macro:
 *	#define pass_sn_nm(nm,elems)
 *   or #define pass_sn_nm(nm,elemsp,space)
 *
 * (Below we use caddr_t instead of void* to match match the internal
 * interfaces which require caddr_t to do pointer arithmatic.)
 */
struct i405_fixed_array {
	u_int fa_elems;
	int fa_ary[1];   /* actually unbounded length */
};
struct i405_indirect_fixed_array {
	u_int fa_elems;
	caddr_t fa_ary;
};
struct i405_indirect_variable_array {
	u_int va_elems, va_space;
	caddr_t va_ary;
};

/*
 * Unions have their own structures,
 * but we use this one to mimic them all.
 *
 * PORTABILITY WARNING: This assumes that everything will always line up on
 * "int" boundries.  This is not necessarily true on all architectures.
 */
struct i405_generic_union {
	int un_tag;
	int un_data;   /* actually is more complex */
};





/*
 * UCLA's transport information is guaranteed to come first.
 *
 * NEEDSWORK:  This is a hack.  A more sophisticated slot
 * allocation scheme is required, but has been avoided
 * now because there's currently no need.
 */
#ifdef TRANSPORT_UCLA_XDR
#undef TRANSPORT_UCLA_XDR
#endif
#define TRANSPORT_UCLA_XDR 0
#define TRANSPORT_UCLA_UTOK 1
#define TRANSPORT_UCLA_ALLOC 2
/*
 * NEEDSWORK: FXDR is a slightly hacked XDR server for Ficus purposes.
 * It should go away.
 */
#define TRANSPORT_UCLA_FXDR 3


/*
 * Each parameter is context information:
 *   there - Information about "the other side".
 *		For XDR, this is a struct xdr *.
 *		For utok, this will be the remote pointer stack.
 *   here - Information about "this side".
 *		Where the data will go or already is.
 *   what - What the data is.  struct xport_datatype_instance.
 *   how - What protocol we're using.  An offset into the protocol ops vector.
 *   why - Why we're doing this: one of alloc, free, in_encode, in_decode,
 * 		out_encode, out_decode.
 */
/*			     caddr_t  caddr_t  struct xport_datatype*,
			     				int, int */
#define I405_XPORT_INTERPRET(there,   here,    what_d,    how, why) \
	((*(what_d)->fns[(how)]) ((there),(here),(what_d),(how),(why)))

/*
 * Why information.
 */
#define WHY_ALLOC	0x001
#define WHY_FREE	0x002
#define WHY_IN		0x004
#define WHY_OUT		0x008
#define WHY_NEEDALLOC	0x010
#define WHY_UNKNOWN	0x020	/* if we don't know if we need to allocate */
#define WHY_ENCODE	0x100
#define WHY_DECODE	0x200
#define WHY_MODE(why)		((why) & 0x0ff)
#define WHY_XDR_MODE(why)	((why) & 0xf00)
#define WHY_IN_ENCODE		(WHY_IN|WHY_ENCODE)
#define WHY_IN_DECODE		(WHY_IN|WHY_DECODE)
#define WHY_OUT_ENCODE		(WHY_OUT|WHY_ENCODE)
#define WHY_OUT_DECODE		(WHY_OUT|WHY_DECODE)
#define WHY_ENCODE_P(why)	(WHY_XDR_MODE(why) == WHY_ENCODE)
#define WHY_DECODE_P(why)	(WHY_XDR_MODE(why) == WHY_DECODE)
/* To compare to whys, we check to see if any of the modes match */
#define WHY_COMPATIBLE_P(W1,W2)	(WHY_MODE((W1) & (W2)) != 0)
#define WHY_IN_P(why)		(WHY_COMPATIBLE_P(why,WHY_IN))
#define WHY_OUT_P(why)		(WHY_COMPATIBLE_P(why,WHY_OUT))
#define WHY_ALLOC_P(why)	(WHY_COMPATIBLE_P(why,WHY_ALLOC))
#define WHY_FREE_P(why)		(WHY_COMPATIBLE_P(why,WHY_FREE))
#define WHY_NEEDALLOC_P(why)	(WHY_COMPATIBLE_P(why,WHY_NEEDALLOC))
#define WHY_UNKNOWN_P(why)	(WHY_COMPATIBLE_P(why,WHY_UNKNOWN))

#ifndef __PFI_defined
typedef int (*PFI)();   /* the standard Pointer to a Function returning an Int */
#define __PFI_defined
#endif

struct xport_datatype {
	/* struct nca_uuid id; */
	char *name;
	PFI *fns;
	int size;
	void *datatype_ptr;   /* addtional stuff needed by fns */
	int datatype_int;
		/* NEEDSWORK -- should these be instance specific? */
};

struct xport_datatype_instance {
	struct xport_datatype *datatype;
	int instance_flags;	/* WHY_* fields */
	int offset;		/* when things are embedded in structs */
};


/*
 * extern declarations
 */
extern int i405_excessive_array_elements;
extern PFI i405_indirect_fns[],
	i405_implicit_array_fns[],
	i405_indirect_implicit_array_fns[],
	i405_indirect_fixed_array_fns[],
	i405_indirect_variable_array_fns[];

#include <ficus/cdefs.h>

/* Pre-declare some types */
struct pathname;
struct ucred;

#include <i405/i405_transport__ah.h>

#include <i405/i405_alloc_xport.h>

#include <i405/i405_xdr_xport.h>
#include <i405/i405_xdr_fhvp__ah.h>

#include <i405/i405_utok_xport.h>
#include <i405/i405_utok_fdvp.h>

#include <i405/i405_fxdr_xport.h>
#include <i405/i405_fxdr_fhvp__ah.h>

#endif   /* __i405_transport_h */
