
/*
 * i405_utok_xport.c
 *
 *	@(#)i405_utok_xport.c	2.2 (UCLA Ficus Project) 11/4/93
 *
 * Copyright (c) 1993 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/types.h>
#include <sys/errno.h>
#include <rpc/types.h>
#include <rpc/xdr.h>
#include <sys/param.h>
#include <sys/ucred.h>
#include <sys/vnode.h>
#include <sys/vfs.h>
#include <i405/i405.h>
#include <nfs/nfs.h>
#include <nfs/export.h>
#include <nfs/rnode.h>   /* for vtofh */
#include <sys/user.h>
#ifdef OUTOFKERNEL
#include <i405/ook_compat.h>
#endif



#if 0
#define DODEBUG(A) A
#else
#define DODEBUG(A)
#endif



int
i405_utok_char(there, here, what_d, how, why)
	void *there;   /* IN */
	char *here;  /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	panic("i405_utok_char");
#if 0
	if (WHY_IN_P(why) || WHY_OUT_P(why))
		if (!xdr_int(there, here))
			return EINVAL;
	return 0;
#endif
}


/*
 *--------------------------------------------------------------------------
 *
 * i405_utok_copy -- copy data between user and kernel space
 *
 * DESCRIPTION:
 *	i405_utok_copy moves data from user to kernel space
 *	as part of the i405 transport layer prototcol.
 *	Data move directoy is based on the "why" argument.
 *
 * RETURN VALUES:
 *      Returns an errno value.
 *
 *--------------------------------------------------------------------------
 */
int
i405_utok_copy(there, here, what_d, how, why)
	caddr_t there;		/* IN: user space ptr we're copying from */
	caddr_t here;		/* IN: kernel space ptr we're copying to */
	struct xport_datatype *what_d;	/* IN: what we're copying */
	int how;		/* IN: how we're doing this (XDR or UTOK) */
	int why;		/* IN: why--are we copying IN, OUT or what? */
{
	int error;
	int size = what_d->size;

	if (WHY_IN_P(why)) {
		error = copyin(there, here, size);
	} else if (WHY_OUT_P(why)) {
		error = copyout(here, there, size);
	} else if (WHY_COMPATIBLE_P(why, WHY_ALLOC|WHY_FREE)) {
		return 0;
	} else {
		panic("i405_utok_copy: bad why.");
	};
	return error;
}


/*
 *--------------------------------------------------------------------------
 *
 * i405_utok_vnode_p -- map between fd's and vp's for OOK work
 *
 * DESCRIPTION:
 *	Map between fd's and vp's for OOK work.
 *
 *	Note: for the utok stuff, vnode_pp is just a normal reference
 * 	(by the grace of state).
 *
 * SEE ALSO:
 *	i405_xdr_vnode_p, i405_utok_vnode_pp.
 *
 * RETURN VALUES:
 *      Returns an errno value.
 *
 *--------------------------------------------------------------------------
 */
int
i405_utok_struct_vnode_p(there, here, what_d, how, why)
	struct vnode **there;	/* IN: user space ptr we're copying from */
	struct vnode **here;	/* IN: kernel space ptr we're copying to */
	struct xport_datatype *what_d;	/* IN: what we're copying */
	int how;		/* IN: how we're doing this (XDR or UTOK) */
	int why;		/* IN: why--are we copying IN, OUT or what? */
{
	int error;
	int uvp;   /* user space vnode (i.e., a file descriptor) */

	if (WHY_IN_P(why)) {
		if (error = copyin(there, &uvp, sizeof(int)))
			return error;
		error = fdtovp(uvp, here);
	} else if (WHY_OUT_P(why)) {
		if (error = vptofd(*here, &uvp))
			return error;
		error = copyout(&uvp, there, sizeof(int));
	} else if (WHY_FREE_P(why)) {
		error = fdvpfree(here);
	} else if (WHY_ALLOC_P(why)) {
		return 0;
	} else {
		panic("i405_utok_struct_vnode_p: bad why.");
	};
	return error;
}


/*
 *--------------------------------------------------------------------------
 *
 * i405_utok_structure -- utok structures around
 *
 * DESCRIPTION:
 *	Move structures around the utok world.
 *	Walk the structure, picking each element out of the
 *	list and filling it in to our structure.
 *
 *	This differs from i405_xdr_structure because we must move
 *	both the here and the there pointers ourself.
 *
 * SEE ALSO:
 *	i405_xdr_structure
 *
 * RETURN VALUES:
 *	An errno value.
 *
 *--------------------------------------------------------------------------
 */
int debug_i405_utok_structure = 0;
int
i405_utok_structure(there, here, what_d, how, why)
	caddr_t there; /* IN */
	caddr_t here;  /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	int i, error = 0;
	int num_elems = what_d->datatype_int;
	struct xport_datatype_instance *elems =
		(struct xport_datatype_instance *)what_d->datatype_ptr;

	if (WHY_UNKNOWN_P(elems[0].instance_flags))
		(void)i405_alloc_structure(NULL, NULL, what_d, TRANSPORT_UCLA_ALLOC, WHY_NEEDALLOC);

	if (debug_i405_utok_structure)
		printf ("i405_utok_structure: what_d->name %s, why=0x%x\n", what_d->name, why);
	for (i = 0; i < num_elems; i++) {
		if (debug_i405_utok_structure)
			printf ("i405_utok_structure:   %s do field %d, flags=%x\n",
				(WHY_COMPATIBLE_P(why, elems[i].instance_flags) ?
					"will" : "won't"),
				i, elems[i].instance_flags);
		/* Check for in/out match. */
		if (WHY_COMPATIBLE_P(why, elems[i].instance_flags)) {
			/* DODEBUG(printf("i405_utok_structure: elem %d done.\n", i)); */
			/* If a match, do the object. */
			if (error = I405_XPORT_INTERPRET(
					there + elems[i].offset,
					here + elems[i].offset,
					elems[i].datatype,
					how,
					why))
				return error;
		} else if (WHY_IN_P(why) || WHY_FREE_P(why)) {
			/*
			 * If we need to alloc, do so.
			 */
			if (WHY_IN_P(why) && WHY_ALLOC_P(elems[i].instance_flags)) {
				if (debug_i405_utok_structure)
					printf ("i405_utok_structure: alloc'ing field %d\n", i);
				if (error = I405_XPORT_INTERPRET(
					     there + elems[i].offset,
					     here + elems[i].offset,
					     elems[i].datatype,
					     how,
					     WHY_ALLOC))
					return error;
			};
			if (WHY_FREE_P(why) && WHY_FREE_P(elems[i].instance_flags)) {
				if (debug_i405_utok_structure)
					printf ("i405_utok_structure: free'ing field %d\n", i);
				if (error = I405_XPORT_INTERPRET(
					     there + elems[i].offset,
					     here + elems[i].offset,
					     elems[i].datatype,
					     how,
					     WHY_FREE))
					return error;
			};
		} else {
			/* DODEBUG(printf("i405_utok_structure: elem %d skipped.\n", i)); */
		};
	};
	return 0;
}


/*
 *--------------------------------------------------------------------------
 *
 * i405_utok_reference -- XDR pointers and what they point to around
 *
 * DESCRIPTION:
 *	XDR pointers and what they point to around.
 *
 * RETURN VALUES:
 *	An errno value.
 *
 *--------------------------------------------------------------------------
 */
int
i405_utok_reference(there, here, what_d, how, why)
	caddr_t *there; /* IN */
	caddr_t *here;  /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	struct xport_datatype *child_d = 
		(struct xport_datatype *) what_d->datatype_ptr;
	u_int size = child_d->size;
	caddr_t loc = *here;
	int error;
	caddr_t userp;

	if (loc == NULL) {
		if (WHY_FREE_P(why))
			return 0;
		*here = loc = (caddr_t) mem_alloc(size);
#ifndef KERNEL
		if (loc == NULL) {
			(void) fprintf(stderr,
			    "xdr_reference: out of memory\n");
			return ENOMEM;
		}
#else
		/* In the kernel mem_alloc guarantees success. */
#endif
		bzero(loc, size);
	};

	/*
	 * We now have space to store the result.
	 * Next, indirect, saving where we were in marker.
	 * NEEDSWORK: We don't really care that we save it.
	 * Stop this XDR holdover now?
	 */
	if (WHY_FREE_P(why))
		userp = NULL;
	else if (error = copyin(there, &userp, sizeof(void *)))
		return error;

	error = I405_XPORT_INTERPRET(userp, loc, child_d, how, why);

	if (WHY_FREE_P(why)) {
		mem_free(loc, size);
		*here = NULL;
	};

	return error;
}



/*
 *--------------------------------------------------------------------------
 *
 * i405_utok_handle_array_elems -- generic element code for array routines
 *
 * DESCRIPTION:
 *	Copy each array element in/out with copyin/out.
 *
 *	We special-case small array element types
 *	(chars and u_chars).
 *
 * SEE ALSO:
 *	<fill in>
 *
 * RETURN VALUES:
 *	An errno value.
 *
 *--------------------------------------------------------------------------
 */
int
i405_utok_handle_array_elems(there, here, what_d, count, how, why)
	caddr_t there;  /* IN */
	caddr_t here;   /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN: element type */
	int count;	/* IN: number of array elements to copy */
	int how;        /* IN */
	int why;        /* IN */
{
	int elsize = what_d->size;
	int error = 0;
	int i;
	
	if (count == 0)
		panic ("i405_utok_handle_array_elems: count == 0 sanity check.");
	
	if (what_d->size < sizeof(int)) {
		struct xport_datatype other_what_d;

		if (what_d != &xd_char && what_d != &xd_u_char) {
			/*
			 * This panic isn't strictly necessary.
			 */
			panic("i405_utok_handle_array_elems: special-case for small elems of unknown kind.");
		};
		other_what_d.size = elsize * count;
		return i405_utok_copy(there, here, &other_what_d, how, why);
	};

	for (i = 0; i < count; i++) {
		if (error = I405_XPORT_INTERPRET(there, here, what_d, how, why))
			return error;
		there += elsize;
		here += elsize;
	};

	return error;
}



/*
 *--------------------------------------------------------------------------
 *
 * i405_utok_implicit_array -- send implicit arrays around
 *
 * DESCRIPTION:
 *
 * SEE ALSO:
 *	i405_utok_indirect_implicit_array, i405_utok_implicit_array
 *
 * RETURN VALUES:
 *      Returns an errno value.
 *
 *--------------------------------------------------------------------------
 */
int
i405_utok_implicit_array(there, here, what_d, how, why)
	void *there;   /* IN */
	caddr_t here;  /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	return i405_utok_handle_array_elems(there, here,
		(struct xport_datatype *) what_d->datatype_ptr,
		what_d->datatype_int,
		how, why);
}

int
i405_utok_indirect_implicit_array(there, here, what_d, how, why)
	void *there;   /* IN */
	caddr_t *here;  /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	struct xport_datatype *elem_what_d =
			(struct xport_datatype *)what_d->datatype_ptr;
	int elem_count = what_d->datatype_int;
	int error;
	caddr_t userp;

	if (WHY_FREE_P(why)) {
		int elems = elem_count, space = elem_count;
		return i405_free_array(there, here,
				elem_what_d, &elems, &space, how, why);
	};

	/*
	 * May need to allocate space.
	 */
	if (WHY_IN_P(why) || WHY_ALLOC_P(why))
		if (error = i405_alloc_array(there, here, elem_what_d,
					     elem_count, how, why))
			return error;

	/*
	 * Always need to push down the user-space pointer.
	 */
	if (error = copyin(there, &userp, sizeof(void *)))
			return error;

	return i405_utok_handle_array_elems(userp, *here,
				elem_what_d, elem_count, how, why);
}

int
i405_utok_fixed_array(there, here, what_d, how, why)
	struct i405_fixed_array *there;   /* IN */
	struct i405_fixed_array *here;  /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	return EINVAL;
}

int
i405_utok_indirect_fixed_array(there, here, what_d, how, why)
	struct i405_indirect_fixed_array *there;   /* IN */
	struct i405_indirect_fixed_array *here;  /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	struct xport_datatype *elem_what_d =
			(struct xport_datatype *)what_d->datatype_ptr;
	int error;
	caddr_t userp;

	if (WHY_FREE_P(why)) {
		return i405_free_array(there, &here->fa_ary,
				elem_what_d, &here->fa_elems, &here->fa_elems,
				how, why);
	};

	/*
	 * First get the size.
	 */
	if (WHY_IN_P(why)) {
		if (error = copyin(&there->fa_elems, &here->fa_elems,
				   sizeof(here->fa_elems)))
			return error;
		if (here->fa_elems > i405_excessive_array_elements) {
			printf ("i405_utok_fixed_array: unusually high array length causes abort.\n");
			here->fa_elems = 0;
			return EINVAL;
		};
		if (here->fa_elems == 0)
			return 0;
	};

	/*
	 * May need to allocate space.
	 */
	if (WHY_IN_P(why) || WHY_ALLOC_P(why))
		if (error = i405_alloc_array(there, &here->fa_ary, elem_what_d,
					     here->fa_elems, how, why))
			return error;

	/*
	 * Always need to push down the user-space pointer.
	 */
	if (error = copyin(&there->fa_ary, &userp, sizeof(caddr_t)))
		return error;

	return i405_utok_handle_array_elems(userp, here->fa_ary,
				elem_what_d, here->fa_elems, how, why);
}

int
i405_utok_indirect_variable_array(there, here, what_d, how, why)
	struct i405_indirect_variable_array *there;   /* IN */
	struct i405_indirect_variable_array *here;  /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	struct xport_datatype *elem_what_d =
			(struct xport_datatype *)what_d->datatype_ptr;
	int error;
	caddr_t userp;

	DODEBUG(printf("i405_utok_variable_array: why=%x\n", why));
	if (WHY_FREE_P(why)) {
		return i405_free_array(there, &here->va_ary,
				elem_what_d, &here->va_elems, &here->va_space,
				how, why);
	};

	/*
	 * First get the size.
	 */
	if (WHY_COMPATIBLE_P(why, WHY_IN|WHY_OUT)) {
		if (WHY_IN_P(why))
			error = copyin(&there->va_elems, &here->va_elems, sizeof(here->va_elems));
		else error = copyout(&here->va_elems, &there->va_elems, sizeof(here->va_elems));
		if (error)
			return error;
		if (here->va_elems > i405_excessive_array_elements) {
			printf ("i405_xdr_variable_array: unusually high array elems causes abort.\n");
			here->va_elems = 0;
			return EINVAL;
		};
		if (here->va_elems == 0 && !WHY_IN_P(why))
			return 0;
	};
	if (WHY_COMPATIBLE_P(why, WHY_IN|WHY_ALLOC)) {
		if (error = copyin(&there->va_space, &here->va_space, sizeof(here->va_space)))
			return error;
		if (here->va_space > i405_excessive_array_elements) {
			printf ("i405_xdr_variable_array: unusually high array space causes abort.\n");
			here->va_space = 0;
			return EINVAL;
		};
		if (here->va_space == 0)
			return 0;
	};

	/*
	 * May need to allocate space.
	 */
	if (WHY_COMPATIBLE_P(why, WHY_IN|WHY_ALLOC) /* && WHY_DECODE_P(why) */)
		if (error = i405_alloc_array(there, &here->va_ary, elem_what_d,
					     here->va_space, how, why))
			return error;

	/*
	 * Always need to push down the user-space pointer.
	 */
	if (WHY_COMPATIBLE_P(why, WHY_IN|WHY_OUT)) {
                if (error = copyin(&there->va_ary, &userp, sizeof(caddr_t)))
                        return error;
		return i405_utok_handle_array_elems(userp, here->va_ary,
						    elem_what_d, here->va_elems, how, why);
	};

	return 0;
}



/*
 *--------------------------------------------------------------------------
 *
 * i405_utok_struct_uio -- XDR struct uio's around
 *
 * DESCRIPTION:
 *	Done by hand because of the list of iovecs.
 *
 * RETURN VALUES:
 *	An errno value.
 *
 *--------------------------------------------------------------------------
 */
int
i405_utok_struct_i405_uio(there, here, what_d, how, why)
	struct uio *there; /* IN: user-level pointer (actual type struct uio*) */
	struct i405_uio *here;  /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;           /* IN */
	int why;           /* IN */
{
	struct iovec *user_iov;
	int error;

	/*
	 * A simple implementation.  Doesn't always do the Right Thing.
	 *
	 * The UIO PROBLEM:
	 *
	 * Sun plays all sorts of games with the uio
	 * structure.  In particular, when doing scatter/gather I/O, Sun 
	 * will increment the uio->uio_iov pointer to point to where the 
	 * next bit of data should go.  This is Bad; it will cause a 
	 * memory loss.  Here's the scenario:
	 *
	 * Assume you're OOK, doing a scatter/gather rdwr on an in-kernel layer.
	 * First we have to XDR the uio into kernel space.  This happens
	 * fine, and the kernel does dynamic memory allocation to allocate
	 * the iov elements, leaving the *only pointer* to this memory in
	 * uio->uio_iov.
	 *
	 * Then we call the lower layer, say the UFS.  This calls uiomove,
	 * which ends up incrementing uio->uio_iov as it copies each region.
	 * The data is correctly copied out.
	 *
	 * Before we can return to the user, we must free up the in-kernel
	 * structures we allocated.  One of these is the uio and iov structs.
	 * Unforunately, the only pointer to the iov is uio->uio_iov, which
	 * now points one (structure element) beyond the end of the array.
	 * Furthermore, uio->uio_iovcnt == 0, so we don't know how big
	 * this array was.  We are therefore unable to free this memory.
	 * We'll call kmem_free with the end of the array and a size of zero.
	 *
	 * There are (at least) two solutions to this problem.  First the hack:
	 *
	 *
	 * When we allocate a uio structure, allocate a special one
	 * with extra fields for copies of the iov ptr and the iov cnt.
	 * Duplicate these.  Then when we go to free it, use the duplicates.
	 *
	 * A better solution would be to modify the kernel to not mess with
	 * iovcnt and iov, probably by using the array stuff of the new
	 * interface.  This is, of course, a fair bit of work.
	 *
	 *
         * With intgen v2 we now take the first approach, solving
	 * the memory leak.  -JSH 7-May-93
	 */
	
	if (WHY_IN_P(why)) {
		if (here->iuio_iov)
			panic("i405_uio: iov already allocated.");
		if (error = copyin(there, &here->iuio_uio, sizeof(struct uio)))
			return error;
		user_iov = here->iuio_uio.uio_iov;
		here->iuio_uio.uio_iov = here->iuio_iov = NULL;
		here->iuio_iovcnt = here->iuio_uio.uio_iovcnt;
		if (here->iuio_iovcnt > 256) {
			printf ("i405_utok_i405_uio: %d is too many iov's in the uio struct.\n", here->iuio_iovcnt);
			return EINVAL;
		};
		/*
		 * Allocate and fill the iov.
		 */
		here->iuio_uio.uio_iov = here->iuio_iov = (struct iovec*)
			mem_alloc(sizeof(struct iovec) * here->iuio_iovcnt);
		if (error = copyin(user_iov, here->iuio_iov,
				   sizeof(struct iovec) * here->iuio_iovcnt))
			return error;
	} else if (WHY_OUT_P(why)) {
		struct uio user_uio;

		user_uio = here->iuio_uio;
		/*
		 * Don't change most of the user's iov.
		 */
		if (error = copyin(&there->uio_iov, &user_uio.uio_iov,
				   sizeof(struct iov*)))
			return error;
		/*
		 * Actually, change it a little :->
		 * It gets incremented as iov's are chewed up.
		 */
		user_iov += (here->iuio_uio.uio_iov - here->iuio_iov);
		if (error = copyout(&user_uio, there, sizeof(struct uio)))
			return error;

		/*
		 * Finally, we have to update that residual value.
		 */
		if (error = copyout(&here->iuio_uio.uio_resid, &there->uio_resid,
				    sizeof(there->uio_resid)))
			return error;
	} else if (WHY_FREE_P(why)) {
		/*
		 * Free our saved copy of the iov ptr.
		 */
		if (here->iuio_iov)
			mem_free(here->iuio_iov, sizeof(struct iovec) * here->iuio_iovcnt);
		here->iuio_iov = NULL;
		here->iuio_iovcnt = 0;
		error = 0;
	} else {
		panic("i405_utok_i405_uio: bad why.");
	};
	return error;
}



/*
 *--------------------------------------------------------------------------
 *
 * i405_utok_struct_uio_p -- alloc/free uio ptrs
 *
 * DESCRIPTION:
 *	Alloc/free uio structures.
 *	Actually we allocate an i405_uio structure
 *	so we can play games.
 *
 *	This code is based on i405_utok_reference,
 *	but hacked for the different types.
 *
 * SEE ALSO:
 *	Something.
 *
 * RETURN VALUES:
 *      Returns an errno value.
 *
 *--------------------------------------------------------------------------
 */
int
i405_utok_struct_uio_p(there, here, what_d, how, why)
	struct uio **there;
	struct uio **here;
	struct xport_datatype *what_d;
	int how;
	int why;
{
	struct i405_uio *loc = (struct i405_uio*)*here;
	int error;
	struct uio *userp;

	if (loc == NULL) {
		if (WHY_FREE_P(why))
			return 0;
		loc = (struct i405_uio *) mem_alloc(sizeof(struct i405_uio));
		*here = (struct uio*)loc;
#ifndef KERNEL
		if (loc == NULL) {
			(void) fprintf(stderr,
			    "xdr_reference: out of memory\n");
			return ENOMEM;
		}
#else
		/* In the kernel mem_alloc guarantees success. */
#endif
		bzero(loc, sizeof(struct i405_uio));
	};

	/*
	 * We now have space to store the result.
	 * Next, indirect, saving where we were in marker.
	 * NEEDSWORK: We don't really care that we save it.
	 * Stop this XDR holdover now?
	 */
	if (WHY_FREE_P(why)) {
		userp = NULL;
	} else {
		if (error = copyin(there, &userp, sizeof(struct uio *)))
			return error;
	};

	error = i405_utok_struct_i405_uio(userp, loc, &xd_struct_i405_uio, how, why);

	if (WHY_FREE_P(why)) {
		mem_free(loc, sizeof(struct i405_uio));
		*here = NULL;
	};

	return error;
}



/*
 *--------------------------------------------------------------------------
 *
 * i405_utok_struct_ucred -- copy{in,ou} ucreds around
 *
 * SEE ALSO:
 *	Something.
 *
 * RETURN VALUES:
 *      Returns an errno value.
 *
 *--------------------------------------------------------------------------
 */
int
i405_utok_struct_ucred(there, here, what_d, how, why)
	struct ucred *there;
	struct ucred *here;
	struct xport_datatype *what_d;
	int how;
	int why;
{
	int error;
	u_short cr_ref;

	/* DODEBUG(printf("i405_utok_struct_ucred: called\n")); */
	if (WHY_IN_P(why)) {
		cr_ref = here->cr_ref;   /* save that refct */
		error = copyin(there, here, sizeof(struct ucred));
		here->cr_ref = cr_ref;
	} else if (WHY_OUT_P(why)) {
		if (error = copyin(&there->cr_ref, &cr_ref, sizeof(u_short)))
			return error;
		if (error = copyout(here, there, sizeof(struct ucred))) {
			/* NEEDSWORK: there's cr_ref is bad at this point. */
			return error;
		};
		error = copyout (&cr_ref, &there->cr_ref, sizeof(u_short));
	} else {
		panic("i405_utok_struct_ucred: bad why.");
	};
	return error;
}


/*
 *--------------------------------------------------------------------------
 *
 * i405_utok_struct_ucred_p -- XDR struct ucred's around, handling alloc/free
 *
 *
 * DESCRIPTION:
 *	This rouine is required by hand because of the list of uid/gids.
 *
 * RETURN VALUES:
 *	An errno value.
 *
 *--------------------------------------------------------------------------
 */
int
i405_utok_struct_ucred_p(there, here, what_d, how, why)
	struct ucred **there;
	struct ucred **here;
	struct xport_datatype *what_d;
	int how;
	int why;
{
	struct ucred *ucred;   /* pointer to cred in user-space */
	int error = 0;

	if (WHY_ALLOC_P(why)) {
		if (*here)
			panic ("i405_utok_struct_ucred_p: ALLOC of new cred over non-zero pointer.");
		*here = crget();
		return 0;
	} else if (WHY_FREE_P(why)) {
		if (*here != NULL) {
			crfree (*here);
			*here = NULL;
		};
		return 0;
	} else if (WHY_IN_P(why) || WHY_OUT_P(why)) {   /* IN or OUT */
		/*
		 * First figure out where the user has them
		 * (or wants them to go).
		 */
		if (error = copyin(there, &ucred, sizeof(struct ucred*)))
			return error;
		if (WHY_IN_P(why)) {
			if (ucred == NULL) {
				if (*here)
					crfree(*here);
				*here = crgetcred();
				return 0;
			} else {
				/*
				 * Alloc new cred (possibly).
				 */
				if (!*here)
					*here = crget();
			};
		};
		return i405_utok_struct_ucred(*there, *here, what_d, how, why);
	} else {
		panic("i405_utok_struct_ucred_p: bad why.");
	};
}



/*
 *--------------------------------------------------------------------------
 *
 * i405_utok_char_p -- XDR strings around
 *
 *
 * DESCRIPTION:
 *	This routine moves strings around.
 *	Currently strings have max length I405_MAXSTRLEN.
 *
 * RETURN VALUES:
 *	An errno code.
 *
 *--------------------------------------------------------------------------
 */
int
i405_utok_char_p(there, here, what_d, how, why)
	char **there;
	char **here;
	struct xport_datatype *what_d;
	int how;
	int why;
{
	int error = 0;
	char *ucp;  /* users char * */
	int realsize;

	if (WHY_FREE_P(why)) {
		if (*here){
			mem_free(*here, I405_MAXSTRLEN);
			*here = NULL;
		};
		return 0;
	};

	if (WHY_ALLOC_P(why) || WHY_IN_P(why))
		if (!*here)
			*here = mem_alloc(I405_MAXSTRLEN);
	if (WHY_ALLOC_P(why))
		return 0;

	if (error = copyin(there, &ucp, sizeof(char*)))
		return error;
	if (WHY_IN_P(why)) {
		if (!*here)
			*here = mem_alloc(I405_MAXSTRLEN);
		error = copyinstr(ucp, *here, I405_MAXSTRLEN, &realsize);
	} else {
		if (!WHY_OUT_P(why))
			panic ("i405_utok_char_p: bad why.");
		realsize = strlen(*here);
		if (realsize > I405_MAXSTRLEN)
			panic ("i405_utok_char_p: outgoing string too long.\n");
		error = copyout(*here, ucp, realsize);
	};
	return error;
}


/*
 *--------------------------------------------------------------------------
 *
 * i405_utok_union -- handle unions
 *
 * DESCRIPTION:
 *	i405_utok_union moves unions between user user and kernel space.
 *
 *      Restrictions:
 *
 *	      - All unions must have a tag.  Unknown tags return EINVAL.
 *
 *	      - Unions cannot be of type INOUT since then a user
 *		could change the tag and then loose memory
 *		in complex data structures.
 *
 *	      - If the union is of type OUT then no internal substructures
 *		can be variable length.  If they were, we'd have to
 *		send information about pre-allocated space
 *		but we don't know which one has pre-allocated space.
 *
 *	We assume that the intgen complier enforces these restrictions.
 *
 *	Caveats:
 *	Tag searching is done linearly and so may be expensive with
 *	lots of union types.
 *
 *	Information about what union types are supported are in 
 *	what_d->datatype_{ptr,int} == {elems, num_elems}.
 *	Each elem has the tag stored in elems[i].offset and
 *	the datatype in elems[i].datatype.
 *
 *	Note that we explicitly panic if why != WHY_{IN,OUT}.
 *	This is a feature; we expect a higher-level caller
 *	to make sure that we're not called with WHY_{ALLOC,FREE}
 *	since we can't deal with it.
 *
 * RETURN VALUES:
 *	An errno code.
 *
 *--------------------------------------------------------------------------
 */
int
i405_utok_union(there, here, what_d, how, why)
	struct i405_generic_union *there;   /* IN */
	struct i405_generic_union *here;  /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	int num_elems = what_d->datatype_int;
	struct xport_datatype_instance *elems =
		(struct xport_datatype_instance *)what_d->datatype_ptr;
	int error;
	caddr_t userp;
	int i;

	if (WHY_IN_P(why)) {
		error = copyin(&there->un_tag, &here->un_tag, sizeof(int));
	} else if (WHY_OUT_P(why)) {
		error = copyout(&here->un_tag, &there->un_tag, sizeof(int));
	} else if (WHY_FREE_P(why)) {
		return 0;   /* We can't free things, by fiat. */
	} else {
		panic ("i405_utok_union: called with why != WHY_IN|WHY_OUT.");
	};
	for (i = 0; i < num_elems; i++)
		if (here->un_tag == elems[i].offset) {
			return I405_XPORT_INTERPRET(
					(void*)&there->un_data,
					(void*)&here->un_data,
					elems[i].datatype,
					how,
					why);
		};
	/*
	 * Union tag was not found.  Bail.
	 */
	return EINVAL;
}

