
/*
 * i405_xdr_xport.c
 *
 *	@(#)i405_xdr_xport.c	1.24 (UCLA Ficus Project) 10/10/94
 *
 * Copyright (c) 1993 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/types.h>
#include <sys/errno.h>
#include <rpc/types.h>
#include <rpc/xdr.h>
#include <sys/param.h>
#include <sys/ucred.h>
#include <sys/vfs.h>
#include <sys/vnode.h>
#include <i405/i405.h>
#include <i405/i405_transport.h>
#include <nfs/nfs.h>
#include <nfs/export.h>
#include <nfs/rnode.h>   /* for vtofh */

#define INTGEN_ONE_COMPAT


#ifdef NOTDEF
#define DODEBUG(A) A
#else
#define DODEBUG(A)
#endif


int
i405_xdr_char(there, here, what_d, how, why)
	void *there;   /* IN */
	char *here;  /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	panic("i405_xdr_char");
#ifdef NOTDEF
	if (WHY_IN_P(why) || WHY_OUT_P(why))
		if (!xdr_int(there, here))
			return EINVAL;
	return 0;
#endif
}

int
i405_xdr_int(there, here, what_d, how, why)
	void *there;   /* IN */
	int *here;  /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	if (WHY_IN_P(why) || WHY_OUT_P(why))
		if (!xdr_int(there, here))
			return EINVAL;
	return 0;
}

int
i405_xdr_short(there, here, what_d, how, why)
	void *there;   /* IN */
	short *here;  /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	int i, result;
	if (WHY_ENCODE_P(why)) {
		i = *here;
		result = xdr_int(there, &i);
	} else if (WHY_DECODE_P(why)) {
		result = xdr_int(there, &i);
		*here = (short)i;
	} else
		return 0;
	return result ? 0 : EINVAL;
}

int
i405_xdr_u_short(there, here, what_d, how, why)
	void *there;   /* IN */
	u_short *here;  /* INOUT (gets different sign extension) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	int i, result;
	if (WHY_ENCODE_P(why)) {
		i = *here;
		result = xdr_int(there, &i);
	} else if (WHY_DECODE_P(why)) {
		result = xdr_int(there, &i);
		*here = i;
	} else
		return 0;
	return result ? 0 : EINVAL;
}


int
i405_xdr_struct_vnode_p(there, vp_p, what_d, how, why)
	void *there;   /* IN */
	struct vnode **vp_p; /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	if (WHY_IN_P(why)) {
		if (WHY_ENCODE_P(why)) {
			return i405_xdr_vptofh(there, vp_p);
		} else if (WHY_DECODE_P(why)) {
			return i405_xdr_fhtovp(there, vp_p);
		};
	};
	if (WHY_ALLOC_P(why))
		return 0;
	if (WHY_FREE_P(why))
		return i405_xdr_fhvpfree(vp_p);
	if (WHY_OUT_P(why))
		panic("i405_xdr_struct_vnode_p: vnode innn OUT mode.");
	panic("i405_xdr_struct_vnode_p");
}

int
i405_xdr_struct_vnode_pp(there, vpp_p, what_d, how, why)
	void *there;   /* IN */
	struct vnode ***vpp_p; /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	if (WHY_OUT_P(why)) {
		if (WHY_ENCODE_P(why)) {
			return i405_xdr_vpptofh(there, vpp_p);
		} else if (WHY_DECODE_P(why)) {
			return i405_xdr_fhtovpp(there, vpp_p);
		};
	};
	if (WHY_ALLOC_P(why)) {
		if (*vpp_p == NULL)
			*vpp_p=(struct vnode**)
				mem_alloc(sizeof(struct vnode**));
		**vpp_p = NULL;   /* zero the vpp */
		return 0;
	};
	if (WHY_FREE_P(why)) {
		/*
		 * This routine will free the pointer we allocated above.
		 */
		return i405_xdr_fhvppfree(vpp_p);
	};
	if (WHY_IN_P(why))
		panic("i405_xdr_struct_vnode_pp: vnode in IN mode.");
	panic("i405_xdr_struct_vnode_pp");
}



/*
 *--------------------------------------------------------------------------
 *
 * i405_xdr_structure -- XDR structures
 *
 * DESCRIPTION:
 *	XDR structures around.
 *	Walk the structure, picking each element out of the
 *	list and filling it in to our structure.
 *
 * SEE ALSO:
 *	i405_utok_structure
 *
 * RETURN VALUES:
 *	An errno value.
 *
 *--------------------------------------------------------------------------
 */
int debug_i405_xdr_structure = 0;
int
i405_xdr_structure(there, here, what_d, how, why)
	void *there;   /* IN */
	caddr_t here;  /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	int i, error = 0;
	int num_elems = what_d->datatype_int;
	struct xport_datatype_instance *elems =
		(struct xport_datatype_instance *)what_d->datatype_ptr;

	if (WHY_UNKNOWN_P(elems[0].instance_flags))
		(void)i405_alloc_structure(NULL, NULL, what_d, TRANSPORT_UCLA_ALLOC, WHY_NEEDALLOC);

	if (debug_i405_xdr_structure)
		printf ("i405_xdr_structure: what_d->name %s, why=0x%x\n", what_d->name, why);
	for (i = 0; i < num_elems; i++) {
		if (debug_i405_xdr_structure)
			printf ("i405_xdr_structure:   %s do field %d, flags=%x\n",
				(WHY_COMPATIBLE_P(why, elems[i].instance_flags) ?
					"will" : "won't"),
				i, elems[i].instance_flags);
		/* Check for in/out match. */
		if (WHY_COMPATIBLE_P(why, elems[i].instance_flags)) {
			/* If a match, do the object. */
			if (error = I405_XPORT_INTERPRET(there,
							 here + elems[i].offset,
							 elems[i].datatype,
							 how,
							 why))
				return error;
		} else if (WHY_IN_P(why) || WHY_FREE_P(why)) {
			/*
			 * If we need to alloc, do so.
			 */
			if (WHY_IN_P(why) && WHY_ALLOC_P(elems[i].instance_flags)) {
				if (debug_i405_xdr_structure)
					printf ("i405_xdr_structure: alloc'ing field %d\n", i);
				if (error = I405_XPORT_INTERPRET(
					     there,
					     here + elems[i].offset,
					     elems[i].datatype,
					     how,
					     WHY_ALLOC|(why&(WHY_ENCODE|WHY_DECODE))))
					return error;
			};
			if (WHY_FREE_P(why) && WHY_FREE_P(elems[i].instance_flags)) {
				if (debug_i405_xdr_structure)
					printf ("i405_xdr_structure: free'ing field %d\n", i);

				if (error = I405_XPORT_INTERPRET(
					     there,
					     here + elems[i].offset,
					     elems[i].datatype,
					     how,
					     WHY_FREE))
					return error;
			};
		} else {
			DODEBUG(printf("i405_xdr_structure: elem %d skipped.\n", i));
		};
	};
	return 0;
}



/*
 *--------------------------------------------------------------------------
 *
 * i405_xdr_reference -- XDR pointers and what they point to around
 *
 * DESCRIPTION:
 *	XDR pointers and what they point to around.
 *
 * RETURN VALUES:
 *	An errno value.
 *
 *--------------------------------------------------------------------------
 */
int
i405_xdr_reference(there, here, what_d, how, why)
	XDR *there;       /* IN */
	caddr_t *here;    /* INOUT */
	struct xport_datatype *what_d;   /* IN */
	int how;   /* IN */
	int why;   /* IN */
{
	struct xport_datatype *child_d = 
		(struct xport_datatype *) what_d->datatype_ptr;
	int size = child_d->size;
	register caddr_t loc = *here;
	int error;
	
	if (loc == NULL) {
		if (WHY_FREE_P(why))
			return 0;
		*here = loc = (caddr_t) mem_alloc(size);
#ifndef KERNEL
		if (loc == NULL) {
			(void) fprintf(stderr,
			    "xdr_reference: out of memory\n");
			return ENOMEM;
		};
#endif
		bzero(loc, size);
	};

	error = I405_XPORT_INTERPRET(there, loc, child_d, how, why);

	if (WHY_FREE_P(why)) {
		mem_free(loc, size);
		*here = NULL;
	};

	return error;
}




/*
 *--------------------------------------------------------------------------
 *
 * i405_xdr_handle_array_elems -- generic element XDR code for array routines
 *
 * DESCRIPTION:
 *	Copy each arry element in/out with XDR.
 *
 *	This routine handles packed arrays of characters,
 *	but it does not handle packed arrays of shorts (NEEDSWORK).
 *
 * SEE ALSO:
 *	<fill in>
 *
 * RETURN VALUES:
 *	An errno value.
 *
 *--------------------------------------------------------------------------
 */
int
i405_xdr_handle_array_elems(there, here, what_d, count, how, why)
	XDR *there;    /* IN */
	caddr_t here;   /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN: element type */
	int count;	/* IN: number of array elements to copy */
	int how;        /* IN */
	int why;        /* IN */
{
	int elsize = what_d->size;
	int error = 0;
	int i;

	if (count == 0)
		panic ("i405_xdr_handle_array_elems: count == 0 sanity check.");
	
	if (elsize < sizeof(int)) {
		if (elsize != 1)
			panic ("i405_xdr_handle_array_elems: sending short things without packing.");
		/*
		 * If we're getting charcater arrays, use xdr_opaque,
		 * since that will handle packing.
		 */
		if (!xdr_opaque (there, here, elsize * count))
			error = EINVAL;
	} else {   /* handle normal things */
		for (i = 0; i < count; i++) {
			DODEBUG(printf("i405_xdr_handle_array_elems: doing elem %d\n", i));
			if (error = I405_XPORT_INTERPRET(there, here, what_d, how, why))
				return error;
			here += elsize;
		};
	};
	return error;
}


/*
 *--------------------------------------------------------------------------
 *
 * i405_xdr_implicit_array -- send implicit arrays around
 *
 * DESCRIPTION:
 *
 * SEE ALSO:
 *	i405_xdr_indirect_implicit_array, i405_utok_implicit_array
 *
 * RETURN VALUES:
 *      Returns an errno value.
 *
 *--------------------------------------------------------------------------
 */
int
i405_xdr_implicit_array(there, here, what_d, how, why)
	void *there;   /* IN */
	caddr_t here;  /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	return i405_xdr_handle_array_elems(there, here,
		(struct xport_datatype *) what_d->datatype_ptr,
		what_d->datatype_int,
		how, why);
}

/*
 *--------------------------------------------------------------------------
 *
 * i405_xdr_indirect_implicit_array -- send indirect implicit arrays around
 *
 * DESCRIPTION:
 *	This routine differs from i405_xdr_implicit_array
 *	in that it must handle memory allocation.
 *
 * SEE ALSO:
 *	i405_xdr_implicit_array, i405_utok_indirect_implicit_array
 *
 * RETURN VALUES:
 *      Returns an errno value.
 *
 *--------------------------------------------------------------------------
 */
int
i405_xdr_indirect_implicit_array(there, here, what_d, how, why)
	void *there;   /* IN */
	caddr_t *here;  /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	struct xport_datatype *elem_what_d =
			(struct xport_datatype *)what_d->datatype_ptr;
	int elem_count = what_d->datatype_int;
	int error;

	if (WHY_FREE_P(why)) {
		int elems = elem_count, space = elem_count;
		return i405_free_array(there, here,
				elem_what_d, &elems, &space, how, why);
	};

	/*
	 * May need to allocate space.
	 */
	if (WHY_IN_P(why) && WHY_DECODE_P(why))
		if (error = i405_alloc_array(there, here, elem_what_d,
					     elem_count, how, why))
			return error;

	return i405_xdr_handle_array_elems(there, *here,
				elem_what_d, elem_count, how, why);
}

int
i405_xdr_fixed_array(there, here, what_d, how, why)
	void *there;   /* IN */
	struct i405_fixed_array *here; /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	return EINVAL;
}

int
i405_xdr_indirect_fixed_array(there, here, what_d, how, why)
	void *there;   /* IN */
	struct i405_indirect_fixed_array *here; /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	struct xport_datatype *elem_what_d =
			(struct xport_datatype *)what_d->datatype_ptr;
	int error;

	/*
	 * Free stuff?
	 */
	if (WHY_FREE_P(why)) {
		return i405_free_array(there, &here->fa_ary,
				elem_what_d, &here->fa_elems, &here->fa_elems,
				how, why);
	};

	/*
	 * First get the size.
	 */
	if (WHY_IN_P(why)) {
		if (!xdr_int(there, &here->fa_elems))
			return EINVAL;
		if (here->fa_elems > i405_excessive_array_elements) {
			printf ("i405_xdr_fixed_array: unusually high array length causes abort.\n");
			here->fa_elems = 0;
			return EINVAL;
		};
		if (here->fa_elems == 0)
			return 0;
	};

	/*
	 * May need to allocate space.
	 */
	if (WHY_IN_P(why) && WHY_DECODE_P(why))
		if (error = i405_alloc_array(there, &here->fa_ary,
				elem_what_d, here->fa_elems, how, why))
			return error;

	return i405_xdr_handle_array_elems(there, here->fa_ary,
				elem_what_d, here->fa_elems, how, why);
}

#ifdef INTGEN_ONE_COMPAT
int
i405_xdr_indirect_variable_array(there, here, what_d, how, why)
	void *there;   /* IN */
	struct i405_indirect_variable_array *here; /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	struct xport_datatype *elem_what_d =
			(struct xport_datatype *)what_d->datatype_ptr;
	int error;
	int elem_count;

	/*
	 * Free stuff?
	 */
	if (WHY_FREE_P(why)) {
		return i405_free_array(there, &here->va_ary,
				elem_what_d, &here->va_elems, &here->va_space,
				how, why);
	};

	/*
	 * This special compatibility version is just a hack.
	 */
	if (WHY_ALLOC_P(why)) {
		DODEBUG(printf("i405_xdr_array: doing space.\n"));
		if (!xdr_int(there, &here->va_space))
			return EINVAL;
		if (here->va_space > i405_excessive_array_elements) {
			printf ("i405_xdr_variable_array: unusually high array space causes abort.\n");
			here->va_elems = 0;
			return EINVAL;
		};
		if (here->va_space == 0)
			return 0;
		if (WHY_DECODE_P(why)) {
			DODEBUG(printf("i405_xdr_array: doing alloc.\n"));
			if (error = i405_alloc_array(there, &here->va_ary,
						     elem_what_d, here->va_space, how, why))
				return error;
		};
		return 0;
	};

	/*
	 * elems (and space, possibly, implicitly)
	 */
	if (WHY_COMPATIBLE_P(why,WHY_IN|WHY_OUT)) {
		DODEBUG(printf("i405_xdr_array: doing elems.\n"));
		if (!xdr_int(there, &here->va_elems))
			return EINVAL;
		if (here->va_elems > i405_excessive_array_elements) {
			printf ("i405_xdr_variable_array: unusually high array elems causes abort.\n");
			here->va_elems = 0;
			return EINVAL;
		};
		if (here->va_elems == 0 && !WHY_IN_P(why))
			return 0;
		/*
		 * Hack to handle IN things.
		 */
		if (here->va_ary == NULL) {
			here->va_space = here->va_elems;
			if (error = i405_alloc_array(there, &here->va_ary,
						     elem_what_d, here->va_space, how, why))
				return error;
		};
	};

	if (WHY_COMPATIBLE_P(why,WHY_IN|WHY_OUT)) {
		DODEBUG(printf("i405_xdr_array: doing handle of array elems.\n"));
		elem_count = here->va_elems;
		if (here->va_space < elem_count)
			elem_count = here->va_space;
		if (elem_count == 0)
			return 0;   /* don't do anything if there's not anything to do */
		return i405_xdr_handle_array_elems(there, here->va_ary,
				elem_what_d, elem_count, how, why);
	};
	return 0;
}
#else   /* INTGEN_ONE_COMPAT */
int
i405_xdr_indirect_variable_array(there, here, what_d, how, why)
	void *there;   /* IN */
	struct i405_indirect_variable_array *here; /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	struct xport_datatype *elem_what_d =
			(struct xport_datatype *)what_d->datatype_ptr;
	int error;
	int elem_count;

	/*
	 * Free stuff?
	 */
	if (WHY_FREE_P(why)) {
		return i405_free_array(there, &here->va_ary,
				elem_what_d, &here->va_elems, &here->va_space,
				how, why);
	};

	/*
	 * First get the size, both real and used.
	 */
	if (WHY_COMPATIBLE_P(why,WHY_IN|WHY_ALLOC)) {
		DODEBUG(printf("i405_xdr_array: doing space.\n"));
		if (!xdr_int(there, &here->va_space))
			return EINVAL;
		if (here->va_space > i405_excessive_array_elements) {
			printf ("i405_xdr_variable_array: unusually high array space causes abort.\n");
			here->va_space = 0;
			return EINVAL;
		};
		if (here->va_space == 0)
			return 0;
	};
	if (WHY_COMPATIBLE_P(why,WHY_IN|WHY_OUT)) {
		DODEBUG(printf("i405_xdr_array: doing elems.\n"));
		if (!xdr_int(there, &here->va_elems))
			return EINVAL;
		if (here->va_elems > i405_excessive_array_elements) {
			printf ("i405_xdr_variable_array: unusually high array elems causes abort.\n");
			here->va_elems = 0;
			return EINVAL;
		};
		if (here->va_elems == 0 && !WHY_IN_P(why))
			return 0;
	};

	/*
	 * May need to allocate space.
	 */
	if (WHY_COMPATIBLE_P(why,WHY_IN|WHY_ALLOC) && WHY_DECODE_P(why)) {
		DODEBUG(printf("i405_xdr_array: doing alloc.\n"));
		if (error = i405_alloc_array(there, &here->va_ary,
					     elem_what_d, here->va_space, how, why))
			return error;
	};

	if (WHY_COMPATIBLE_P(why,WHY_IN|WHY_OUT)) {
		DODEBUG(printf("i405_xdr_array: doing handle of array elems.\n"));
		elem_count = here->va_elems;
		if (here->va_space < elem_count)
			elem_count = here->va_space;
		if (elem_count == 0)
			return 0;
		else return i405_xdr_handle_array_elems(there, here->va_ary,
				elem_what_d, elem_count, how, why);
	};
	return 0;
}
#endif   /* INTGEN_ONE_COMPAT */


/*
 *--------------------------------------------------------------------------
 *
 * i405_xdr_struct_i405_uio -- XDR struct uio's around
 *
 * DESCRIPTION:
 *	Done by hand because of the list of iovecs.
 *
 * RETURN VALUES:
 *	An errno value.
 *
 *--------------------------------------------------------------------------
 */
int
i405_xdr_struct_i405_uio(there, here, what_d, how, why)
	void *there;   /* IN */
	struct i405_uio *here; /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	panic("i405_xdr_struct_i405_uio");
	/*NOTREACHED*/
}
int
i405_xdr_struct_uio_p(there, here, what_d, how, why)
	void *there;   /* IN */
	struct uio **here; /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	panic("i405_xdr_struct_uio_p");
	/*NOTREACHED*/
}

/*
 *--------------------------------------------------------------------------
 *
 * xdr_struct_pathname_p -- XDR pathnames around
 *
 *
 * DESCRIPTION:
 *	Sun doesn't currently support pathnames in its lookup interface,
 *	so neither do we.  This is just stub code.
 *
 * RETURN VALUES:
 *	An errno value.
 *
 *--------------------------------------------------------------------------
 */
int
i405_xdr_struct_pathname_p(there, pp, what_d, how, why)
	void *there;   /* IN */
	struct pathname **pp; /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	return 0;
}

/*
 *--------------------------------------------------------------------------
 *
 * i405_xdr_struct_ucred -- XDR struct ucred's contents
 *
 *
 * DESCRIPTION:
 *	This rouine is required by hand because of the list of uid/gids.
 *
 * RETURN VALUES:
 *	An errno value.
 *
 *--------------------------------------------------------------------------
 */
int
i405_xdr_struct_ucred(there, cred, what_d, how, why)
	void *there;   /* IN */
	struct ucred *cred; /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	int error;
	int i;

	DODEBUG(printf("xdr_struct_ucred: called\n"));
	if (error = i405_xdr_short(there, &cred->cr_uid, NULL, how, why))
		return error;
	if (error = i405_xdr_short(there, &cred->cr_gid, NULL, how, why))
		return error;
	if (error = i405_xdr_short(there, &cred->cr_auid, NULL, how, why))
		return error;
	if (error = i405_xdr_short(there, &cred->cr_ruid, NULL, how, why))
		return error;
	if (error = i405_xdr_short(there, &cred->cr_rgid, NULL, how, why))
		return error;
	/*
	 * What Sun grossosity.
	 * As of SunOS4.1, everything in the creds structure
	 * is u_short, except of course for the array of groups,
	 * which is int[].  Who really needs gid_t and uid_t?
	 */
	for (i = 0; i < NGROUPS; i++)
		if (error = i405_xdr_int(there, &cred->cr_groups[i], NULL, how, why))
			return error;
#ifdef NOTDEF
/* I'm not sure that this part works, so we just send them all. :-( */
		else if (cred->cr_groups[i] == 0)
			break;   /* 0 terminated list */
#endif
	/* NEEDSWORK: zero cr_label and cr_audit? */
	DODEBUG(printf ("Cred: uid=%d, gid=%d, g1=%d, g2=%d, g3=%d\n",
		cred->cr_uid, cred->cr_gid,
		cred->cr_groups[0], cred->cr_groups[1], cred->cr_groups[2]));
	return 0;
}

/*
 *--------------------------------------------------------------------------
 *
 * i405_xdr_struct_ucred_p -- XDR struct ucred's around, handling alloc/free
 *
 *
 * DESCRIPTION:
 *	This rouine is required by hand because of the list of uid/gids.
 *
 * RETURN VALUES:
 *	An errno value.
 *
 *--------------------------------------------------------------------------
 */
int
i405_xdr_struct_ucred_p(there, credp, what_d, how, why)
	void *there;   /* IN */
	struct ucred **credp; /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	register struct ucred *cred = *credp;

	DODEBUG(printf("xdr_struct_ucred_p called\n"));
	if (WHY_FREE_P(why)) {
		if (cred != NULL) {
			crfree(cred);
			*credp = NULL;
		};
		return 0;
	};
	if (cred == NULL && WHY_DECODE_P(why)) {
		*credp = cred = crget();
	};
	if (cred == NULL && WHY_ENCODE_P(why))
		panic("i405_xdr_struct_ucred_p: attempt to send null cred.");
	return i405_xdr_struct_ucred(there, *credp, NULL, how, why);
}




/*
 *--------------------------------------------------------------------------
 *
 * i405_xdr_char_p -- XDR strings around
 *
 *
 * DESCRIPTION:
 *	This routine moves strings around.
 *	Currently strings have max length I405_MAXSTRLEN.
 *
 * RETURN VALUES:
 *	An errno code.
 *
 *--------------------------------------------------------------------------
 */
int
i405_xdr_char_p(there, here, what_d, how, why)
	XDR *there;
	char **here;
	struct xport_datatype *what_d;
	int how;
	int why;
{
	if (WHY_FREE_P(why)) {
		if (*here){
			mem_free(*here, I405_MAXSTRLEN);
			*here = NULL;
		};
		return 0;
	};

	if (WHY_ALLOC_P(why) || WHY_IN_P(why))
		if (!*here)
			*here = mem_alloc(I405_MAXSTRLEN);

	if (WHY_IN_P(why) || WHY_OUT_P(why))
		return xdr_string(there, here, I405_MAXSTRLEN) ? 0 : EINVAL;

	return 0;
}




/*
 *--------------------------------------------------------------------------
 *
 * i405_xdr_union -- handle unions
 *
 * DESCRIPTION:
 *	i405_xdr_union moves unions between useer user and kernel space.
 *     
 *	See i405_utok_union for a complete description of the semantics
 *	and restrictions of unions.
 *
 * RETURN VALUES:
 *	An errno code.
 *
 *--------------------------------------------------------------------------
 */
int
i405_xdr_union(there, here, what_d, how, why)
	XDR *there;   /* IN */
	struct i405_generic_union *here;  /* INOUT (depends on the how argument) */
	struct xport_datatype *what_d;   /* IN */
	int how;       /* IN */
	int why;       /* IN */
{
	int num_elems = what_d->datatype_int;
	struct xport_datatype_instance *elems =
		(struct xport_datatype_instance *)what_d->datatype_ptr;
	int error;
	int i;

	if (WHY_COMPATIBLE_P(why,WHY_IN|WHY_OUT)) {
		if (error = i405_xdr_int(there, &here->un_tag, NULL, how, why))
			return error;
	} else if (WHY_FREE_P(why)) {
		return 0;   /* We can't free things, by fiat. */
	} else {
		panic ("i405_xdr_union: called with why != WHY_IN|WHY_OUT.");
	};
	for (i = 0; i < num_elems; i++)
		if (here->un_tag == elems[i].offset) {
			return I405_XPORT_INTERPRET(
					there,
					here /*NEEDSWORK */, 
					elems[i].datatype,
					how,
					why);
		};
	/*
	 * Union tag was not found.  Bail.
	 */
	return EINVAL;
}


