
/*
 * Copyright (c) 1991,1992 The Regents of the University of California.
 * All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * John Heidemann of the Ficus project at UCLA.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*
 * i405_op_conf.c
 *
 * @(#)i405_op_conf.c 1.27 92/07/20 UCLA Ficus project
 *
 * This is the user-changable configuration file for the new interface.
 * It parallels vfs_conf.c, but for the vnode interface rather than the
 * vfs inteface.
 *
 * BEWARE:  When adding a new operation, add it to both the external
 * declaration _and_ the array of operations.
 *
 */

#include <sys/param.h>
#include <sys/time.h>
#include <sys/vnode.h>
#include <i405/i405.h>


extern struct vnodeop_desc
	nvop_open_desc,
	nvop_close_desc,
	nvop_rdwr_desc,
	nvop_ioctl_desc,
	nvop_select_desc,
	nvop_getattr_desc,
	nvop_setattr_desc,
	nvop_access_desc,
	nvop_lookup_desc,
	nvop_create_desc,
	nvop_remove_desc,
	nvop_link_desc,
	nvop_rename_desc,
	nvop_mkdir_desc,
	nvop_rmdir_desc,
	nvop_readdir_desc,
	nvop_symlink_desc,
	nvop_readlink_desc,
	nvop_fsync_desc,
	nvop_inactive_desc,
	nvop_lockctl_desc,
	nvop_fid_desc,
	nvop_getpage_desc,
	nvop_putpage_desc,
	nvop_map_desc,
	nvop_dump_desc,
	nvop_cmp_desc,
	nvop_realvp_desc,
/* Ficus ops */
#ifdef FICUS
	fvop_debug_desc,
	fvop_orphan_desc,
	fvop_getfattrs_desc,
	fvop_setfattrs_desc,
	fvop_newcreate_desc,
	fvop_newrename_desc,
        fvop_newremove_desc,
	fvop_replidalloc_desc,
	fvop_addentry_desc,
	fvop_openshadow_desc,
	fvop_commitshadow_desc,
	fvop_removeshadow_desc,
	fvop_ffshlook_desc,
	fvop_ffshremove_desc,
	fvop_openqueue_desc,
	fvop_openunnamed_desc,
	fvop_propdirentry_desc,
	fvop_upddirentry_desc,
	fvop_remdirentry_desc,
	fvop_propfile_desc,
	fvop_proprename_desc,
	fvop_propsymlink_desc,
	fvop_control_desc,
	fvop_dropvolcache_desc,
	fvop_readunnamed_desc,
	fvop_makeunnamedentry_desc,
	fvop_cntl_desc,
	fvop_newrename_rat_desc,
	fvop_proprename_rat_desc,
#endif FICUS
#ifdef NEWOFS
	nvop_newopen_desc,
	nvop_newfid_desc,
	nvop_purge_cache_desc,
	nvop_newfid_to_vp_desc,
	nvop_lock_desc,
	nvop_unlock_desc,
#endif NEWOFS
#ifdef USE_NEWMAP
	nvop_newgetpage_desc,
	nvop_newputpage_desc,
#endif
#ifdef CONSISTENTFS
	nvop_get_token_desc,
#endif CONSISTENTFS
#ifdef ENCRFS
	nvop_bogus_desc,
#endif
/* and the default */
	nvop_default_desc;



/*
 * This lists all vnodeoperations supported.  It's used in initialization
 * at compile time.  Position in this null-terminated list
 * determines vnode operation offset (possibly with compatibilty allowed).
 */

struct vnodeop_desc *i405_op_descs[] = {
	&nvop_default_desc,   /* must be first */
	&nvop_open_desc,
	&nvop_close_desc,
	&nvop_rdwr_desc,
	&nvop_ioctl_desc,
	&nvop_select_desc,
	&nvop_getattr_desc,
	&nvop_setattr_desc,
	&nvop_access_desc,
	&nvop_lookup_desc,
	&nvop_create_desc,
	&nvop_remove_desc,
	&nvop_link_desc,
	&nvop_rename_desc,
	&nvop_mkdir_desc,
	&nvop_rmdir_desc,
	&nvop_readdir_desc,
	&nvop_symlink_desc,
	&nvop_readlink_desc,
	&nvop_fsync_desc,
	&nvop_inactive_desc,
	&nvop_lockctl_desc,
	&nvop_fid_desc,
	&nvop_getpage_desc,
	&nvop_putpage_desc,
	&nvop_map_desc,
	&nvop_dump_desc,
	&nvop_cmp_desc,
	&nvop_realvp_desc,
	&nvop_cntl_desc,
/* Ficus vnodeops */
#ifdef FICUS
	&fvop_debug_desc,
	&fvop_orphan_desc,
	&fvop_getfattrs_desc,
	&fvop_setfattrs_desc,
	&fvop_newcreate_desc,
	&fvop_newrename_desc,
        &fvop_newremove_desc,
	&fvop_replidalloc_desc,
	&fvop_addentry_desc,
	&fvop_openshadow_desc,
	&fvop_commitshadow_desc,
	&fvop_removeshadow_desc,
	&fvop_ffshlook_desc,
	&fvop_ffshremove_desc,
	&fvop_openqueue_desc,
	&fvop_openunnamed_desc,
	&fvop_propdirentry_desc,
	&fvop_upddirentry_desc,
	&fvop_remdirentry_desc,
	&fvop_propfile_desc,
	&fvop_proprename_desc,
	&fvop_propsymlink_desc,
	&fvop_control_desc,
	&fvop_dropvolcache_desc,
	&fvop_readunnamed_desc,
	&fvop_makeunnamedentry_desc, 
	&fvop_cntl_desc,
	&fvop_newrename_rat_desc,
	&fvop_proprename_rat_desc,
#endif FICUS
#ifdef NEWOFS
	&nvop_newopen_desc,
	&nvop_newfid_desc,
	&nvop_purge_cache_desc,
	&nvop_newfid_to_vp_desc,
	&nvop_lock_desc,
	&nvop_unlock_desc,
#endif
#ifdef USE_NEWMAP
	&nvop_newgetpage_desc,
	&nvop_newputpage_desc,
#endif
#ifdef CONSISTENTFS
	&nvop_get_token_desc,
#endif
#ifdef ENCRFS
	&nvop_bogus_desc,
#endif
	NULL
};



int num_operations;
#ifdef BACKVINT
int num_back_operations;
#endif

