
/*
 * vfs_nodemgt.c
 *      generic vnode management code
 *
 *	%W% (UCLA Ficus Project) %G%
 *
 * Copyright (c) 1992 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/user.h>
#include <sys/vnode.h>
#include <i405/i405.h>
#include <i405/i405_svcm.h>
#include <newofs/newo_interface.h>
#include <sys/vfs.h>
#include <sys/vfs_stat.h>
#include <sys/file.h>
#include <sys/uio.h>
#include <sys/buf.h>
#include <sys/kernel.h>
#include <sys/proc.h>
#include <sys/pathname.h>
#include <vm/seg.h>
#include <sys/dnlc.h>
#include <sys/syslog.h>

#ifdef USE_NEWMAP
#include <sys/mman.h>
#include <vm/hat.h>
#include <vm/as.h>
#include <vm/seg.h>
#include <vm/seg_vn.h>
#endif USE_NEWMAP

#include <nullfs/null_node.h>
#include <nullfs/null_info.h>


struct vnodemgt_instance {
	struct vnode **hash_buckets;   /* we assume that all hash values are in this range */
};

struct mgt_node {
	struct *forw, *back;   /* List of cached nodes.  Currently on forw is used. */
	int hash;
	void *mgt_data;   /* mgt specific data */
};


static void
add_nullnode_to_hash(mp, inst)
	struct mgt_node *mp;
	struct vnodemgt_instacne *inst;
{
	int hash = mp->hash;
	/*
	 * Put the null_node in the beginning of the bucket.
	 */
	mp->forw = inst->hash_buckets[hash];
	inst->hash_buckets[hash] = mp;
}
	
struct vnode *
vnodemgt_make_node (lvp, ninfo) 
	struct vnode *lvp;
	struct null_info *ninfo;
{
	struct vnode *vp;
	struct null_node *np;

	if (vp=null_find_nullnode(lvp, ninfo)) 
		return vp;
	/*
	 * Null_node is not present.
	 */
	np = (struct null_node *) kmem_alloc(sizeof (struct null_node));
	bzero((char *)np, sizeof (struct null_node));
	np->null_lowervp = lvp;
	np->null_flags = 0;
	/*
	 * Add np to the hash list.
	 */
	add_nullnode_to_hash(np, lvp);
	
	VTONULL(NULLTOV(np)) = np;
	VN_INIT(NULLTOV(np), NULLINFOTOVFS(ninfo), lvp->v_type,
		lvp->v_rdev); 
	NULLTOV(np)->v_op = (struct vnodeops *) null_vnodeop_p;
	/*
	 * All other vnode fields are zero.
	 */
	ninfo->info_refct++;
	return NULLTOV(np);
}

struct vnode *
null_find_nullnode (lvp, ninfo) 
	struct vnode *lvp;
	struct null_info *ninfo;
{
	struct vnode *vp;
	struct null_node *np;
        int hash = NULL_TO_HASH(lvp);

        /*
	 * Check the hash chain.
	 */
        np = null_hash[hash];
        while (np) {
                if ((NULLTOLOWERV(np)==lvp) &&
		    (NULLTOV(np)->v_vfsp==NULLINFOTOVFS(ninfo))) {
			VN_HOLD(NULLTOV(np));
                        return NULLTOV(np);
                }
		else np = np->null_next;
        };
	/*
	 * Null_node is not present.
	 */
	return NULL;
}

struct vnode *
null_free_nullnode (np)
	struct null_node *np;
{

	int hash = NULL_TO_HASH(NULLTOLOWERV(np));	
	struct null_node *ntablep;
	struct null_info *ninfo = VFSTONULLINFO(NULLTOV(np)->v_vfsp);

	if (ninfo->info_refct < 1) 
		panic ("null_free_nullnode: nullinfo refct is zero");

	ninfo->info_refct--;
        ntablep = null_hash[hash];
	if (ntablep == np) {
		null_hash[hash]=np->null_next;
	} else {
		while (ntablep) {
			if (ntablep->null_next==np) {
				ntablep->null_next=np->null_next;
				break;
			}
			else ntablep = ntablep->null_next;
		}
	}
	/*
	 * Np can be freed.
	 */
	kmem_free((caddr_t)np, sizeof(struct null_node));
}
		
	

#ifdef USE_NEWMAP
int
null_flush_nullnodes(vfsp, cred)
	struct vfs *vfsp;
	struct ucred *cred;
{
}
#endif
