
/*
 * mount_null
 *
 *	$Id: mount_null.c,v 1.10 1995/07/17 11:06:43 johnh Exp $ (UCLA Ficus Project) $Date: 1995/07/17 11:06:43 $
 *
 * Copyright (c) 1992 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *
 * mount a null file system.
 *
 *
 * compile me with <cc -I.. -o mount_null mount_null.c>
 */


#define USER_LEVEL_TAKES_PRECEDENCE

#include <sys/types.h>
#include <sys/vfs.h>
#include <sys/time.h>   /* for va_atime */
#include <sys/vnode.h>   /* for i405.h */
#include <i405/i405.h>   /* to get PFI */
#include <i405/i405_syscalls.h>
#include <sys/mount.h>
#include <nullfs/null_info.h>
#include <ficus/cdefs.h>
#include <ficus/fix_sunos_headers.h>

__dead void
usage(prog)
	char *prog;
{
	/* NEEDSWORK: should go to stderr. */
	printf ("usage: %s [-v] [-c | -C] [-F fwl] source target\n", prog);
	exit(1);
}


int nfwls = 0;
#define MFWLS 20
char *fwls[MFWLS];

int
main(argc, argv)
	int argc;
	char *argv[];
{
	char *fs_type = "null";
	char *source, *target;
	int  flags = M_NEWTYPE;
	int error, i, c;
	struct null_mountargs args;
	int use_cache = 1;
	int verbose = 1;
	extern char *optarg;
	extern int optind;

	if (argc < 3) {
		usage(argv[0]);
	};

	/*
	 * Parse user args.
	 */
	i = 1;
	while ((c = getopt(argc, argv, "cCF:v?")) != -1) {
		switch (c) {
		case 'c': use_cache = 1; break;
		case 'C': use_cache = 0; break;
		case 'F':
			if (nfwls >= MFWLS - 1) {
				printf("Too many featherweight layers.\n");
				exit(1);
			};
			fwls[nfwls++] = strdup(optarg);
			break;
		case 'v': verbose++; break;
		default: usage(argv[0]);
		};
	};

	if (argc - optind != 2) {
		usage(argv[0]);
	};
	source = argv[optind++];
	target = argv[optind++];
	fwls[nfwls] = NULL;   /* null terminate list */

	/*
	 * Setup mount call args.
	 */
	args.source = source;
	args.enable_dnlc = use_cache;

	if (verbose) {
		printf("calling mount(%s,%s,%d,<%s,%d>,",
		       fs_type, target, flags,
		       args.source, args.enable_dnlc);
		if (nfwls) {
			c = '[';
			for (i = 0; fwls[i]; i++) {
				printf("%c%s", c, fwls[i]);
				c = ',';
			};
			printf("])\n");
		} else {
			printf ("no fwls)\n");
		};
		
	};
	if (nfwls)
		error = syscall(SYS_fwl_mount, fs_type, target, flags, &args, fwls);
	else error = mount(fs_type, target, flags, &args);
	if (error)
		perror ("mount");

	exit (error ? 1 : 0);
}
