
/*
 * mount_null_ook
 *
 *  5-Feb-91 - stolen from flfs_mount.c - JSH
 */


#ifdef NFSCLIENT
#undef NFSCLIENT
#endif
#ifdef NFSSERVER
#undef NFSSERVER
#endif

#include <sys/param.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/errno.h>
#include <sys/time.h>
#include <sys/vnode.h>
#include <sys/file.h>
#define NFSCLIENT
#include <rpc/types.h>
#include <rpc/rpc.h>
#include <nfs/nfs.h>
#include <sys/mount.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>

#include <ficus/lib/fix_sunos_headers.h> 
#include <nullfs/null_port.h>


static struct sockaddr_in sin; /* static causes zero init */


#define OUR_TIMEOUT 20     /* timeout in this program */
#define NFS_SHORT_TIMEOUT 30   /* timeout for nfs (in the kernel) */
#define NFS_LONG_TIMEOUT 3000



/* 
 * Make an NFS lookup call to the OOK NFS server via RPC
 * to get a file handle for the root of the OOK file
 * system that is being mounted. We pass a file handle of 
 * "mount" to tell the server that this is not a normal lookup.
 */
int
getfh(fname,fhp,local)
	char *fname;
	fhandle_t *fhp;
	int local;
{
	struct timeval pertry_timeout, total_timeout;
	int sock = RPC_ANYSOCK;
	CLIENT *client, *clntudp_create();
	enum clnt_stat status;
	struct nfsdiropargs da;
	struct nfsdiropres dr;
	AUTH *authunix_create_default();

	pertry_timeout.tv_sec = OUR_TIMEOUT;
	pertry_timeout.tv_usec = 0;
	if ((client = clntudp_create(&sin, NFS_PROGRAM, NFS_VERSION, pertry_timeout, &sock)) == NULL) {
		clnt_pcreateerror("clntudp_create");
		exit(1);
	}

	total_timeout.tv_sec = OUR_TIMEOUT;
	total_timeout.tv_usec = 0;
	bzero(&da.da_fhandle, sizeof(fhandle_t));
	strncpy(&da.da_fhandle, "mount", strlen("mount"));
	da.da_name = fname;
   	client->cl_auth = authunix_create_default();
	status = clnt_call(client, RFS_LOOKUP, xdr_diropargs, &da,
				xdr_diropres, &dr, total_timeout);
	if(status != RPC_SUCCESS) {
		clnt_perror(client, "rpc");
		exit(2);
	}

	if(dr.dr_status != NFS_OK) {
		perror("getfh clnt_call");
		exit(3);
	}

	bcopy(&dr.dr_fhandle, fhp, sizeof(fhandle_t));
        auth_destroy(client->cl_auth);
	clnt_destroy(client);
	close(sock);
	return 0;
}
	
	



int
main(argc,argv)
	int argc;
	char *argv[];
{
	struct nfs_args args;
	fhandle_t localfh;
	char namebuf[MAXHOSTNAMELEN+1];
	struct hostent *hp, *gethostbyname();
	int i;
	int long_timeout = 0;
	int is_remount=0;
	char *source="source", *dest="dest";

	gethostname(namebuf,MAXHOSTNAMELEN);
	hp = gethostbyname(namebuf);

	sin.sin_family = AF_INET;
	sin.sin_port = htons(NULLFS_PORT);
	bcopy((char *)hp->h_addr, (char *)&sin.sin_addr,hp->h_length);
	bzero(sin.sin_zero, sizeof(sin.sin_zero));
	bzero(&args, sizeof(struct nfs_args));
	args.addr = &sin;
 
	/*
	 * Handle command line flags
	 */
	for (i = 1; i<argc && argv[i][0] == '-'; i++)
		switch (argv[i][1]) {
			case 'r':
				is_remount=1;
				break;
			case 'l':
				long_timeout = 1;
				break;
			default:
				printf ("Invalid option: %s\n", argv[i]);
				break;
			}

	if (argc - i != 1) {
		printf("usage: %s [-l] [-r] dest\n", argv[0]);
		exit(1);
	}

/*	source = argv[i++]; */
	dest = argv[i++];

	/* 
	 * Get file handles for all file systems being mounted.
	 * The first one is passed to the mount system call.
	 */
	if (getfh(source, &localfh, 1)) {	
			perror("getfh");
			exit(2);
	}

	args.fh = (caddr_t)&localfh;
	args.flags |= NFSMNT_TIMEO;
	args.timeo = long_timeout ? NFS_LONG_TIMEOUT : NFS_SHORT_TIMEOUT;
	args.flags |= NFSMNT_INT; /* make it interruptible */
	args.hostname = "null-hostname";
	args.netname = "null-netname";


	printf ("nfs mount to %s\n", dest);
	if(is_remount)
		printf ("Nfs mount skipped.\n");
	else if (mount("nfs", dest, M_NEWTYPE, &args)) {
		perror("mount");
		if (errno==EPERM) {
			printf ("You must be root to mount a file system.\n");
		};
		exit(3);
	}

	exit (0);
}


