#include <sys/param.h>
#include <sys/systm.h>
#include <sys/user.h>
#include <sys/vnode.h>
#include <i405/i405.h>
#include <i405/i405_svcm.h>
#include <newofs/newo_interface.h>
#include <sys/vfs.h>
#include <sys/vfs_stat.h>
#include <sys/file.h>
#include <sys/uio.h>
#include <sys/buf.h>
#include <sys/kernel.h>
#include <sys/proc.h>
#include <sys/pathname.h>
#include <vm/seg.h>
#include <sys/dnlc.h>
#include <sys/syslog.h>

#ifdef USE_NEWMAP
#include <sys/mman.h>
#include <vm/hat.h>
#include <vm/as.h>
#include <vm/seg.h>
#include <vm/seg_vn.h>
#endif USE_NEWMAP

#include <nullfs/null_node.h>
#include <nullfs/null_info.h>


#define NULL_HASH_BUCKETS 0x20
#define NULL_HASH_MASK (NULL_HASH_BUCKETS-1)
#define NULL_TO_HASH(LVP) ((((int)LVP)>>2) & NULL_HASH_MASK)
struct null_node * null_hash[NULL_HASH_BUCKETS];

extern int (**null_vnodeop_p)();
extern struct vnode * null_find_nullnode();

struct vnode * null_make_nullnode (lvp, ninfo) 
struct vnode *lvp;
struct null_info *ninfo;
{
	struct vnode *vp;
	struct null_node *np;
	if (vp=null_find_nullnode(lvp, ninfo)) 
		return vp;
	/* null_node not present */
	np = (struct null_node *) kmem_alloc(sizeof (struct null_node));
	bzero((char *)np, sizeof (struct null_node));
	np->null_lowervp = lvp;
	np->null_flags = 0;
	/* add np to the hash list */
	add_nullnode_to_hash(np, lvp);
	
	VTONULL(NULLTOV(np)) = np;
	VN_INIT(NULLTOV(np), NULLINFOTOVFS(ninfo), lvp->v_type,
		lvp->v_rdev); 
	NULLTOV(np)->v_op = (struct vnodeops *) null_vnodeop_p;
	/* All other vnode fields are zero.*/
	ninfo->info_refct++;
	return NULLTOV(np);
}

struct vnode * null_find_nullnode (lvp, ninfo) 
struct vnode *lvp;
struct null_info *ninfo;
{
	struct vnode *vp;
	struct null_node *np;
        int hash = NULL_TO_HASH(lvp);

        /* check the hash chain */
        np = null_hash[hash];
        while (np) {
                if ((NULLTOLOWERV(np)==lvp) &&
		    (NULLTOV(np)->v_vfsp==NULLINFOTOVFS(ninfo))){
			VN_HOLD(NULLTOV(np));
                        return NULLTOV(np);
                }
		else np = np->null_next;
        };
	/* null_node not present */
	return NULL;
}

add_nullnode_to_hash(np, lvp)
struct null_node *np;
struct vnode *lvp;
{
        int hash = NULL_TO_HASH(lvp);
	/* Put the null_node in the beginning of the bucket */
	np->null_next = null_hash[hash];
	null_hash[hash] = np;
}
	
struct vnode * null_free_nullnode (np)
struct null_node *np;
{

	int hash = NULL_TO_HASH(NULLTOLOWERV(np));	
	struct null_node *ntablep;
	struct null_info *ninfo = VFSTONULLINFO(NULLTOV(np)->v_vfsp);

	if (ninfo->info_refct < 1) 
		panic("null_free_nullnode: nullinfo refct is zero");

	ninfo->info_refct--;
        ntablep = null_hash[hash];
	if (ntablep == np) {
		null_hash[hash]=np->null_next;
	} else {
		while (ntablep) {
			if (ntablep->null_next==np) {
				ntablep->null_next=np->null_next;
				break;
			}
			else ntablep = ntablep->null_next;
		}
	}
	/* np can be freed */
	kmem_free((caddr_t)np, sizeof(struct null_node));
}
		
	

