
/*-
 *
 * null_vnodeops.c
 *
 *	$Id: null_debug.c,v 1.4 1995/07/17 11:06:17 johnh Exp $ (UCLA Ficus Project) $Date: 1995/07/17 11:06:17 $
 *
 * Copyright (c) 1992-1994 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Los Angeles and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 ***********************************************************************
 *
 * A null layer with the new interface.
 * Initial work by Yuguang Wu, DNLC, tuning by John Heidemann.
 * Vnode management code re-writen by Ashvin Goel to be SunOS-license
 * free.  SVCM interface by John Heidemann.
 *   
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/user.h>
#include <ficus/cdefs.h>
#include <sys/vnode.h>
#include <i405/i405.h>
#ifdef SVCM_TWO
#include <i405/SVCM/svcm.h>
#endif /* SVCM_TWO */
#include <sys/vfs.h>
#include <sys/vfs_stat.h>
#include <sys/file.h>
#include <sys/uio.h>
#include <sys/buf.h>
#include <sys/kernel.h>
#include <sys/proc.h>
#include <sys/pathname.h>
#include <vm/seg.h>
#include <sys/dnlc.h>
#include <sys/syslog.h>
#include <sys/debug.h>

#ifdef SVCM_TWO
#include <sys/mman.h>
#include <vm/hat.h>
#include <vm/as.h>
#include <vm/seg.h>
#include <vm/seg_vn.h>
#endif SVCM_TWO

#include <nullfs/null_node.h>
#include <nullfs/null_info.h>

#ifdef SVCM_TWO


#ifdef JOHNH_DATA_PAGE_DEBUGGING
static int null_do_panic;

void
null_debug_panic(s)
	char *s;
{
	if (null_do_panic > 0)   /* avoid recursive panics */
		return;
	null_do_panic++;
	/* A debugger can change null_do_panic back to zero here. */
	if (null_do_panic > 0) {
		panic(s);
	};
}

struct vnode *null_special_null_vp = NULL;
struct vnode *null_special_ufs_vp = NULL;

void
null_check_special_vp(ufs_vp)
	struct vnode *ufs_vp;
{
	if (null_special_ufs_vp == NULL)
		return;
	if (ufs_vp == null_special_ufs_vp)
		null_debug_panic("check_special_vp");
}

void
null_check_special_vps(vp)
	struct vnode *vp;
{
	static int foo;
	if (vp == NULL || (null_special_ufs_vp != vp && null_special_null_vp != vp))
		return;
	foo++;
}

void
null_check_ufs_vp(ufs_vp)
	struct vnode *ufs_vp;
{
	extern int (**ufs_vnodeop_p);

	if (null_special_ufs_vp == NULL)
		return;
	ASSERT(ufs_vp->v_op == (struct vnodeops*)ufs_vnodeop_p);
	if (ufs_vp->v_pages != NULL) {
		printf ("ufs vp: ");
		pvn_dump_pages(ufs_vp);
		null_debug_panic("check_ufs_vp");
	};
}

void
null_check_null_ufs_vp(vp)
	struct vnode *vp;
{
	struct null_node *np = VTONULL(vp);
	struct vnode *ufs_vp = np->null_name->vp;   /* hack */
	extern int (**ufs_vnodeop_p);

	if (null_special_ufs_vp == NULL || null_special_null_vp == NULL)
		return;
	if (null_do_panic)   /* avoid recursive panics */
		return;
	ASSERT(ufs_vp->v_op == (struct vnodeops*)ufs_vnodeop_p);
	if (ufs_vp->v_pages != NULL) {
		printf ("null vp: ");
		pvn_dump_pages(vp);
		printf ("ufs vp: ");
		pvn_dump_pages(ufs_vp);
		null_debug_panic("null_check_null_ufs_vp");
		};
	};
}
#endif /* JOHNH_DATA_PAGE_DEBUGGING */



#ifdef JOHNH_DATA_PAGE_LIST_VERIFY
/*-
 *--------------------------------------------------------------------------
 *
 * null_verify_pages
 *
 * DESCRIPTION:
 *	Check the null-node's page list against its svcm_byterange
 *	ownership list.  Optionally (if both is set) check the lowervp
 *	too, iff it's a ufs vp.
 *
 * RETURN VALUES:
 *      None.  Panics on error.
 *
 *--------------------------------------------------------------------------
 */
void
null_verify_pages(nvp, both)
	struct vnode *nvp;	/* IN: null-node to check */
	int both;		/* IN: check underlying ufs-vp? */
{
	struct null_node *nnp = VTONULL(nvp);
	struct vnode *uvp;
	extern int (**ufs_vnodeop_p)();
	
	if (nnp->null_name == NULL)
		return;
#if 0
	if (nnp->null_name->lock->lockcount > 1)
		return;
#endif
	/* can't reasonably check if its being changed */
	if (nnp->null_name->lock->unstable)
		return;
	if (both)
		svcm_verify_vp_page_ownership(nvp, nnp->null_name);
	svcm_verify_vp_page_ownership(nnp->null_lowervp, nnp->null_name);
	uvp = nnp->null_lowervp;
	if (uvp->v_op == (struct vnodeops *)ufs_vnodeop_p) {
		if (both)
			ufs_verify_backing_store(nvp, uvp);
		ufs_verify_backing_store(uvp, uvp);
	};
}
#endif /* JOHNH_DATA_PAGE_LIST_VERIFY */


#endif /* SVCM_TWO */
