
/*
 * Nullfs specific stuff.
 */


/*
 * These flags control some compilation options of the null layer.
 */
#define NULL_USE_DNLC
#define NULL_USE_FAST_ALLOC


/*
 * null_info structures exist one per null layer.
 * They capture information about the null layer as a whole.
 *
 * info_mountvfs points to the vfs for the null layer.
 * info_lowervfs points to the vfs of the lower-level file system.
 */
struct null_info {
	int		info_enable_dnlc;/* flag to enable name lookup caching*/
	struct vfs	*info_lowervfs;	/* lower-level vfs of mount */
	struct vfs	*info_vfs;	/* loopback vfs */
	struct vnode	*info_rootvp;	/* root null-node for this layer */
	int		info_refct;	/* # outstanding vnodes */
	int		info_mntno;	/* hack for stat's st_dev */
#ifdef I_FWL
	PFI *null_vnodeop_p;		/* ops vector w/ FWLs stacked on top */
#endif /* I_FWL */
};

#define	INHERIT_VFS_FLAG	(VFS_RDONLY|VFS_NOSUID)


#ifdef KERNEL
#define	VFSTONULLINFO(VFSP)	((struct null_info *)((VFSP)->vfs_data))
#define NULLINFOTOVFS(NINFOP)	((NINFOP)->info_vfs)
#define NULLINFOTOLOWERVFS(NINFOP) ((NINFOP)->info_lowervfs)
#define VFSTOLOWERVFS(VFSP)	(VFSTONULLINFO(VFSP)->info_lowervfs)
#endif



/*
 * Arguments to be passed to the null mount call.
 */
struct null_mountargs {
	char *source;
	int enable_dnlc;
};


